@@ -25,7 +25,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -38,135 +38,81 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/account.php');
 require_once ('../include/sane.php');
 
 utils_disable_cache ();
-extract (sane_import ('request', ['true' => 'from_brother']));
+extract (sane_import ('get', ['true' => 'from_brother']));
 
 # Logged users have no business here.
 if (user_isloggedin () && !$from_brother)
   session_redirect ("{$sys_home}my/");
 
 # Input checks.
-extract (sane_import('request',
+extract (sane_import ('request',
   [
-    'true' => ['brotherhood', 'cookie_for_a_year', 'login', 'cookie_test'],
+    'true' => ['brotherhood', 'login'],
+    'digits' => 'cookie_for_a_year',
     'name' => 'form_loginname',
     'pass' => 'form_pw',
     'internal_uri' => 'uri'
   ]
 ));
+if (empty ($cookie_for_a_year))
+  $cookie_for_a_year = 0;
 if (!$from_brother)
   form_check ('login');
 
 $stay_in_ssl = session_stay_in_ssl ();
 $uri_enc = utils_urlencode ($uri);
 
-# Check cookie support.
-if (!$from_brother && !isset ($_COOKIE["cookie_probe"]))
-  {
-    if ($cookie_test)
-      {
-        # TRANSLATORS: the first argument is a domain (like
-        # "savannah.gnu.org" vs. "savannah.nongnu.org");
-        # the second argument is a URL ("[URL label]" transforms to a link).
-        $msg =
-          sprintf (
-            _("Savane thinks your cookies are not activated for %s.\nPlease "
-              . "activate cookies in your web browser for this website\n"
-              . "and [%s try to login again]."),
-            $sys_default_domain,
-            "$sys_https_url{$sys_home}account/login.php?uri=$uri"
-          );
-        fb ($msg, 1);
-      }
-    else
-      {
-        # Attempt to set a cookie to go to a new page to see
-        # if the client will indeed send that cookie.
-        session_cookie ('cookie_probe', 1);
-        # $uri used to be not url-encoded, it caused login problems,
-        # see Savannah sr #108277.
-        header ("Location: login.php?uri=$uri_enc&cookie_test=1");
-      }
-  }
+if (!$from_brother)
+  session_check_cookies ($uri, $uri_enc);
+
+function validate_login ($from_brother, $form_loginname, $form_pw)
+{
+  global $sesion_uid, $session_hash, $cookie_for_a_year;
+  if ($from_brother)
+    extract (sane_import ('get',
+      ['digits' => 'session_uid', 'hash' => 'session_hash']
+    ));
+  if (isset ($session_uid) && session_exists ($session_uid, $session_hash))
+    {
+      session_set_new_cookies ($session_uid, $cookie_for_a_year);
+      return true;
+    }
+  return session_login_valid ($form_loginname, $form_pw, $cookie_for_a_year);
+}
+
+function arrange_session ($uri, $uri_enc)
+{
+  global $sys_home, $stay_in_ssl, $sys_https_url;
+  session_set_theme ();
+  # We return to our brother 'my', where we login originally,
+  # unless we are request to go to an uri.
+  if (!$uri)
+    {
+      $uri = "{$sys_home}my/";
+      $uri_enc = utils_urlencode ($uri);
+    }
+  session_login_brother ($uri, $uri_enc);
+  # If no brother domain is defined, just return to the page the login
+  # was requested from.
+  $url = $uri;
+  if ($stay_in_ssl)
+    $url = "$sys_https_url$url";
+  session_redirect ($url);
+}
 
 if (!empty ($login))
   {
-    if ($from_brother)
-      {
-        extract (sane_import ('get',
-          ['digits' => 'session_uid', 'xdigits' => 'session_hash']
-        ));
-      }
-
-    if (isset ($session_uid) && session_exists ($session_uid, $session_hash))
-      {
-        session_set_new_cookies ($session_uid, $cookie_for_a_year);
-        $success = 1;
-      }
-    else
-      $success =
-        session_login_valid ($form_loginname, $form_pw, $cookie_for_a_year);
+    $success = validate_login ($from_brother, $form_loginname, $form_pw);
     if ($success)
-      {
-        # Set up the theme, if the user has selected any in the user
-        # preferences -- but give priority to a cookie, if set.
-        if (!isset($_COOKIE['SV_THEME']))
-          {
-            $theme_result = user_get_result_set (user_getid ());
-            $theme = db_result ($theme_result, 0, 'theme');
-            if (strlen ($theme) > 0)
-              utils_setcookie ('SV_THEME', $theme, time () + 60 * 60 * 24);
-          }
-        # We return to our brother 'my', where we login originally,
-        # unless we are request to go to an uri.
-        if (!$uri)
-          {
-            $uri = "{$sys_home}my/";
-            $uri_enc = utils_urlencode ($uri);
-          }
-        # If a brother server exists, login there too, if we are not
-        # already coming from there.
-        if (!empty ($sys_brother_domain) && $brotherhood)
-          {
-            $root_url = session_issecure ()? "https": "http";
-            $root_url .= "://$sys_brother_domain";
-
-            if (!$from_brother)
-              {
-                # Go there saying hello to your brother.
-                header (
-                  "Location: $root_url{$sys_home}"
-                  . "account/login.php?session_uid=" . user_getid ()
-                  . "&session_hash={$session_hash}&login=1"
-                  . "&cookie_for_a_year=$cookie_for_a_year&from_brother=1"
-                  . "&stay_in_ssl=$stay_in_ssl&brotherhood=1&uri=$uri_enc"
-                );
-                exit;
-              }
-            else
-              {
-                header ("Location: $root_url$uri");
-                exit;
-              }
-          }
-        else
-          {
-            # If No brother server exists, just go to 'my' page
-            # unless we are request to go to an uri.
-            $url = $uri;
-            if ($stay_in_ssl) # Enforce HTTPS mode.
-              $url = "$sys_https_url$url";
-            header ("Location: $url");
-            exit;
-          }
-      } # $success
-  } # !empty ($login)
+      arrange_session ($uri, $uri_enc);
+  }
 
 if (isset ($session_hash))
   {
@@ -179,11 +125,11 @@ if (isset ($session_hash))
   }
 
 site_header (['title' => _("Login")]);
-if (!empty ($login) && !$success)
+if (!empty ($login) && empty ($success))
   {
     if (isset ($signal_pending_account) && $signal_pending_account == 1)
       {
-        print '<h2>' . _("Pending Account") . "</h2>\n";
+        print html_h (2, _("Pending Account"));
         print '<p>'
           . _("Your account is currently pending your email confirmation.\n"
               . "Visiting the link sent to you in this email will activate "
@@ -229,13 +175,14 @@ print form_hidden (['uri' => $uri]);
 # Shortcuts to New Account and Lost Password have a tabindex superior to
 # the rest of form, so they don't mess with the normal order when you
 # press TAB on the keyboard (login -> password -> post).
-print '<p><span class="preinput">' . _("Login Name:")
+print '<p><span class="preinput">'
+  . html_label ('form_loginname', _("Login name:"))
   . "</span><br />&nbsp;&nbsp;\n";
 print "<input type='text' name='form_loginname' value=\"$form_loginname"
   . '" tabindex="1" /> <a class="smaller" href="register.php" tabindex="2">['
   . _("No account yet?") . "]</a></p>\n";
 
-print '<p><span class="preinput">' . _("Password:")
+print '<p><span class="preinput">' . html_label ('form_pw', _("Password:"))
   . "</span><br />\n&nbsp;&nbsp;";
 print '<input type="password" name="form_pw" tabindex="1" /> '
   . '<a class="smaller" href="lostpw.php" tabindex="2">['
@@ -253,23 +200,24 @@ if (!isset ($sys_https_host))
       . "</p>\n";
   }
 
+$attr_list['label'] = '<span class="preinput">' . _("Remember me")
+  . "</span><br />\n"
+  . _("For a year, your login information will be stored in a cookie. Use\n"
+      . "this only if you are using your own computer.");
 print '<p>'
   . form_checkbox ('cookie_for_a_year', $cookie_for_a_year, $attr_list)
-  . '<span class="preinput">' . _("Remember me") . "</span><br />\n";
-print '<span class="text">'
-  . _("For a year, your login information will be stored in a cookie. Use\n"
-      . "this only if you are using your own computer.")
-  . '</span>';
+  . "</p>\n";
 
 if (!empty ($sys_brother_domain))
   {
-    print '<p>'
-      .  form_checkbox ('brotherhood', $brotherhood || !$login, $attr_list)
-      . '<span class="preinput">';
-    # TRANSLATORS: the argument is a domain (like "savannah.gnu.org"
-    # vs. "savannah.nongnu.org").
-    printf (_("Login also in %s"), $sys_brother_domain);
-    print  "</span><br />\n";
+    print '<p>';
+    print form_checkbox ('brotherhood', $brotherhood || !$login,
+      # TRANSLATORS: the argument is a domain (like "savannah.gnu.org"
+      # vs. "savannah.nongnu.org").
+      ['label' => '<span class="preinput">'
+        . sprintf (_("Login also in %s"), $sys_brother_domain) . '</span>']
+    );
+    print  "</p>\n";
   }
 print form_footer (_("Login"), 'login');
 $HTML->footer ([]);
