@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 
@@ -65,7 +65,7 @@ print '<p class="warn">'
 
 print form_header ('lostpw-confirm.php');
 print '<p><span class="preinput"> &nbsp;&nbsp;';
-print _("Login Name:") . ' &nbsp;&nbsp;</span>';
+print _("Login name:") . ' &nbsp;&nbsp;</span>';
 print '<input type="text" name="form_loginname" /> &nbsp;&nbsp;';
 print form_submit (_("Send lost password hash"), 'send') . "</p>\n";
 print "</form>\n";
