@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/sane.php');
@@ -68,7 +68,7 @@ $hdr = [
     . "account/su.php?action=logout&from_brother=1&uri="
     . utils_urlencode ($uri)
 ];
-$prot = su_getprotocol ();
+$prot = session_protocol ();
 foreach ($hdr as $k => $v)
   $hdr[$k] = "Location: $prot://$v";
 $hdr[0] = "Location: $uri";
@@ -100,11 +100,4 @@ else
   exit_error (_("You shouldn't have come to this page."));
 
 header ($hdr[$i]);
-
-function su_getprotocol ()
-{
-  if (session_issecure ())
-    return "https";
-  return "http";
-}
 ?>
