@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,13 +41,11 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 require_once ('../../include/trackers/general.php');
 
-$is_admin_page = 'y';
-
 extract (sane_import ('request',
   [
     'strings' => [['func', ['deltransition', 'delcanned']]],
@@ -235,14 +233,13 @@ $td_select_box = function ($field)
     && trackers_data_is_select_box ($field);
 };
 
-function print_predefined_val_entry ($fld_val)
+function print_predefined_val_entry ($fld_val, &$defs)
 {
   extract ($fld_val);
   # Non-active value are not important here.
   if ($status != "A")
     return;
-  print "<dt><b>$value</b> ($order_id)</dt>\n";
-  print "<dd>$description</dd>\n";
+  $defs["<b>$value</b> ($order_id)"] = $description;
 }
 
 function list_predefined_values ($field)
@@ -258,10 +255,10 @@ function list_predefined_values ($field)
       fb ($msg, 1);
       return;
     }
-  print "<dl>\n";
+  $defs = [];
   while ($fld_val = db_fetch_array ($res))
-    print_predefined_val_entry ($fld_val);
-  print "</dl>\n";
+    print_predefined_val_entry ($fld_val, $defs);
+  print html_dl ($defs);
 }
 
 if ($list_value)
@@ -340,7 +337,7 @@ if ($list_value)
                 if (!$is_group_scope)
                   $html .= "<td>$value_id</td>\n";
 
-                # The permanent values cant be modified (No link).
+                # The permanent values can't be modified (No link).
                 $txt_val = $value;
                 if ($status != 'P')
                   $txt_val = "<a href=\"$php_self?update_value=1"
@@ -465,11 +462,10 @@ if ($list_value)
         # Get all the value_id - value pairs.
         $res_value = db_execute ($sql, [$group_id, $field_id]);
 
-        $rows = db_numrows ($res_value);
-        if ($rows <= 0)
+        if (!db_numrows ($res_value))
           $res_value = db_execute ($sql, [100, $field_id]);
 
-        if ($rows > 0)
+        if (db_numrows ($res_value))
           {
             $val_label = [];
             while ($val_row = db_fetch_array ($res_value))
