@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/group.php');
@@ -51,21 +51,20 @@ require_once ('../include/trackers/votes.php');
 if (!$group_id)
   exit_no_group ();
 
-$is_trackeradmin =
-  member_check (0, $group_id, member_create_tracker_flag (ARTIFACT) . '2');
+$is_trackeradmin = member_check (0, $group_id, 2);
 
-function extract_revert_order ()
+function extract_reverse_order ()
 {
-  global $revert_comment_order, $order_reverted;
-  $revert_comment_order = false;
-  $revert = sane_import ('request', ['true' => ['revert_order', 'revert_bis']]);
-  $revert_comment_order = isset ($revert['revert_order']);
-  $order_reverted = isset ($revert['revert_bis']);
-  if ($order_reverted)
-    $revert_comment_order = !$revert_comment_order;
+  global $reverse_comment_order, $order_reversed;
+  $reverse_comment_order = false;
+  $reverse = sane_import ('request', ['true' => ['reverse_order', 'reverse_bis']]);
+  $reverse_comment_order = isset ($reverse['reverse_order']);
+  $order_reversed = isset ($reverse['reverse_bis']);
+  if ($order_reversed)
+    $reverse_comment_order = !$reverse_comment_order;
 }
 
-extract_revert_order ();
+extract_reverse_order ();
 
 # Mention if there was an attached file: we cannot pre-fill an HTML input file.
 function warn_about_uploads ()
@@ -202,7 +201,7 @@ if (!$func)
   $func = 'browse';
 
 $process_comment = false;
-if ($preview || isset ($quote_no) || $order_reverted)
+if ($preview || isset ($quote_no) || $order_reversed)
   $process_comment = true;
 if ($process_comment)
   $submitreturn = 1;
@@ -236,43 +235,23 @@ switch ($func)
     $fields = sane_import ('post',
       [
         'strings' => [['check', '1984']],
-        # As of 2022-02, frontend never reads from the spam_stats table,
-        # so we may safely 'pass' 'details' and 'form_id'.
         'pass' => ['form_id', 'details'], 'true' => 'submit'
       ]
     );
     if (!isset ($fields['submit']))
       $preview = true;
-    db_autoexecute (
-      'spam_stats',
-      [
-        'tracker' => ARTIFACT, 'bug_id' => 0, 'type' => 'new',
-        'user_id' => user_getid (), 'form_id' => $fields['form_id'],
-        'ip' => '127.0.0.1', 'check_value' => $fields['check'],
-        'details' => $fields['details']
-      ]
-    );
-    $stat_id = db_insertid (NULL);
 
     $anon_check_failed = false;
     if (!user_isloggedin ())
       $anon_check_failed = empty ($fields['check']);
 
     form_check ();
-
     # Get the list of bug fields used in the form.
     $vfl = trackers_extract_field_list ();
 
     $item_id = null;
     if (empty ($preview) && !$anon_check_failed)
-      {
-        # Data control layer.
-        $item_id = trackers_data_create_item ($group_id, $vfl, $address);
-        db_execute (
-          'UPDATE spam_stats SET bug_id = ? WHERE id = ?',
-         [$item_id, $stat_id]
-        );
-      }
+      $item_id = trackers_data_create_item ($group_id, $vfl, $address);
     if ($previous_form_bad_fields || !empty ($preview) || $anon_check_failed)
        warn_about_uploads ();
     if ($item_id && empty ($preview) && !$anon_check_failed)
@@ -312,15 +291,7 @@ switch ($func)
                         . "to the Carbon-Copy list."), 1);
               }
           }
-        # Send an email to notify the user of the item update
-        # (third arg of get_item_notification must be 0 for a first
-        # submission).
-        list ($additional_address, $sendall) =
-          trackers_data_get_item_notification_info ($item_id, ARTIFACT, 0);
-
-        if ((trim ($address) != "") && $sendall)
-          $address .= ", ";
-        $address .= $additional_address;
+        trackers_append_followup_notif_addresses ($address, $item_id, 0);
         trackers_mail_followup ($item_id, $address);
       }
     else # !($item_id && empty ($preview) && !$anon_check_failed)
@@ -364,50 +335,13 @@ switch ($func)
         'strings' => [['check', '1984']], 'pass' => ['comment', 'form_id']
       ]
     );
-    db_autoexecute (
-      'spam_stats',
-      [
-        'tracker' => ARTIFACT, 'bug_id' => $fields['item_id'],
-        'type' => 'comment', 'user_id' => user_getid (),
-        'form_id' => $fields['form_id'], 'ip' => '127.0.0.1',
-        'check_value' => $fields['check'], 'details' => $fields['comment']
-      ]
-    );
     form_check ();
     $anon_check_failed = false;
     if (!user_isloggedin ())
       $anon_check_failed = empty ($fields['check']) && !$process_comment;
 
-    # Filter out people that would submit data while they are not allowed
-    # too (obviously by using an old form, or something else).
-    $result = db_execute ("
-      SELECT privacy, discussion_lock, submitted_by
-      FROM " . ARTIFACT . " WHERE bug_id = ? AND group_id = ?",
-      [$item_id, $group_id]
-    );
-
-    if (db_numrows ($result) > 0)
-      {
-        # Check if the item is private, refuse post if it is and the
-        # users has no appropriate rights (not member, not submitter).
-        if (db_result ($result, 0, 'privacy') == '2')
-          {
-            if (
-              !member_check (user_getid (), $group_id)
-              && db_result ($result, 0, 'submitted_by') != user_getid ()
-            )
-              {
-                # As the user here is expected to behave maliciously,
-                # return an error message that does not give too much info.
-                exit_permission_denied ();
-              }
-          }
-        if (!$is_trackeradmin && db_result ($result, 0, 'discussion_lock'))
-          exit_permission_denied ();
-        if (!group_restrictions_check ($group_id, ARTIFACT, TRACKER_EVENT_COMMENT))
-          exit_permission_denied ();
-      }
-    elseif (!$is_trackeradmin)
+    list ($may_comment) = trackers_may_user_comment (user_getid (), $item_id);
+    if (!$may_comment)
       exit_permission_denied ();
 
     # To keep track of changes.
@@ -497,13 +431,7 @@ switch ($func)
     # Now handle notification, after all necessary actions has been.
     if ($changed)
       {
-        # Check if we re supposed to send all modifications to an address.
-        list ($additional_address, $sendall) =
-          trackers_data_get_item_notification_info ($item_id, ARTIFACT, 1);
-
-        if ($sendall && trim ($address) != "")
-          $address .= ", ";
-        $address .= $additional_address;
+        trackers_append_followup_notif_addresses ($address, $item_id);
         trackers_mail_followup ($item_id, $address, $changes);
 
         # If the assigned_to was changed and the previously assigned guy
@@ -528,14 +456,9 @@ switch ($func)
       $reassign_change_group
       || ($reassign_change_artifact && ($reassign_change_artifact != ARTIFACT))
     )
-      {
-        dbg("reassign item: reassign_change_group:$reassign_change_group, "
-            . "reassign_change_artifact:$reassign_change_artifact, ARTIFACT:"
-            . ARTIFACT);
-        trackers_data_reassign_item (
-          $item_id, $reassign_change_group, $reassign_change_artifact
-        );
-      }
+      trackers_data_reassign_item (
+        $item_id, $reassign_change_group, $reassign_change_artifact
+      );
 
     # Show browse item page, unless the user want to get back
     # to the same report, to make something else.
@@ -549,8 +472,7 @@ switch ($func)
         if (isset ($item_id))
           {
             # Include tracker item number in URL, if present.
-            header ("Location: {$_SERVER['PHP_SELF']}?$item_id");
-            exit (0);
+            session_redirect ("{$_SERVER['PHP_SELF']}?$item_id");
           }
         $_POST = $_FILES = [];
         $depends_search = $reassign_change_group_search =
@@ -564,7 +486,8 @@ switch ($func)
     # Remove an attached file.
     if ($is_trackeradmin)
       {
-        trackers_data_delete_file($group_id, $item_id, $item_file_id);
+        form_check_id (true);
+        trackers_data_delete_file ($group_id, $item_id, $item_file_id);
 
          # Unset previous settings and return to the item.
          $depends_search = $reassign_change_group_search = $add_cc
@@ -593,13 +516,8 @@ switch ($func)
     );
     if ($changed)
       {
-        # See if we are supposed to send all modifications to an address.
-        list ($additional_address, $sendall) =
-          trackers_data_get_item_notification_info ($item_id, ARTIFACT, 1);
-        if ($sendall && trim ($address) != "")
-          $address .= ", ";
-        $address .= $additional_address;
-        trackers_mail_followup($item_id, $address, $changes);
+        trackers_append_followup_notif_addresses ($address, $item_id);
+        trackers_mail_followup ($item_id, $address, $changes);
       }
 
     # Unset previous settings and return to the item.
@@ -638,7 +556,7 @@ switch ($func)
     break;
 
   case 'unflagspam':
-    # Unflag an alledged spam: for group admins only.
+    # Unflag an alleged spam: for group admins only.
     if (!member_check (0, $group_id, 'A'))
       {
         # Do not use exit_not_logged_in(), because the user has no
