@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $includes = [
   'init', 'gpg', 'member', 'trackers/general', 'trackers/format',
@@ -53,23 +53,11 @@ extract (sane_import ('get', ['name' => 'user']));
 
 $tracker = ARTIFACT;
 
-if (empty ($item_id))
-  exit_missing_param (['item_id']);
+exit_if_missing ('item_id');
 
-$fields = ['group_id', 'privacy'];
-$field_list = join (', ', $fields);
-
-$result = db_execute (
-  "SELECT $field_list FROM $tracker WHERE bug_id = ? AND spamscore < ?",
-  [$item_id, 5]
-);
-
-if (db_numrows ($result) < 1)
-  exit_error (_("Item not found."));
-
-$arr = db_fetch_array ($result);
-foreach ($fields as $k)
-  $$k = $arr[$k];
+extract (utils_find_item ($tracker, $item_id, ['privacy', 'spamscore']));
+if ($spamscore >= 5)
+  exit_error (sprintf (_("Item #%s not found."), $item_id));
 
 $group = project_get_object ($group_id);
 if ($group->isError ())
@@ -80,16 +68,10 @@ $ctype = "text/plain";
 $fname = "$item_id.txt";
 if ($data_are_private)
   {
-    if (empty ($user))
-      exit_missing_param (['user']);
-
-    $result = db_execute (
-      "SELECT user_id FROM user WHERE user_name = ?", [$user]
-    );
-
-    if (db_numrows ($result) < 1)
-      exit_error (_("User not found."));
-    $user_id = db_fetch_array ($result)['user_id'];
+    exit_if_missing ('user');
+    $user_id = user_getid ($user);
+    if (empty ($user_id))
+      exit_error (markup_rich (sprintf (_("User *%s* not found."), $user)));
 
     if ($privacy == '2' && !member_check_private ($user_id, $group_id))
       exit_permission_denied ();
