@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,11 +39,10 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-require('include/init.php');
-
-$HTML->header(array('title'=>_("Contact Us")));
-utils_get_content("contact");
-$HTML->footer(array());
+require ('include/init.php');
+$HTML->header (['title' => _("Contact Us")]);
+utils_get_content ("contact");
+$HTML->footer ([]);
 ?>
