@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 require_once ('../../include/sane.php');
@@ -65,8 +65,9 @@ for ($i = 0; $i < $n; $i++)
 
 extract (sane_import ('get', ['strings' => [['func', ['up', 'down']]]]));
 extract (sane_import ('post',
-  ['digits' => ['repo_no'], 'true' => ['submit'], 'path' => ['name']]
+  ['digits' => ['repo_no'], 'true' => ['submit']]
 ));
+extract (sane_import ('request', ['path' => ['name']]));
 form_check ('submit');
 
 function page_start ()
@@ -283,7 +284,7 @@ if (isset ($submit) && $have_name)
     )
       if (isset ($v))
         vcs_set_repo_pref ($vcs, $group_id, $k, $name, $v);
-    $repos = vcs_get_repos ($vcs, $group_id);
+    $repos = vcs_get_repos ($vcs, $group_id, true);
   }
 
 page_start ();
