@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('./include/init.php');
 require_once ('./include/sane.php');
@@ -62,10 +62,9 @@ if (!isset ($_COOKIE["cookie_probe"]))
         # Attempt to set a cookie to go to a new page to see
         # if the client will indeed send that cookie.
         session_cookie ('cookie_probe', 1);
-        header ('Location: i18n.php?lang_uri='
+        session_redirect ('i18n.php?lang_uri='
           . utils_urlencode ($lang_uri) . '&cookie_test=1'
         );
-        exit;
       }
     # TRANSLATORS: the first argument is a domain (like "savannah.gnu.org"
     # vs. "savannah.nongnu.org"); the second argument
