@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,12 +39,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/utils.php');
 require_once (dirname (__FILE__) . '/pwqcheck.php');
 require_once (dirname (__FILE__) . '/random-bytes.php');
 
+define ('HASH_NEW_ACCOUNT', 'new');
+define ('HASH_LOSTPW', 'lostpw');
+
 # Return a string explaining current pwcheck requirements.
 function expand_pwqcheck_options ()
 {
@@ -181,26 +184,57 @@ function account_pwvalid ($newpass, $oldpass = '', $user = '')
   return 0;
 }
 
-function account_realname_valid ($name)
+function account_name_too_short ($name, $min_length)
+{
+  if (strlen ($name) >= $min_length)
+    return 0;
+  fb (sprintf (_("The name %s is too short"), $name), 1);
+  $msg = ngettext (
+    "It must be at least %s character.", "It must be at least %s characters.",
+    $min_length
+  );
+  fb (sprintf ($msg, $min_length), 1);
+  return 1;
+}
+
+function account_name_too_long ($name, $max_length)
+{
+  if (strlen ($name) <= $max_length)
+    return 0;
+  fb (sprintf (_("The name %s is too long"), $name), 1);
+  $msg = ngettext (
+    "It must be at most %s character.", "It must be at most %s characters.",
+    $max_length
+  );
+  fb (sprintf ($msg, $max_length), 1);
+  return 1;
+}
+
+function account_name_length_invalid ($name, $min_length, $max_length)
+{
+  if (account_name_too_long ($name, $max_length))
+    return 1;
+  return account_name_too_short ($name, $min_length);
+}
+
+function account_realname_valid ($name, $max_realname_length = 32)
 {
   if ($name === null)
     return 0;
-  utils_get_content ("forbidden_realnames");
-  if (empty ($GLOBALS['forbid_realname_regexp']))
-    return 1;
-  return !preg_match ($GLOBALS['forbid_realname_regexp'], $name);
+  return !account_name_length_invalid ($name, -1, $max_realname_length);
 }
 
 function account_sanitize_realname ($name)
 {
   if (!is_string ($name))
     return '';
-  return strtr ($name, "'\",<", "    ");
+  return trim (strtr ($name, "<", " "));
 }
 
-function account_namevalid ($name, $allow_dashes=0, $allow_underscores=1,
-                            $allow_dots=0, $MAX_ACCNAME_LENGTH=16,
-                            $MIN_ACCNAME_LENGTH=3)
+function account_namevalid (
+  $name, $allow_dashes = 0, $allow_underscores = 1, $allow_dots = 0,
+  $MAX_NAME_LENGTH = 16, $MIN_NAME_LENGTH = 3
+)
 {
   $underscore = '';
   $dashe = '';
@@ -229,39 +263,8 @@ function account_namevalid ($name, $allow_dashes=0, $allow_underscores=1,
       fb (_("There cannot be any spaces in the name"), 1);
       return 0;
     }
-
-  # Min and max length.
-  if (strlen ($name) < $MIN_ACCNAME_LENGTH)
-    {
-      fb (_("The name is too short"), 1);
-      $msg =
-        sprintf (
-          ngettext (
-            "It must be at least %s character.",
-            "It must be at least %s characters.",
-            $MIN_ACCNAME_LENGTH
-          ),
-          $MIN_ACCNAME_LENGTH
-        );
-      fb ($msg, 1);
-      return 0;
-    }
-
-  if (strlen ($name) > $MAX_ACCNAME_LENGTH)
-    {
-      fb (_("The name is too long"), 1);
-      $msg =
-        sprintf (
-          ngettext (
-            "It must be at most %s character.",
-            "It must be at most %s characters.",
-            $MAX_ACCNAME_LENGTH
-          ),
-          $MAX_ACCNAME_LENGTH
-        );
-      fb ($msg, 1);
-      return 0;
-    }
+  if (account_name_length_invalid ($name, $MIN_NAME_LENGTH, $MAX_NAME_LENGTH))
+    return 0;
 
   $alphabet = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
   if (strspn ($name, $alphabet) == 0)
@@ -431,26 +434,20 @@ function account_groupnamevalid ($name)
   # Forbidden names: check the hardcoded list unless the variable
   # $only_specific_forbid_group_regexp is true.
   if (!$GLOBALS['only_specific_forbid_group_regexp'])
-    {
-      dbg ("apply standard regexp");
-      if (preg_match ($forbid_group_regexp, $name))
-        {
-          fb (_("This group name is not allowed."), 1);
-          return 0;
-        }
-    }
+    if (preg_match ($forbid_group_regexp, $name))
+      {
+        fb (_("This group name is not allowed."), 1);
+        return 0;
+      }
 
   # Forbidden names: check the site-specific list if a list is given
   # (by consequence, the variable return true).
   if ($GLOBALS['specific_forbid_group_regexp'])
-    {
-      dbg ("apply specific regexp");
-      if (preg_match ($GLOBALS['specific_forbid_group_regexp'], $name))
-        {
-          fb (_("This group name is not allowed."), 1);
-          return 0;
-        }
-    }
+    if (preg_match ($GLOBALS['specific_forbid_group_regexp'], $name))
+      {
+        fb (_("This group name is not allowed."), 1);
+        return 0;
+      }
 
   if (strpos ($name, "_") === FALSE)
     return 1;
@@ -472,22 +469,97 @@ function account_genunixpw ($plainpw)
   return account_encryptpw ($plainpw);
 }
 
-function account_encryptpw ($plainpw)
+function account_get_pw_rounds ($use_few_rounds = false)
+{
+  global $sys_pw_rounds;
+  if ($use_few_rounds)
+    # When storing a random hash as opposed to a passphrase, the number
+    # of rounds doesn't matter because the search space is guaranteed to be
+    # large.  Use the minimum round number in such cases.
+    return 1000;
+  if (empty ($sys_pw_rounds))
+    return 5000;
+  return $sys_pw_rounds;
+}
+
+function account_encryptpw ($plainpw, $use_few_rounds = false)
 {
   $salt = account_gensalt (16);
   # rounds=5000 is the 2010 glibc default, possibly we'll upgrade in
   # the future, better have this explicit.
   # Cf. http://www.akkadia.org/drepper/sha-crypt.html
-  $pfx = '$6$rounds=5000$';
+  $pfx = '$6$rounds=' . account_get_pw_rounds ($use_few_rounds) . '$';
   return crypt ($plainpw, "$pfx$salt");
 }
 
-# Return next userid.
-function account_nextuid ()
+function account_set_pw ($user_id, $plainpw, $more_fields = [])
 {
-  db_query ("SELECT max(unix_uid) AS maxid FROM user");
-  $row = db_fetch_array ();
-  return ($row[maxid] + 1);
+  $params = $more_fields;
+  $params['user_pw'] = account_encryptpw ($plainpw);
+  return db_autoexecute (
+    'user', $params, DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
+  );
+}
+
+function account_upgrade_pw ($stored_pw, $plainpw, $user_id)
+{
+  if (!preg_match ('/[$]rounds=(\d+)[$]/', $stored_pw, $matches))
+    return;
+  if (account_get_pw_rounds () <= $matches[1])
+    return;
+  account_set_pw ($user_id, $plainpw);
+}
+
+function account_get_random_byte ()
+{
+  if (!defined ('TESTING_ACCOUNT'))
+    return ord (random_bytes (1));
+  # Return deterministic numbers when testing.
+  static $init_needed = true;
+  if ($init_needed)
+    mt_srand (0);
+  $init_needed = false;
+  return mt_rand (0, 255);
+}
+
+function account_gen_random_order ($len)
+{
+  $ret = [];
+  for ($i = 0; $i < $len; $i++)
+    $ret[] = $i;
+  for ($i = $j = 0; $i < $len - 1; $i++)
+    {
+      if (!$j)
+        {
+          for ($s = $k = 0; $k < 4; $k++)
+            $s = $s * 256 + account_get_random_byte ();
+          mt_srand ((int)$s);
+          $j = 63;
+        }
+      $r = mt_rand ($i, $len - 1);
+      $t = $ret[$i];
+      $ret[$i] = $ret[$r];
+      $ret[$r] = $t;
+      $j--;
+    }
+  return $ret;
+}
+
+function account_compare_hash ($h0, $h)
+{
+  # Try to run in input-independent time; randomize the order
+  # the characters are compared in (hopefully PHP optimizing procedures
+  # won't be able to figure out that the result is just string comparison).
+  $n = strlen ($h0);
+  $order = account_gen_random_order ($n);
+  $weights = [false => 2, true => 1];
+  $ret = $n * 2;
+  for ($i = 0; $i < $n; $i++)
+    {
+      $j = $order[$i];
+      $ret -= $weights[substr ($h0, $j, 1) === substr ($h, $j, 1)];
+    }
+  return $ret == $n;
 }
 
 function account_validpw ($stored_pw, $plain_pw)
@@ -496,7 +568,7 @@ function account_validpw ($stored_pw, $plain_pw)
     return false;
   if (strlen ($stored_pw) < 2) # Disabled account, for sure.
     return false;
-  return crypt ($plain_pw, $stored_pw) == $stored_pw;
+  return account_compare_hash (crypt ($plain_pw, $stored_pw), $stored_pw);
 }
 
 function account_key_separator ()
@@ -506,7 +578,9 @@ function account_key_separator ()
 
 function account_filter_empty_keys ($keys)
 {
-  return array_filter ($keys, function ($x) { return !empty (trim ($x)); });
+  return array_filter (
+    $keys, function ($x) { $y = trim ($x); return !empty ($y); }
+  );
 }
 
 function account_get_authorized_keys ($uid = 0)
@@ -554,11 +628,65 @@ function account_new_keys_alert ($user_id)
       . "If it wasn't you, maybe someone is trying to compromise your "
       . "account..."), $sys_name
   );
-  # TRANSLATORS: the argument is site name (like Savannah).
-  $message .= "\n" . sprintf (_("-- the %s team."), $sys_name) . "\n";
+  $message .= "\n" . utils_team_signature ();
   sendmail_mail (
     ['to' => user_get_email ($user_id)],
     ['subject' => $subject, 'body' => $message]
   );
 }
+
+function account_clear_confirm_hash ($name)
+{
+  $uid = user_getid ($name);
+  if (empty ($uid))
+    return false;
+  return db_autoexecute (
+    'user', ['confirm_hash' => null, 'email_new' => null], DB_AUTOQUERY_UPDATE,
+    'user_id = ?', [$uid]
+  );
+}
+
+define ('CONFIRM_HASH_SEPARATOR', '!');
+
+# Exit if $confirm_hash and $item doesn't match the $uid's data from `user`.
+function account_validate_confirm_hash ($confirm_hash, $item, $uid = 0)
+{
+  if (empty ($confirm_hash))
+    $confirm_hash = '';
+  $confirm_field = user_get_field ($uid, 'confirm_hash');
+  if (empty ($confirm_field))
+    exit_error (_("Invalid confirmation hash."));
+  $ch = explode (CONFIRM_HASH_SEPARATOR, $confirm_field);
+  if (count ($ch) <= 2)
+    exit_error (_("Invalid confirmation hash."));
+  while (count ($ch) > 2)
+    array_shift ($ch); # Drop the hash creation date.
+  if ($ch[0] != $item)
+    exit_error (_("Invalid confirmation hash."));
+  if (!account_validpw ($ch[1], $confirm_hash))
+    exit_error (_("Invalid confirmation hash."));
+}
+
+# The hash is stored in the database hashed: if it weren't, an attacker
+# with a read access to the database would be able e.g. to set passwords
+# for any accounts.
+function account_generate_confirm_hash ($item, $params = [], $user_id = 0)
+{
+  if (!$user_id)
+    $user_id = user_getid ();
+  $s = CONFIRM_HASH_SEPARATOR;
+  $confirm_hash = random_hash ();
+  $hash_enc = account_encryptpw ($confirm_hash, true);
+  $params['confirm_hash'] = time () . "$s$item$s$hash_enc";
+  $success = db_autoexecute ('user', $params,
+    DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
+  );
+  if (!$success)
+    {
+      fb (_("Failed to update the database."), 1);
+      return null;
+    }
+  fb (_("Database updated."));
+  return $confirm_hash;
+}
 ?>
