@@ -29,7 +29,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -42,7 +42,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 define ('DB_AUTOQUERY_INSERT', 1);
 define ('DB_AUTOQUERY_UPDATE', 2);
@@ -64,12 +64,30 @@ function db_check_mysqli ()
   exit;
 }
 
-function db_connect ()
+function db_charset_name ()
+{
+  global $mysql_conn;
+  return mysqli_character_set_name ($mysql_conn);
+}
+
+function db_set_charset ($charset)
+{
+  global $sys_dbcharset, $mysql_conn;
+  if (empty ($sys_dbcharset))
+    $sys_dbcharset = 'utf8mb4';
+  if (empty ($charset))
+    $charset = $sys_dbcharset;
+  return mysqli_set_charset ($mysql_conn, $charset);
+}
+
+function db_connect ($charset = null)
 {
   global $sys_dbhost, $sys_dbport, $sys_dbuser, $sys_dbpasswd, $sys_dbname;
-  global $sys_dbsocket, $mysql_conn;
+  global $sys_dbsocket, $mysql_conn, $db_queries_filed;
 
   $mysql_conn = null;
+  if (empty ($db_queries_filed))
+    $db_queries_filed = [];
   db_check_mysqli ();
 
   mysqli_report (MYSQLI_REPORT_ERROR);
@@ -78,11 +96,19 @@ function db_connect ()
   if (!$conn)
     return
       "<p>Failed to connect to database: " . mysqli_connect_error () . "</p>\n";
-  mysqli_set_charset ($conn, 'utf8');
   $mysql_conn = $conn;
+  db_set_charset ($charset);
   return null;
 }
 
+function db_reconnect ($charset)
+{
+  global $mysql_conn;
+  if ($mysql_conn)
+    mysqli_close ($mysql_conn);
+  db_connect ($charset);
+}
+
 function db_real_escape_string ($string)
 {
   return mysqli_real_escape_string ($GLOBALS['mysql_conn'], $string);
@@ -231,8 +257,8 @@ function db_autoexecute ($table, $dict, $mode = DB_AUTOQUERY_INSERT,
       $fields = join (', ', $fields);
       $question_marks = utils_placeholders ($dict);
       return db_execute ("
-        INSERT INTO $tables_string ($fields) VALUES ($question_marks)",
-        array_values ($dict)
+        INSERT INTO $tables_string ($fields)
+        VALUES ($question_marks)", array_values ($dict)
       );
       break;
     case 'UPDATE':
@@ -245,9 +271,9 @@ function db_autoexecute ($table, $dict, $mode = DB_AUTOQUERY_INSERT,
         }
       $sql_fields = join (', ', $sql_fields);
       $values = array_merge ($values, $where_inputarr);
-      $where_sql = $where_condition? "WHERE $where_condition": '';
+      $where_sql = $where_condition? "\nWHERE $where_condition": '';
       return db_execute (
-        "UPDATE $tables_string SET $sql_fields $where_sql", $values
+        "UPDATE $tables_string SET $sql_fields$where_sql", $values
       );
       break;
     default:
@@ -271,7 +297,7 @@ function db_execute ($sql, $inputarr = null, $multi_query = 0)
   if (empty ($GLOBALS['mysql_conn']))
     return null;
   $expanded_sql = db_variable_binding ($sql, $inputarr);
-  return db_query ($expanded_sql, 0, $multi_query);
+  return db_query ($expanded_sql, $multi_query);
 }
 
 function db_query_prevent_die ($disable = null)
@@ -296,88 +322,57 @@ function db_query_die ($qstring, $errors = null)
   return false;
 }
 
-function db_query ($qstring, $print = 0, $multi_query = 0)
+function db_multi_query ($qstring)
 {
   global $mysql_conn, $db_qhandle;
-
-  # Store query for recap display.
-  if ($GLOBALS['sys_debug_on'])
+  mysqli_multi_query ($mysql_conn, $qstring);
+  $db_qhandle = $errors = [];
+  $i = 0;
+  while (true)
     {
-      $GLOBALS['debug_query_count']++;
-      $backtrace = debug_backtrace();
-      $outside = null;
-      foreach ($backtrace as $step)
-        {
-          if ($step['file'] != __FILE__)
-            {
-              $outside = $step;
-              break;
-            }
-        }
-      # Strip installation prefix.
-      $relative_path = str_replace (
-        $GLOBALS['sys_www_topdir'] . '/', '', $outside['file']
-      );
-      $location = "$relative_path:{$outside['line']}";
-      array_push ($GLOBALS['debug_queries'], [$qstring, $location]);
+      $res = mysqli_store_result ($mysql_conn);
+      if (!$res && mysqli_errno ($mysql_conn))
+        $errors[$i] = db_error ();
+      $db_qhandle[$i++] = $res;
+      if (!mysqli_more_results ($mysql_conn))
+        break;
+      mysqli_next_result ($mysql_conn);
     }
+  if (count ($errors))
+    return db_query_die ($qstring, $errors);
+  return $db_qhandle;
+}
 
-  if ($GLOBALS['sys_debug_sqlprofiler'] && extension_loaded('XCache'))
-    {
-      $backtrace = debug_backtrace ();
-      $outside = null;
-      foreach ($backtrace as $step)
-        {
-          if ($step['file'] != __FILE__)
-            {
-              $outside = $step;
-              break;
-            }
-        }
-      # Strip installation prefix.
-      $relative_path = str_replace (
-        $GLOBALS['sys_www_topdir'] . '/', '', $outside['file']
-      );
-      $location = "$relative_path:{$outside['line']}";
-      xcache_inc ($location);
-    }
-
-  if ($print)
-    {
-      print "<pre>[";
-      print_r ($qstring);
-      print "]</pre>";
-    }
+function db_format_backtrace ()
+{
+  return error_format_backtrace (false);
+}
 
+function db_query_ ($qstring, $multi_query)
+{
+  global $mysql_conn, $db_qhandle;
   if ($multi_query)
-    {
-      mysqli_multi_query ($mysql_conn, $qstring);
-      $db_qhandle = [];
-      $i = 0;
-      $fail = false;
-      while (true)
-        {
-          $res = mysqli_store_result ($mysql_conn);
-          if (!$res && mysqli_errno ($mysql_conn))
-            {
-              $fail = true;
-              $errors[$i] = db_error ();
-            }
-          $db_qhandle[$i++] = $res;
-          if (!mysqli_more_results ($mysql_conn))
-            break;
-          mysqli_next_result ($mysql_conn);
-        }
-      if ($fail)
-        return db_query_die ($qstring, $errors);
-    }
-  else
-    $db_qhandle = mysqli_query ($mysql_conn, $qstring);
+    return db_multi_query ($qstring);
+  $db_qhandle = mysqli_query ($mysql_conn, $qstring);
   if ($db_qhandle)
     return $db_qhandle;
   return db_query_die ($qstring);
 }
 
+function db_query ($qstring, $multi_query = 0)
+{
+  global $sys_debug_footer, $db_queries_filed;
+  if (!empty ($sys_debug_footer))
+    $t = error_timestamp ();
+  $ret = db_query_ ($qstring, $multi_query);
+  if (empty ($sys_debug_footer))
+    return $ret;
+  $db_queries_filed[] = [
+    $qstring, $multi_query, db_format_backtrace (), error_timestamp () - $t
+  ];
+  return $ret;
+}
+
 function db_numrows ($qhandle)
 {
   if (!$qhandle)
