@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,39 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+
+if (function_exists ('hrtime'))
+  {
+    function error_timestamp ()
+    {
+      return hrtime (true) / 1000000;
+    }
+  }
+else
+  {
+    function error_timestamp ()
+    {
+      return microtime (true) * 1000;
+    }
+  }
+
+function error_test_timestamp ($delay = 5000)
+{
+  $t = error_timestamp ();
+  usleep ($delay);
+  $t -= error_timestamp ();
+  $t *= -1000;
+  $dt = abs ($t - $delay);
+  if ($dt / $delay < .1)
+    return 'OK';
+  if ($delay < 30000)
+    # No success; give it a chance with a lower resolution.
+    return error_test_timestamp ($delay * 4);
+  return "<strong>Timestamp mismatch: $t vs $delay</strong>";
+}
+
+$TIMESTAMP_START = error_timestamp ();
 
 function error_level_name ($errno)
 {
@@ -126,12 +158,23 @@ function error_format_bt_args (&$entry)
   return '(' . join (', ', $a) .  ')';
 }
 
-function error_format_backtrace_entry ($entry)
+function error_relative_source_path ($path)
+{
+  global $sys_www_topdir;
+  static $prefix = null;
+  if ($prefix === null && isset ($sys_www_topdir))
+    $prefix = dirname ($sys_www_topdir);
+  if ($prefix === null)
+    return $path;
+  return preg_replace (",$prefix,", '$topdir', $path);
+}
+
+function error_format_backtrace_entry ($entry, $include_args)
 {
   $ret = '';
   $k = 'file';
   if (array_key_exists ($k, $entry))
-    $ret .= $entry[$k] . ':';
+    $ret .= error_relative_source_path ($entry[$k]) . ':';
   $k = 'line';
   if (array_key_exists ($k, $entry))
     $ret .= $entry[$k] . ':';
@@ -139,18 +182,19 @@ function error_format_backtrace_entry ($entry)
   if (array_key_exists ($k, $entry))
     {
       $ret .= $entry[$k];
-      $ret .= error_format_bt_args ($entry);
+      if ($include_args)
+        $ret .= error_format_bt_args ($entry);
     }
   return $ret;
 }
 
-function error_format_backtrace ()
+function error_format_backtrace ($include_args = true)
 {
   $bt = debug_backtrace ();
   array_shift ($bt); array_shift ($bt); array_shift ($bt);
   $ret = [];
   while (count ($bt))
-    $ret[] = error_format_backtrace_entry (array_pop ($bt));
+    $ret[] = error_format_backtrace_entry (array_pop ($bt), $include_args);
   return join ("\n-> ", $ret);
 }
 
@@ -226,20 +270,133 @@ function error_normalize_conf ($conf)
   return $ret;
 }
 
+function error_cc_limit_path ($override = null)
+{
+  global $sys_trackers_attachments_dir, $sys_cc_error_file;
+  if ($override !== null)
+    return $override;
+  if (!empty ($sys_cc_error_file))
+    return $sys_cc_error_file;
+  return "$sys_trackers_attachments_dir/cc-error";
+}
+
+define ('ERROR_CC_LIMIT', 17);
+define ('ERROR_CC_PERIOD', 289);
+
+function error_count_cc ($cc_error_file = null, $period = ERROR_CC_PERIOD)
+{
+  $t = time ();
+  $ret = [$t];
+  $t -= $period;
+  $lines = file (error_cc_limit_path ($cc_error_file), FILE_IGNORE_NEW_LINES);
+  if ($lines === false)
+    return $ret;
+  foreach ($lines as $l)
+    if ($l > $t)
+      $ret[] = $l;
+  return $ret;
+}
+
+function error_check_cc_limit ($cc_error_file = null, $limit = ERROR_CC_LIMIT)
+{
+  $sem = null; $ret = true;
+  $state = utils_disable_warnings ();
+  if (function_exists ('sem_get'))
+    $sem = utils_sem_acquire (__FILE__);
+  $timestamps = error_count_cc ($cc_error_file);
+  if (count ($timestamps) <= $limit)
+    {
+      $ret = false;
+      $f = fopen (error_cc_limit_path ($cc_error_file), 'w');
+      if ($f !== false)
+        {
+          fwrite ($f, join ("\n", $timestamps));
+          fclose ($f);
+        }
+    }
+  if ($sem !== null)
+    sem_release ($sem);
+  utils_restore_warnings ($state);
+  return $ret;
+}
+
+function error_test_cc_limit_path ()
+{
+  $p = error_cc_limit_path ();
+  $ret = "$p:";
+  $tests = [
+    'file_exists' => no_i18n ("file doesn't exist"),
+    'is_file' => no_i18n ("not a file"),
+    'is_readable' => no_i18n ("file isn't readable"),
+    'is_writable' => no_i18n ("file isn't writable")
+  ];
+  foreach ($tests as $f => $label)
+    if (!$f ($p))
+      return "$ret <strong>$label</strong>";
+  return "$ret OK";
+}
+
+function error_test_cc_limit_update ()
+{
+  $cc_limit = 3;
+  $ret = $res = false;
+  $state = utils_disable_warnings ();
+  $cc_file = utils_mktemp ('cc-error');
+  utils_restore_warnings ($state);
+  if ($cc_file === null)
+    return "<strong>" . no_i18n ("can't create temporary file") . "</strong>";
+  for ($i = 0; $i < $cc_limit + 2 && !$res; $i++)
+    $res = error_check_cc_limit ($cc_file, $cc_limit);
+  if ($i < $cc_limit + 1)
+    $ret = sprintf (
+      no_i18n ("limit is too low (%d vs. %d)"), $i, $cc_limit + 1
+    );
+  if (!$res)
+    $ret = no_i18n ("no limit encountered");
+  utils_rm_fr ($cc_file);
+  if ($ret !== false)
+    return "<strong>$ret</strong>";
+  return no_i18n ('OK');
+}
+
+function error_test_cc_limit ()
+{
+  $defs = [no_i18n ('timestamp file') => error_test_cc_limit_path ()];
+  $state = utils_disable_warnings ();
+  $cnt = count (error_count_cc ()) - 1;
+  utils_restore_warnings ($state);
+  $defs[no_i18n ('timestamp count')] = $cnt;
+  $defs[no_i18n ('updating timestamps')] = error_test_cc_limit_update ();
+  $ret = html_dl ($defs);
+  if (!function_exists ('sem_get'))
+    $ret .= "<p>"
+      . "<strong>Note: the sem_get function doesn't exist.  "
+      . "A race condition is possible.</strong></p>\n";
+  return $ret;
+}
+
 function error_cc_log ($location, $title, $msg)
 {
-  global $sys_cc_error;
+  global $sys_cc_error, $sys_debug_footer;
   if (empty ($sys_cc_error))
     return;
   $cc_err = $sys_cc_error;
   if (!is_array ($cc_err))
     $cc_err = [$cc_err => []];
   $location = preg_replace (',^[^:]*/frontend/,', '', $location);
+  $email_counter = 0;
   foreach ($cc_err as $addr => $conf)
     {
       $conf = error_normalize_conf ($conf);
-      if (error_title_filter ($title, $conf['exclude']))
-        error_sendmail ($addr, "{$conf['subject']} $location", $msg);
+      if (!error_title_filter ($title, $conf['exclude']))
+        continue;
+      # $sys_debug_footer is only used in debugging environments, and it
+      # adds an error report for every page served, so error rate isn't
+      # limited in that case.
+      if (!$sys_debug_footer && !$email_counter++)
+        if (error_check_cc_limit ())
+          return;
+      error_sendmail ($addr, "{$conf['subject']} $location", $msg);
     }
 }
 
