@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . "/utils.php");
 
@@ -48,23 +48,22 @@ require_once (dirname (__FILE__) . "/utils.php");
 # (set HTTP response, etc).
 function exit_error ($title = '', $text = null, $status = false)
 {
-  global $HTML, $feedback;
+  global $HTML;
 
   exit_header ($status);
   $msg = $title;
-  if ($text)
-    $msg .=
-      # TRANSLATORS: this string separates error title from further
-      # description, like _("Invalid User") . _(': ')
-      # . _("That user does not exist.")
-      _(': ') . $text;
-
-  if ($msg)
-    fb ($msg, 1);
+  if ($msg === null)
+    $msg = $text;
+  elseif ($text)
+    # TRANSLATORS: this string separates error title from further description,
+    # like _("Invalid User") . _(': ') . _("That user does not exist.")
+    $msg .= _(': ') . $text;
+  if ($title !== null)
+    $msg = html_h (1, _("Error")) . "<p>$msg</p>\n";
 
   $HTML->header (['title' => _("Exiting with Error"), 'notopmenu' => 1]);
   html_feedback_top ();
-
+  print $msg;
   $HTML->footer ([]);
   exit;
 }
@@ -88,8 +87,7 @@ function exit_not_logged_in ()
   if (!empty ($sys_https_host))
     $domain = "https://$sys_https_host";
 
-  header ("Location: $domain{$sys_home}account/login.php?uri=$uri");
-  exit;
+  session_redirect ("$domain{$sys_home}account/login.php?uri=$uri");
 }
 
 function exit_no_group ()
@@ -102,6 +100,23 @@ function exit_missing_param ($param_list = [])
   exit_error (_("Missing Parameters"), join (', ', $param_list));
 }
 
+function exit_if_missing ($param_list, $var_array = null)
+{
+  if (!is_array ($param_list))
+    $param_list = [$param_list];
+  $missing = [];
+  foreach ($param_list as $p)
+    {
+      if ($var_array === null && !empty ($GLOBALS[$p]))
+        continue;
+      if ($var_array !== null && !empty ($var_array[$p]))
+        continue;
+      $missing[] = $p;
+    }
+  if (!empty ($missing))
+    exit_missing_param ($missing);
+}
+
 # Standardize the way we log important exit on error.
 function exit_log ($message)
 {
