@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/savane_error.php');
 require_once (dirname (__FILE__) . '/utils.php');
@@ -54,18 +54,23 @@ define ('TRACKER_PERM_USER', 3);
 define ('TRACKER_PERM_MEMBER', 5);
 define ('TRACKER_PERM_NOBODY', 6);
 
-$PROJECT_OBJ = [];
+define ('GROUP_NONE', 100);
+
+define ('GROUP_STATUS_ACTIVE', 'A');
+define ('GROUP_STATUS_PENDING', 'P');
+define ('GROUP_STATUS_DELETED', 'D');
+define ('GROUP_STATUS_SPECIAL', 'X');
+
+$GROUP_STATUS_EDITABLE = [GROUP_STATUS_ACTIVE => 1, GROUP_STATUS_PENDING => 1];
 
 function group_get_object ($group_id)
 {
   # Create a common set of group objects,
   # save a little wear on the database.
-
-  global $PROJECT_OBJ;
-  $idx = "_{$group_id}_";
-  if (empty ($PROJECT_OBJ[$idx]))
-    $PROJECT_OBJ[$idx] = new Group ($group_id);
-  return $PROJECT_OBJ[$idx];
+  static $groups = [];
+  if (empty ($groups[$group_id]))
+    $groups[$group_id] = new Group ($group_id);
+  return $groups[$group_id];
 }
 
 function project_get_object ($group_id)
@@ -87,25 +92,41 @@ class Group extends savane_error
   function __construct ($id)
   {
     parent::__construct ();
+    $this->type_data_array = $this->data_array = [];
+    if ($this->fill_group_data ($id))
+      return;
+    $this->fill_type_data ();
+  }
+
+  function fill_type_data ()
+  {
+    $type = $this->data_array['type'];
+    $this->type_id = $type;
+    $this->db_type_result =
+      db_execute ("SELECT * FROM group_type WHERE type_id = ?", [$type]);
+    if (db_numrows ($this->db_type_result) >= 1)
+      $this->type_data_array = db_fetch_array ($this->db_type_result);
+  }
+
+  function fill_group_data ($id)
+  {
     $this->group_id = $id;
     $this->db_result =
       db_execute ("SELECT * FROM groups WHERE group_id = ?", [$id]);
-    $this->type_data_array = $this->data_array = [];
     if (db_numrows ($this->db_result) < 1)
       {
-        # Function in class we extended.
         $this->setError ('Group Not Found');
-        return;
+        return true;
       }
-   # Set up an associative array for use by other functions.
-   $this->data_array = db_fetch_array ($this->db_result);
-   # Find group_type information.
-   $type = $this->data_array['type'];
-   $this->type_id = $type;
-   $this->db_type_result =
-     db_execute ("SELECT * FROM group_type WHERE type_id = ?", [$type]);
-   if (db_numrows ($this->db_type_result) >= 1)
-     $this->type_data_array = db_fetch_array ($this->db_type_result);
+    $data = db_fetch_array ($this->db_result);
+    foreach (group_get_preference ($this->group_id, ['use_cookbook'])
+      as $k => $v
+    )
+      $data[$k] = $v === null? '0': $v;
+    foreach ($data as $k => $v)
+      if (!is_int ($k))
+        $this->data_array[$k] = $v;
+    return false;
   }
 
   # Return database result handle for direct access.
@@ -171,6 +192,8 @@ class Group extends savane_error
 
   function CanUse ($artifact)
   {
+    if ($artifact == 'cookbook')
+      return true;
     # Tolerate "bugs" to say "bug" and "mail" to "mailing_list".
     if ($artifact == "bugs")
       $artifact = "bug";
@@ -322,12 +345,12 @@ class Group extends savane_error
   {
     if (isset($this->data_array['group_name']))
       return $this->data_array['group_name'];
-    # TRANSLATORS: this is a placeholder for the name of a deleted group;
-    # the argument is group id (a number).
-    return sprintf (_("[Deleted group #%s]"), $this->group_id);
+    # TRANSLATORS: this is a placeholder for the name of a group that doesn't
+    # exist; the argument is group id (a number).
+    return sprintf (_("[Nonexistent group #%d]"), intval ($this->group_id));
   }
 
-  fuNction getName ()
+  function getName ()
   {
     return $this->getPublicName ();
   }
@@ -354,32 +377,39 @@ class Group extends savane_error
       return 0;
     return $this->data_array['devel_status'];
   }
-  function getGPGKeyring ()
+  function formatGPGKeyring ($users, $keys)
   {
-    $keyring = "";
-    $res =
-      db_execute ("
-        SELECT user_name, realname, gpg_key
-        FROM user
-        JOIN user_group ON user.user_id = user_group.user_id
-        WHERE admin_flags <> 'P' AND status = 'A' AND group_id = ?",
-        [$this->group_id]
-      );
-    $rows = db_numrows ($res);
-    for ($j = 0; $j < $rows; $j++)
+    $keyring = '';
+    foreach ($users as $uid => $u)
       {
-        $key = db_result ($res, $j, 'gpg_key');
-        $user = db_result ($res, $j, 'user_name');
-        $name = db_result ($res, $j, 'realname');
-        if (!$key)
+        if (empty ($keys[$uid]))
           continue;
-        # TRANSLATORS: the first argument is the full name,
-        # the second is the login.
-        $keyring .= sprintf (_("GPG keys of %s <%s>"), $name, $user);
-        $keyring .= "\n$key\n";
+        $keyring .=
+          # TRANSLATORS: the first argument is the full name,
+          # the second is the login.
+          sprintf (_("GPG keys of %s <%s>"), $u['realname'], $u['user_name']);
+        $keyring .= "\n{$keys[$uid]}\n";
       }
     return $keyring;
   }
+  function getGPGKeyring ()
+  {
+    $res =
+      db_execute ("
+        SELECT user.user_id
+        FROM user JOIN user_group ON user.user_id = user_group.user_id
+        WHERE group_id = ? AND admin_flags <> ? AND status = ?",
+        [$this->group_id, MEMBER_FLAGS_PENDING, USER_STATUS_ACTIVE]
+      );
+    if (!db_numrows ($res))
+      return '';
+    $uids = [];
+    while ($row = db_fetch_array ($res))
+      $uids[] = $row['user_id'];
+    $users = user_get_array ($uids);
+    $keys = user_get_gpg_key ($uids);
+    return $this->formatGPGKeyring ($users, $keys);
+  }
 
   function getUrl ($artifact)
   {
@@ -512,12 +542,6 @@ class Group extends savane_error
   }
 } # class Group extends savane_error
 
-# Backward compatibiliy.
-class Project extends Group
-{
-
-}
-
 # Aliases:
 # The object stuff do not allow to easily call a function depending on
 # a variable. These aliases will just do that.
@@ -558,11 +582,10 @@ function group_getid ($group_name)
   return null;
 }
 
-function group_trimmed_array ($res, $suff)
+function group_trimmed_array ($perm, $suff)
 {
-  $a = db_fetch_array ($res);
   $ret = []; $len = strlen ($suff);
-  foreach ($a as $k => $v)
+  foreach ($perm as $k => $v)
     {
       if (is_int ($k))
         continue;
@@ -581,27 +604,36 @@ function group_null_perm ($art)
   return $ret;
 }
 
+function group_get_default_permissions ($group_id, $reload = false)
+{
+  static $cached = [];
+  if (!$reload && array_key_exists ($group_id, $cached))
+    return $cached[$group_id];
+  $res = db_execute (
+    "SELECT * FROM groups_default_permissions WHERE group_id = ?", [$group_id]
+  );
+  $row = null;
+  if (db_numrows ($res))
+    $row = db_fetch_array ($res);
+  $cached[$group_id] = $row;
+  return $row;
+}
+
 function group_get_perm_flags ($group_id, $artifact, $prefix = '')
 {
   if (empty ($artifact))
     return group_null_perm ($artifact);
-  $art = $artifact;
-  if (is_scalar ($artifact))
-    $art = [$art];
+  $art = utils_make_arg_array ($artifact);
   $suff = "_{$prefix}flags";
   foreach ($art as $a)
     if (!preg_match ('/^[a-z]+$/', $a))
-      util_die ('group_getpermissions: unvalid argument artifact');
-  $fields = join ("$suff, ", $art) . $suff;
-  $res = db_execute (
-    "SELECT $fields FROM groups_default_permissions WHERE group_id = ?",
-    [$group_id]
-  );
-  if (!db_numrows ($res))
+      util_die ('group_getpermissions: invalid argument artifact');
+  $perm = group_get_default_permissions ($group_id);
+  if ($perm === null)
     return group_null_perm ($artifact);
   if (is_scalar ($artifact))
-    return db_result ($res, 0, "$artifact$suff");
-  return group_trimmed_array ($res, $suff);
+    return $perm["$artifact$suff"];
+  return group_trimmed_array ($perm, $suff);
 }
 
 function group_getpermissions ($group_id, $artifact)
@@ -630,14 +662,11 @@ function group_getrestrictions (
   if ($flag === null)
     return group_null_perm ($artifact);
   $f = $flag;
-  if (is_scalar ($f))
-    $f = [$f];
+  $f = utils_make_arg_array ($flag);
   $ret = [];
   foreach ($f as $k => $v)
     $ret[$k] = group_flag_value ($v, $event);
-  if (is_scalar ($flag))
-    return $ret[0];
-  return $ret;
+  return utils_return_val ($flag, $ret);
 }
 
 function group_restriction_flag ($group_id, $artifact, $event)
@@ -715,6 +744,35 @@ function group_get_history ($group_id = false)
   );
 }
 
+function group_get_gidNumber ($group_id)
+{
+  $data = group_get_object ($group_id)->data_array;
+  return $data['gidNumber'];
+}
+
+function group_update_member_uidNumbers ($group_id)
+{
+  foreach (member_get_group_members ($group_id) as $m)
+    if (!empty ($GLOBALS['MEMBER_FLAGS_ACTIVE'][$m['admin_flags']]))
+      member_assign_uidNumber ($m['user_id']);
+}
+
+# Assign groups.gidNumber; update numbers cached in user_group.
+function group_assign_gidNumber ($group_id)
+{
+  if (!utils_assign_idNumber ($group_id, 'group'))
+    {
+      fb (no_i18n ("Failed to assign gidNumber"), 1);
+      return;
+    }
+  db_execute ("
+    UPDATE `user_group` u, `groups` g SET u.`cache_gidNumber` = g.`gidNumber`
+    WHERE u.`group_id` = ? AND u.`group_id` = g.`group_id`", [$group_id]
+  );
+  group_update_member_uidNumbers ($group_id);
+  member_update_file ($group_id, 'group');
+}
+
 # Handle the insertion of history for these parameters.
 function group_add_history ($field_name, $old_value, $group_id)
 {
@@ -750,7 +808,7 @@ function group_get_artifact_url ($artifact, $hostname = 1)
 }
 
 # Normalize preference names before feeding it to SQL.
-# $name is either a strig or an array listing names.
+# $name is either a string or an array listing names.
 function group_normalize_pref_name (&$name)
 {
   $norm_name = function (&$n, $idx = null, $arg = null)
@@ -760,24 +818,27 @@ function group_normalize_pref_name (&$name)
   array_walk ($name, $norm_name);
 }
 
-# Make sure that the value is an array; used in further functions.
-function group_make_array ($x)
+function group_query_preference ($group_id, $pref_arr)
 {
-  if (is_array ($x))
-    return $x;
-  return [$x];
+  $arg_list = utils_in_placeholders ($pref_arr);
+  return db_execute ("
+    SELECT preference_name,preference_value FROM group_preferences
+    WHERE group_id = ? AND preference_name $arg_list",
+    array_merge ([$group_id], $pref_arr)
+  );
 }
 
-# Return group preferences. If $preference_names is a string,
+# Return group preferences.  If $preference_names is a string,
 # return a single string or false if the preference isn't set;
 # if $preference_names is an array, return an array of $name => $value,
 # where $value is null when the prefererence isn't set.
 function group_get_preference ($group_id, $preference_names)
 {
-  $pref_names = group_make_array ($preference_names);
+  $pref_names = utils_make_arg_array ($preference_names);
+  if (empty ($pref_names))
+    return [];
   group_normalize_pref_name ($pref_names);
-  $pref_arr = [];
-  $val_arr = [];
+  $pref_arr = $val_arr = [];
   foreach ($pref_names as $name)
     {
       if (array_key_exists ($name, $val_arr))
@@ -785,24 +846,10 @@ function group_get_preference ($group_id, $preference_names)
       $pref_arr[] = $name;
       $val_arr[$name] = null;
     }
-
-  $arg_list = utils_placeholders ($pref_arr);
-  $result =
-    db_execute ("
-      SELECT preference_name,preference_value FROM group_preferences
-      WHERE group_id = ? AND preference_name IN ($arg_list)",
-      array_merge ([$group_id], $pref_arr)
-    );
-  if (!is_array ($preference_names))
-    {
-      if (db_numrows ($result))
-        return db_result ($result, 0, 'preference_value');
-     return false;
-    }
-
-  while ($row = db_fetch_array ())
+  $result = group_query_preference ($group_id, $pref_arr);
+  while ($row = db_fetch_array ($result))
     $val_arr[$row['preference_name']] = $row['preference_value'];
-  return $val_arr;
+  return utils_return_val ($preference_names, $val_arr);
 }
 
 function group_set_pref_insert_sql ($group_id, $to_insert)
@@ -847,18 +894,41 @@ function group_set_pref_array ($group_id, $prefs, $names)
   return $ret;
 }
 
-function group_set_preference ($group_id, $preference_name, $value)
+function group_set_preference_args ($name, $val)
+{
+  if ($val === null && is_array ($name))
+    {
+      $pref_names = $pref_vals = [];
+      foreach ($name as $k => $v)
+        {
+          $pref_names[] = $k;
+          $pref_vals[] = $v;
+        }
+    }
+  else
+    {
+      $pref_names = utils_make_arg_array ($name);
+      $pref_vals = utils_make_arg_array ($val);
+      if (count ($pref_names) != count ($pref_vals))
+        return null;
+    }
+  group_normalize_pref_name ($pref_names);
+  return array_combine ($pref_names, $pref_vals);
+}
+
+# $preference_name and $value can have three formats:
+# * both are scalars;
+# * both are lists;
+# * $preference_name is a key-to-value array and $value is null.
+function group_set_preference ($group_id, $preference_name, $value = null)
 {
   if (!user_ismember ($group_id, 'A'))
     return false;
 
-  $pref_names = group_make_array ($preference_name);
-  $pref_vals = group_make_array ($value);
-  if (count ($pref_names) != count ($pref_vals))
+  $prefs = group_set_preference_args ($preference_name, $value);
+  if ($prefs === null)
     return false;
-  $prefs = array_combine ($pref_names, $pref_vals);
-
-  group_normalize_pref_name ($pref_names);
+  $pref_names = array_keys ($prefs);
   $current = group_get_preference ($group_id, $pref_names);
   $to_update = $to_insert = [];
   foreach ($pref_names as $n)
