@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $dir_name = dirname (__FILE__);
 require_once ("$dir_name/sane.php");
@@ -102,7 +102,7 @@ function html_hidsubpart_set_deployed ($uniqueid, $deployed)
 
   # Try to find a deployed value that match the unique id.
   # If found, override the deployed setting (the deployed setting should be
-  # used to set a default behavior, but if in the case we explicitely
+  # used to set a default behavior, but if in the case we explicitly
   # use an array to determine what is deployed, this matters more).
   if (is_array ($is_deployed)
       && array_key_exists ($uniqueid, $is_deployed))
@@ -130,9 +130,9 @@ function html_hidsubpart_js ($deployed, $title, $uniqueid)
 function html_hidsubpart_header ($uniqueid, $title, $deployed = false)
 {
   $deployed = html_hidsubpart_set_deployed ($uniqueid, $deployed);
-  $ret = "\n<h2 id=\"$uniqueid\">\n"
-    . html_hidsubpart_js ($deployed, $title, $uniqueid)
-    . "</h2>\n\n";
+  $ret = html_h (
+    2, html_hidsubpart_js ($deployed, $title, $uniqueid), $uniqueid
+  );
   $ret .= "<div id=\"hidsubpartcontent$uniqueid\">\n";
   if ($deployed)
     return $ret;
@@ -356,6 +356,26 @@ function html_label ($for, $title)
   return "<label for='$for'>$title</label>";
 }
 
+function html_dl ($defs, $id = '')
+{
+  if (empty ($defs))
+    return '';
+  if (!empty ($id))
+    $id = " id='$id'";
+  $ret = "<dl$id>\n";
+  foreach ($defs as $k => $v)
+    {
+      $id = '';
+      if (is_array ($v))
+        {
+          $id = " id='" . $v[0] . "'";
+          $v = $v[1];
+        }
+      $ret .= "<dt$id>$k</dt>\n  <dd>$v</dd>\n";
+    }
+  return "$ret</dl>\n";
+}
+
 function html_h ($no, $title, $attr = [])
 {
   $extra = '';
@@ -410,11 +430,10 @@ function html_title_attr ($title)
   return "title=\"$title\" ";
 }
 
-# Build a select box from arrays, with the first array being the "id"
-# or value and the array being the text you want displayed.
-#
+# Build a select box from list, the value is the text displayed,
+# the index in the list is 'id' unless the fourth parameter is non-zero,
+# in which case the value is also used as 'id'.
 # The second parameter is the name you want assigned to this form element.
-#
 # The third parameter is the value of the item that should be checked.
 function html_build_select_box_from_array (
   $vals, $select_name, $checked_val = 'xzxz', $samevals = 0, $title = ""
@@ -480,7 +499,7 @@ function html_build_select_box_from_arrays (
   $return = "\n<select " . html_title_attr ($title)
     . "name=\"$select_name\"$id_attr >\n";
 
-  # We want the "Default" on item initial post, only at this momement.
+  # We want the "Default" on item initial post, only at this moment.
   if ($show_unknown)
     $return .= form_option ("!unknown!", NULL, _("Unknown"));
 
@@ -551,7 +570,7 @@ function html_build_localized_select_box (
 # The second parameter is the name you want assigned to this form element.
 # The third parameter is an array of checked values.
 # The fourth parameter is the size of this box.
-# Fifth to eigth params determine whether to show None and Any.
+# Fifth to eighth params determine whether to show None and Any.
 #
 # Ninth param determine whether to show numeric values next to
 # the menu label (default true for backward compatibility.
@@ -580,54 +599,64 @@ function html_build_multiple_select_box (
   return "$return</select>\n";
 }
 
-function html_select_permission_box ($artifact, $row, $level = "member")
+function html_permission_box_vals ($artifact, $row, $level)
 {
   $num = '';
+  $value = $row;
   if ($level == "type")
-    {
-      $value = $row;
-      $default = 0;
-    }
+    $default = null;
   elseif ($level == "group")
-    {
-      $value = $row;
-      $default = _("Group Type Default");
-    }
+    $default = _("Group Type Default");
   else
     {
       $num = $row['user_id'];
       $value = $row["{$artifact}_flags"];
       $default = _("Group Default");
     }
+  return [$num, $value, $default];
+}
 
-  print "<td align=\"center\">\n"
-    . '<select title="' . _("Roles of members")
-    . "\" name=\"{$artifact}_user_$num\">\n";
-  if ($default)
+function html_permission_box_option_labels ()
+{
+  return [
+    [9, _("None")],
+    [1, _("Technician"), 1],
+    [3, _("Manager")],
+    [2, _("Techn. & Manager"), 1],
+  ];
+}
+
+function html_permission_box_print_options ($artifact, $value, $default)
+{
+  if ($default !== null)
     {
       $sel = 'NULL';
       if ($value)
         $sel = 'value';
       print "  " . form_option ('NULL', $sel, $default);;
     }
-  $labels = [
-    [9, _("None")],
-    [1, _("Technician"), 1],
-    [3, _("Manager")],
-    [2, _("Techn. & Manager"), 1],
-  ];
-  foreach ($labels as $vl)
+  foreach (html_permission_box_option_labels () as $vl)
     {
       if ($artifact == 'news' && count ($vl) > 2)
         continue;
       print "  " . form_option ($vl[0], $value, $vl[1]);
     }
+}
+
+function html_select_permission_box ($artifact, $row, $level = "member")
+{
+  list ($num, $value, $default) =
+    html_permission_box_vals ($artifact, $row, $level);
+  print "<td align=\"center\">\n"
+    . '<select title="' . _("Roles of members")
+    . "\" name=\"{$artifact}_user_$num\">\n";
+  html_permission_box_print_options ($artifact, $value, $default);
   print "</select>\n";
   if (!$value && $level == "group")
     {
       $value = group_gettypepermissions ($GLOBALS['group_id'], $artifact);
       print "<br />\n(";
-      foreach ($labels as $vl)
+      foreach (html_permission_box_option_labels () as $vl)
         if ($value == $vl[0])
           print $vl[1];
       print ")\n";
@@ -862,7 +891,7 @@ function show_group_type_box (
   $name = 'group_type', $checked_val = 'xzxz', $show_select_one = false
 )
 {
-  $result = db_query("SELECT * FROM group_type");
+  $result = db_execute ("SELECT * FROM group_type");
   return html_build_select_box (
     $result, 'group_type', $checked_val, $show_select_one,
     "> " . _("Choose one below"), false, 'Any', false, _('Group type')
