@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $file_dir = dirname (__FILE__);
 $inc = [
@@ -79,15 +79,15 @@ $sys_mail_admin = get_current_user ();
 $sys_mail_replyto = "NO-REPLY.INVALID-ADDRESS";
 $sys_email_adress="$sys_mail_admin@$sys_mail_domain";
 $sys_gpg_name = "gpg";
+$sys_min_uidNumber = $sys_min_gidNumber = 5000;
+$sys_passwd_common_gid = 1003;
+$sys_passwd_home_dir = "$sys_pkgconfdir/user-home-dir";
+$sys_passwd_user_shell = "$sys_pkgconfdir/user-shell";
 
 # Debug variables
 # (add them in tests/minimal_configs/Makefile, possibly commented out).
-# Print debug information before exiting:
-$sys_debug_on = false;
 # Prevent redirections like sv.gnu.org -> sv.nongnu.org.
 $sys_debug_nobasehost = false;
-# Log which queries are used the most, using XCache variables.
-$sys_debug_sqlprofiler = false;
 
 # Password strength checking.
 # Do we have the pwqcheck(1) program from the passwdqc package?
@@ -96,7 +96,7 @@ $use_pwqcheck = TRUE;
 # max = 40 is overridden because some users want longer passwords.
 # min = default,24,11,8,7 is overridden for N0 passwords
 # (the passwords consisting of characters from single class)
-# because NIST Electronic Authentification Gudeline
+# because NIST Electronic Authentication Gudeline
 # (Special Publication 800-63-1, Table A.1 on page 107
 #  http://csrc.nist.gov/publications/nistpubs/800-63-1/SP-800-63-1.pdf)
 # suggests that user-chosen 7 characters long password passing extensive
@@ -126,29 +126,13 @@ if (empty ($sys_dbport))
 if (empty ($sys_dbsocket))
   $sys_dbsocket = null;
 
-# Detect where we are, unless it's explicitely specified in the
-# configuration file:
-if (empty ($sys_www_topdir))
-  {
-    $sys_www_topdir = getcwd ();
-    $sys_url_topdir = dirname ($_SERVER['SCRIPT_NAME']);
-    while ($sys_www_topdir != '/' && !file_exists ("$sys_www_topdir/.topdir"))
-      {
-        $sys_www_topdir = dirname ($sys_www_topdir);
-        $sys_url_topdir = dirname ($sys_url_topdir);
-      }
-    if (!file_exists ("$sys_www_topdir/.topdir"))
-      die ("Could not find Savane's top directory (missing .topdir file)");
-  }
+init_detect_topdir ();
 
 # Add a trailing slash.
 $sys_home = $GLOBALS['sys_url_topdir'];
 if (!preg_match ('|/$|', $GLOBALS['sys_url_topdir']))
   $sys_home = $GLOBALS['sys_url_topdir'] . '/';
 
-if (empty ($sys_savane_url))
-  $sys_savane_url = '//savannah.nongnu.org/projects/administration';
-
 if (empty ($sys_savane_cgit))
   $sys_savane_cgit =
     "//git.savannah.nongnu.org/cgit/administration/savane.git";
@@ -169,59 +153,53 @@ $php_self = utils_specialchars ($_SERVER['PHP_SELF']);
 utils_set_csp_headers ();
 utils_update_decimal_separator ();
 
+# Detect where we are unless explicitly specified in the configuration file.
+function init_detect_topdir ()
+{
+  global $sys_www_topdir, $sys_url_topdir;
+  if (!empty ($sys_www_topdir))
+    return;
+  $sys_www_topdir = getcwd ();
+  $sys_url_topdir = dirname ($_SERVER['SCRIPT_NAME']);
+  $test = 'testconfig.php';
+  while ($sys_www_topdir != '/' && !file_exists ("$sys_www_topdir/$test"))
+    {
+      $sys_www_topdir = dirname ($sys_www_topdir);
+      $sys_url_topdir = dirname ($sys_url_topdir);
+    }
+  if (!file_exists ("$sys_www_topdir/$test"))
+    die ("Could not find Savane's top directory (missing $test)");
+}
+
 # Source (require) all specific include files of a module from
 # the include area (all include files of a module are arranged
 # in subdirectories in the includes area, so this routine sources
 # just all of the *.php files found in the module's subdirectory).
 function require_directory ($module)
 {
-  if ($module=="")
-    return;
   if (!empty ($GLOBALS["directory_{$module}_is_loaded"]))
     return;
-
   $dir = dirname (__FILE__) . "/$module";
-  if (is_dir ($dir))
-    {
-      $odir = opendir ($dir);
-      while ($file = readdir ($odir))
-        {
-          # Only include PHP scripts;
-          # avoid Emacs temporary files .#filename.php.
-          if (preg_match ("/^[^\.].*\.(php)$/i", $file))
-            {
-              require_once ("$dir/$file");
-            }
-        }
-      closedir ($odir);
-    }
   $GLOBALS["directory_{$module}_is_loaded"] = 1;
+  if (!is_dir ($dir))
+    return;
+  $odir = opendir ($dir);
+  while ($file = readdir ($odir))
+    # Only include PHP scripts; avoid Emacs temporary files .#filename.php.
+    if (preg_match ("/^[^.].*\.php$/i", $file))
+      require_once ("$dir/$file");
+  closedir ($odir);
 }
 
 $tracker_list = utils_get_tracker_list ();
 
-function get_module_include_dir (
-  $script_name, $true_artifact = 0, $true_dir = 0
-)
+function init_guess_artifact ($script_name)
 {
   global $tracker_list;
   $guess = basename (dirname ($script_name));
 
-  if (!$true_dir && $guess == "admin")
-    {
-      # Need to go deeper.
-      $guess = basename (dirname (dirname ($script_name)));
-    }
-
-  if ($true_artifact)
-    return $guess;
-  # We have some special cases:
-  #  - bugs, patch, task go in trackers
-  #  - news and forum go in news
-  if (in_array ($guess, $tracker_list))
-    $guess = 'trackers';
-  elseif ($guess == 'forum')
-    $guess = 'news';
+  if ($guess == "admin") # Need to go deeper.
+    $guess = basename (dirname (dirname ($script_name)));
   return $guess;
 }
 
@@ -290,9 +268,8 @@ extract (sane_import ('request',
     'digits' => ['group_id', 'item_id', 'forum_id']
   ]));
 
-# Define the artifact we are using.
 if (!defined ('ARTIFACT'))
-   define ('ARTIFACT', get_module_include_dir ($_SERVER['SCRIPT_NAME'], 1));
+  define ('ARTIFACT', init_guess_artifact ($_SERVER['SCRIPT_NAME']));
 
 function init_extract_item_id ($tracker_list)
 {
@@ -331,17 +308,11 @@ if (
   && in_array (ARTIFACT, $tracker_list)
 )
   {
-    $result = db_execute (
-      "SELECT group_id FROM " . ARTIFACT . " WHERE bug_id = ?", [$item_id]
-    );
-    if (db_numrows ($result))
-      $group_id = db_result ($result, 0, 'group_id');
-    else
-      exit_error (_("Item not found"));
+    $group_id = utils_find_item (ARTIFACT, $item_id)['group_id'];
 
-  # Special case: if it the item is from the system group and we are on the
-  # cookbook, we may want to pretend that an item belong a given group while
-  # it actually belongs to the system group.
+    # Special case: if it the item is from the system group and we are on the
+    # cookbook, we may want to pretend that an item belong a given group while
+    # it actually belongs to the system group.
     if (ARTIFACT == 'cookbook'
         && $group_id == $sys_group_id && isset ($comingfrom)
         && $comingfrom && ctype_digit (strval ($comingfrom)))
@@ -422,11 +393,8 @@ function init_run_redirections ($group_id)
   $type_host = $group->getTypeBaseHost ();
   if (!(strcasecmp ($_SERVER['HTTP_HOST'], $type_host) && $type_host))
     return;
-  $prot = 'http://';
-  if (session_issecure ())
-    $prot = 'https://';
-  header ("Location: $prot$type_host{$_SERVER["REQUEST_URI"]}");
-  exit (0);
+  $prot = session_protocol () . '://';
+  session_redirect ("$prot$type_host{$_SERVER["REQUEST_URI"]}");
 }
 
 function init_check_group ()
