@@ -5,11 +5,12 @@
 # Copyright (C) 2000-2006 Derek Feichtinger <derek.feichtinger--cern.ch>
 # Copyright (C) 2000-2006 Free Software Foundation, Inc.
 # Copyright (C) 2000-2006 Mathieu Roy
-# Copyright (C) 2004-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2000-2006 Stéphane Urbanovski <s.urbanovski--ac-nancy-metz.fr>
+# Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
 # Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2015, 2018, 2019, 2020, 2022, 2024 Bob Proulx <bob@proulx.com>
 #
 # This file is part of Savane.
 #
@@ -30,7 +31,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -43,7 +44,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $dir_name = dirname (__FILE__);
 require_once ("$dir_name/savane_error.php");
@@ -147,10 +148,9 @@ class Layout extends savane_error
 
   function generic_header_start ($params)
   {
-    global $G_USER, $G_SESSION, $sys_name, $savane_version;
-    global $sys_home, $stone_age_menu;
+    global $sys_name, $savane_version, $sys_home, $stone_age_menu;
 
-    $url = git_get_savane_url (git_get_commit ());
+    $url = git_get_savane_url ();
 
     # Avoid any cache by setting an expire time in the past, without
     # distinction.
@@ -167,10 +167,7 @@ class Layout extends savane_error
         # and personal area (/my)
         $ct = ["news", "support", "bugs", "task", "my", "myitems", "mygroups"];
         if (in_array ($context, $ct) || preg_match ("/^a/", $context))
-          {
-            header ("Expires: Thu, 22 Dec 1977 15:00:00 GMT");
-            dbg ("Expires is set.");
-          }
+          header ("Expires: Thu, 22 Dec 1977 15:00:00 GMT");
       }
     $title = context_title ($context,
       isset ($params['group'])? $params['group'] : ''
@@ -216,23 +213,23 @@ class Layout extends savane_error
   function generic_footer ($params)
   {
     global $savane_version;
-    $commit = git_get_commit ();
     print '<p class="footer">';
     utils_get_content ("page_footer");
     $root = realpath (dirname (__FILE__) . "/../../..");
     $page =
       preg_replace (":^$root:", '', realpath ($_SERVER['SCRIPT_FILENAME']));
-    $url = git_get_savane_url ($commit, $page);
+    $url = git_get_savane_url ($page);
     print "<br />\n" . utils_link ($url, _('Page source code'));
 
     print "</p>\n<div align='right'><p>";
     # TRANSLATORS: the argument is version of Savane (like 3.2).
     printf (_("Powered by Savane %s."), $savane_version);
-    $url = git_get_savane_url ($commit);
+    $url = git_get_savane_url ();
     print "<br />" . utils_link ($url, _("Corresponding source code"));
     print "</p></div>\n</div> <!-- class='main' -->\n";
     print "</div> <!-- class='realbody' -->\n";
     print "\n</body>\n</html>\n";
+    utils_output_debug_footer ();
   }
 
   function header ($params)
