@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 namespace {
 require_once ("init.php");
@@ -47,33 +47,6 @@ require_once ("sendmail.php");
 }
 
 namespace mm_ns {
-if (function_exists ('hrtime'))
-  {
-    function timestamp ()
-    {
-      return hrtime (true) / 1000000;
-    }
-  }
-else
-  {
-    function timestamp ()
-    {
-      return microtime (true) / 1000;
-    }
-  }
-
-function acquire_lock ()
-{
-  $tok = ftok (__FILE__, 'a');
-  if ($tok === -1)
-    return null;
-  $sem = sem_get ($tok);
-  if ($sem === false)
-    return null;
-  sem_acquire ($sem);
-  return $sem;
-}
-
 function send_request ($cmd, $args)
 {
   $in = "command=$cmd\n";
@@ -113,17 +86,21 @@ function parse_response ($lines)
   return $ret;
 }
 
+function send_request1 ($args)
+{
+  return send_request ($args[0], $args[1]);
+}
+
 function run ($cmd, $args)
 {
-  $t0 = timestamp ();
-  $lock = acquire_lock ();
-  if ($lock === null)
+  $t0 = error_timestamp ();
+  $request = utils_run_lock (__FILE__, '\mm_ns\send_request1', [$cmd, $args]);
+  if ($request === null)
     return ['error' => "Error: can't acquire semaphore",
-        'timestamp' => sprintf ("%.3f", timestamp () - $t0)
+        'timestamp' => sprintf ("%.3f", error_timestamp () - $t0)
       ];
-  list ($lines, $error) = send_request ($cmd, $args);
-  sem_release ($lock);
-  $t0 = timestamp () - $t0;
+  list ($lines, $error) = $request;
+  $t0 = error_timestamp () - $t0;
   $ret = parse_response ($lines);
   if (empty ($ret['error']))
     unset ($ret['error']);
@@ -172,7 +149,7 @@ function format_msg ($group_id, $list_name, $res)
     _("You are advised to change the password, and to avoid using a password\n"
       . "you use for important accounts, as mailman does not really provide\n"
       . "security for these list passwords.");
-  return $msg . sendmail_signature ();
+  return $msg . join (sendmail_signature ());
 }
 
 function report_results ($res, $group_id, $list_name)
