@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/utils.php');
 # Make sure the string has no newlines after translation.
@@ -655,7 +655,8 @@ function markup_match_lists ($prev, $next, &$stack)
   $m = $p > $n? $n: $p;
   if ($m < $n - 1)
     return false; # Only allow one level increase at once.
-  if (substr ($prev, 0, $m) !== substr ($next, 0, $m))
+  # In PHP 5, substr ('', 0, 0) returns false; substr ('0', 0, 0) returns ''.
+  if ($m && substr ($prev, 0, $m) !== substr ($next, 0, $m))
     return false; # List nesting doesn't match (like '0*0' and '*0*').
   if ($p > $n)
     return 'end';
@@ -801,49 +802,99 @@ function markup_substitute_underscored ($line)
   return markup_substitute_highlighted ($line, '_', 'i');
 }
 
-function markup_substitute_img_file_domain ($line)
+function markup_substitute_file_domain ($line)
 {
-  global $sys_file_domain;
+  global $sys_file_domain, $sys_home;
   if ($GLOBALS['sys_default_domain'] == $sys_file_domain)
     return $line;
-  return preg_replace (
-    '/<img src="\/file/', "<img src=\"//$sys_file_domain/file", $line
+  return str_replace (
+    ["<img src=\"{$sys_home}file", "<a href=\"{$sys_home}file"],
+    [
+      "<img src=\"//$sys_file_domain{$sys_home}file",
+      "<a href=\"//$sys_file_domain{$sys_home}file"
+    ], $line
   );
 }
 
+function markup_mark_item_publicity ($items, $res)
+{
+  $ret = $priv = [];
+  while ($row = db_fetch_array ($res))
+    $priv[$row['art']][$row['bug_id']] = [$row['privacy'], $row['group_id']];
+  foreach (array_keys ($items) as $file_id)
+    {
+      $it = $items[$file_id];
+      $ret[$file_id] = $it;
+      # As of 2024-04-11, Savannah has 134 files associated with bugs that
+      # don't exist, the latest was attached on 2005-07-14.  Other trackers
+      # have no such irregularities.  Treat those files as public, though
+      # file.php will result in exit_error ("Item #$item_id not found").
+      $ret[$file_id]['public'] = true;
+      if (empty ($priv[$it['artifact']][$it['item_id']]))
+        continue;
+      $p = $priv[$it['artifact']][$it['item_id']];
+      $ret[$file_id]['public'] = trackers_item_is_public ($p[0], $p[1]);
+    }
+  return $ret;
+}
+
+function markup_fetch_item_publicity ($items, $artifact_set)
+{
+  if (empty ($items) || empty ($artifact_set))
+    return [];
+  $sqls = $params = [];
+  foreach ($artifact_set as $art)
+    {
+      $p = [];
+      foreach ($items as $it)
+        if ($it['artifact'] == $art)
+          $p[$it['item_id']] = 1;
+      $qry = "
+        SELECT bug_id, privacy, group_id, '$art' as art FROM $art
+        WHERE bug_id " . utils_in_placeholders ($p);
+      $params = array_merge ($params, array_keys ($p));
+      $sqls[] = $qry;
+    }
+  $res = db_execute (join (' UNION ', $sqls), $params);
+  return markup_mark_item_publicity ($items, $res);
+}
+
 # Return array [ file_id => filename ] for the given list of file_ids.
 function markup_fetch_file_list ($file_ids)
 {
   if (empty ($file_ids))
     return [];
+  if (function_exists ("markup_fetch_file_list_override"))
+    return markup_fetch_file_list_override ($file_ids);
   $in_ph = utils_in_placeholders ($file_ids);
-  $result =  db_execute ("
-    SELECT file_id, filename FROM trackers_file WHERE file_id $in_ph",
-    $file_ids
+  $result =  db_execute (
+    "SELECT * FROM trackers_file WHERE file_id $in_ph", $file_ids
   );
-  $ret = [];
+  $items = $artifacts = [];
   while ($row = db_fetch_array ($result))
-    $ret[$row['file_id']] = $row['filename'];
-  return $ret;
+    {
+      $items[$row['file_id']] = $row;
+      $artifacts[$row['artifact']] = 1;
+    }
+  return markup_fetch_item_publicity ($items, array_keys ($artifacts));
 }
 
-function markup_expand_img ($line, $file_id, $file_name, $comment)
+function markup_expand_img ($line, $entry, $comment, $url)
 {
-  if (!(preg_match ('/\.(jpe?g|png)$/', strtolower ($file_name))))
-    return $line;
+  $file_id = $entry['file_id'];
   $alt = $comment;
   if (substr ($alt, 0, 1) === ' ')
     $alt = substr ($alt, 1);
   if ($alt !== '')
     $alt = 'alt="' . utils_specialchars ($alt) . '" ';
-  $file_name = utils_specialchars ($file_name);
   return preg_replace ("/\(?((files? ))#{$file_id}[^),]*((\)|, )?)/",
-    "<img src=\"/file/$file_name?file_id=$file_id\" $alt/> ", $line
+    "<img src=\"$url\" $alt/> ", $line
   );
 }
 
-# Replace references to image files with <img>.
-function markup_expand_img_files ($line)
+# Replace references to files with HTML elements: <img> for images
+# and <a> for other files.
+function markup_expand_file_links ($line)
 {
   preg_match_all ('/\(?((files? ))#(?P<file_id>\d+)'
     . '(?P<comment>[^),]*)((\)|, )?)/',
@@ -854,11 +905,17 @@ function markup_expand_img_files ($line)
     {
       if (empty ($file_list[$file_id]))
         continue;
-      $line = markup_expand_img ($line, $file_id,
-        $file_list[$file_id], $matches['comment'][$key]
+      $entry = $file_list[$file_id];
+      $comment = $matches['comment'][$key];
+      $url = format_file_url_path (
+        ['name' => $entry['filename'], 'id' => $file_id], $entry['public']
       );
+      if (preg_match ('/\.(jpe?g|png)$/', strtolower ($entry['filename'])))
+        $line = markup_expand_img ($line, $entry, $comment, $url);
+      else
+        $line = markup_expand_tracker_links ($line, 'files?', $url, $file_id);
     }
-  return markup_substitute_img_file_domain ($line);
+  return markup_substitute_file_domain ($line);
 }
 
 # Prepare usual links: prefix "www." with "$protocol_relative://"
@@ -931,23 +988,21 @@ function markup_convert_mail_links ($line, $protocols)
   );
 }
 
-function markup_expand_tracker_links ($line, $regexp, $link)
+function markup_expand_tracker_links ($line, $regexp, $link, $item_id = null)
 {
-  global $sys_home;
-  $line = markup_expand_img_files ($line);
-  # Allow only two white spaces between the string and the numeric id
-  # to avoid having too time consuming regexp. People just have to pay
-  # attention.
-
+  # $id_re and $item_no are used to adjust regexps and substitutions:
+  # they either expand all tracker links like bugs #N -> {$link}N...
+  list ($id_re, $item_no) = ['([0-9]+)', "\\3"];
+  # ...or, in case of files, only for single file: file #1234 -> $link.
+  if ($item_id !== null)
+    list ($id_re, $item_no) = ["$item_id()", $item_id];
   # Handle named links like [bug #4913 text of the link].
   $line = preg_replace (
-    "/(^|\s|\W)\[($regexp)\s{0,2}#([0-9]+)\s+(.+?)\]/i",
-    "\\1<i><a href=\"$sys_home$link\\3\">\\4</a></i>", $line
-  );
-
+    "/(^|\s|\W)\[($regexp)\s{0,2}#$id_re\s+(.+?)\]/i",
+    "\\1<i><a href=\"$link\\3\">\\4</a></i>", $line);
   # Now process "usual" links like bug #4913.
-  return preg_replace ("/(^|\s|\W)($regexp)\s{0,2}#([0-9]+)/i",
-    "\\1<i><a href=\"$sys_home$link\\3\">\\2&nbsp;#\\3</a></i>",
+  return preg_replace ("/(^|\s|\W)($regexp)\s{0,2}#$id_re\b/i",
+    "\\1<i><a href=\"$link\\3\">\\2&nbsp;#$item_no</a></i>",
     $line);
 }
 
@@ -1037,11 +1092,7 @@ function markup_tracker_list ()
     "support|sr" => "support/?",
     "tasks?" => "task/?",
     "recipes?|rcp" => "cookbook/?func=detailitem$comingfrom&amp;item_id=",
-    "patch" => "patch/?",
-    # In this case, we make the link pointing to support, it won't matter,
-    # the download page is in every tracker and does not check if the tracker
-    # is actually used.
-    "files?" => "support/download.php?file_id=",
+    "patch" => "patch/?"
   ];
 }
 
@@ -1060,6 +1111,7 @@ function markup_protocol_regex ()
 
 function markup_expand_links ($line)
 {
+  global $sys_home;
   $trackers = markup_tracker_list ();
   $artifact_regex = join ('|', array_keys ($trackers)) . '|comments?';
   list ($protocols, $protocol_relative) = markup_protocol_regex ();
@@ -1070,9 +1122,10 @@ function markup_expand_links ($line)
   $line = markup_insert_prot_rel ($line, $protocol_relative);
   $line = markup_convert_standalone_URLs ($line, $protocols);
   $line = markup_convert_mail_links ($line, $protocols);
+  $line = markup_expand_file_links ($line);
 
   foreach ($trackers as $regexp => $link)
-    $line = markup_expand_tracker_links ($line, $regexp, $link);
+    $line = markup_expand_tracker_links ($line, $regexp, "$sys_home$link");
 
   $line = markup_expand_comment_links ($line);
   $line = markup_expand_named_links ($line, $protocols);
