@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 function people_get_type_name ($type_id)
 {
@@ -66,7 +66,9 @@ function people_list_categories ()
 {
   global $php_self;
 
-  $result = db_query ("SELECT * FROM people_job_category ORDER BY category_id");
+  $result = db_execute (
+    "SELECT * FROM people_job_category ORDER BY category_id"
+  );
   $rows = db_numrows ($result);
   if ($rows < 1)
     return false;
@@ -99,7 +101,7 @@ function people_list_project_type ()
 {
   global $php_self;
 
-  $result = db_query ("
+  $result = db_execute ("
     SELECT
       group_type.type_id, group_type.name, COUNT(people_job.job_id) AS count
     FROM
@@ -140,12 +142,12 @@ function people_append_list (&$form_is_empty, $ret, $def)
 function people_show_table ()
 {
   $form_is_empty = true;
-  $return = '<h2>' . _("Category") . "</h2>\n";
+  $return = html_h (2, _("Category"));
   $return .= people_append_list (
     $form_is_empty, people_list_categories (), _("No categories found")
   );
 
-  $return .= '<h2>' . _("Group type") . "</h2>\n";
+  $return .= html_h (2, _("Group type"));
   $return .= people_append_list (
     $form_is_empty, people_list_project_type (), _("No types found")
   );
@@ -156,37 +158,50 @@ function people_show_table ()
     . "\" />\n</form>\n";
 }
 
+function people_fetch_categories ()
+{
+  $ret = [];
+  $result = db_execute (
+    "SELECT * FROM people_job_category ORDER BY category_id"
+  );
+  while ($row = db_fetch_array ($result))
+    $ret[$row['category_id']] = $row;
+  return $ret;
+}
+
+function people_fetch_job_counts ()
+{
+  $result = db_execute ("
+     SELECT category_id, count(*) AS count FROM people_job
+     WHERE status_id = 1 GROUP BY category_id"
+  );
+  $ret = [];
+  while ($row = db_fetch_array ($result))
+    $ret[] = $row;
+  return $ret;
+}
+
 # Show a list of categories.
 # Provide links to drill into a detail page that shows these categories.
 function people_show_category_list ()
 {
   $finalize = function ($r) { return "<ul class=\"boxli\">$r</ul>\n"; };
-  $sql = "SELECT * FROM people_job_category ORDER BY category_id";
-  $result = db_query ($sql);
-  $rows = db_numrows ($result);
-  $return = '';
-  if ($rows < 1)
+  $categories = people_fetch_categories ();
+  if (empty ($categories))
     return $finalize ("<li>" . _("No categories found") . "</li>");
-  for ($i = $j = 0; $i < $rows; $i++)
+  $ret = ''; $j = 0;
+  foreach (people_fetch_job_counts () as $row)
     {
-      $count_res = db_execute ("
-        SELECT count(*) AS count FROM people_job
-        WHERE category_id = ? AND status_id = 1",
-        [db_result ($result, $i, 'category_id')]
-      );
-
-      # Print only if there are results within the category.
-      if (db_result ($count_res, 0, 'count') <= 0)
+      if (!$row['count'])
         continue;
-      $j++;
-      $return .= '<li class="' . utils_altrow ($j)
+      $cat_id = $row['category_id'];
+      $ret .= '<li class="' . utils_altrow ($j++)
         . '"><span class="smaller">&nbsp;&nbsp;- <a href="'
-        . $GLOBALS['sys_home'] . 'people/?categories[]='
-        . db_result ($result, $i, 'category_id') . '">'
-        . db_result ($count_res, 0, 'count')
-        . ' ' . db_result ($result, $i, 'name') . "</a></span></li>\n";
+        . $GLOBALS['sys_home'] . 'people/?categories[]=' . $cat_id . '">'
+        . $row['count'] . ' ' . $categories[$cat_id]['name']
+        . "</a></span></li>\n";
     }
-  return $finalize ($return);
+  return $finalize ($ret);
 }
 
 function people_job_status_box ($name = 'status_id', $checked = 'xyxy')
@@ -200,7 +215,7 @@ function people_job_status_box ($name = 'status_id', $checked = 'xyxy')
     # TRANSLATORS: this string is a job status.
     _("Deleted")
   ];
-  $result = db_query ("SELECT * FROM people_job_status");
+  $result = db_execute ("SELECT * FROM people_job_status");
   return html_build_localized_select_box (
     $result, $name, $checked, true, 'None', false, 'Any', false,
     _('job status')
@@ -233,7 +248,7 @@ function people_job_category_box ($name = 'category_id', $checked = 'xyxy')
     _("Other")
   ];
 
-  $result = db_query ("SELECT * FROM people_job_category");
+  $result = db_execute ("SELECT * FROM people_job_category");
   return html_build_localized_select_box (
     $result, $name, $checked, true, 'None', false, 'Any', false,
     _('job category')
@@ -511,7 +526,7 @@ function people_skill_box ($name = 'skill_id', $checked = 'xyxy')
 {
   global $PEOPLE_SKILL;
   if (!$PEOPLE_SKILL)
-    $PEOPLE_SKILL = db_query ("SELECT * FROM people_skill ORDER BY name ASC");
+    $PEOPLE_SKILL = db_execute ("SELECT * FROM people_skill ORDER BY name");
   return html_build_select_box (
     $PEOPLE_SKILL, $name, $checked, true, 'None', false, 'Any', false, 'skills'
   );
@@ -534,7 +549,7 @@ function people_skill_level_box ($name = 'skill_level_id', $checked = 'xyxy')
     _('Expert')
   ];
   if (!$PEOPLE_SKILL_LEVEL)
-    $PEOPLE_SKILL_LEVEL = db_query ("SELECT * FROM people_skill_level");
+    $PEOPLE_SKILL_LEVEL = db_execute ("SELECT * FROM people_skill_level");
   return html_build_localized_select_box (
     $PEOPLE_SKILL_LEVEL, $name, $checked, true, 'None', false, 'Any', false,
     _('skill level')
@@ -565,7 +580,7 @@ function people_skill_year_box ($name = 'skill_year_id', $checked = 'xyxy')
     _('> 80 years')
   ];
   if (!$PEOPLE_SKILL_YEAR)
-    $PEOPLE_SKILL_YEAR = db_query ("SELECT * FROM people_skill_year");
+    $PEOPLE_SKILL_YEAR = db_execute ("SELECT * FROM people_skill_year");
   return html_build_localized_select_box (
     $PEOPLE_SKILL_YEAR, $name, $checked, true, 'None', false, 'Any', false,
     _('experience level')
