@@ -29,7 +29,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -42,14 +42,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_directory ("people");
 require_directory ("news");
 require_directory ("stats");
-require_once (dirname (__FILE__) . '/vars.php');
-require_once (dirname (__FILE__) . '/vcs.php');
-require_once (dirname (__FILE__) . '/utils.php');
+foreach (['vars', 'vcs', 'utils', 'member'] as $i)
+  require_once (dirname (__FILE__) . "/$i.php");
 
 # If we are at wrong url, redirect.
 $host = $project->getTypeBaseHost ();
@@ -58,23 +57,15 @@ if (
     && strcasecmp ($_SERVER['HTTP_HOST'], $host)
 )
   {
-    $prot = session_issecure()? 'https://': 'http://';
-    header ("Location: $prot$host{$_SERVER['PHP_SELF']}");
-    exit;
+    $prot = session_protocol () . '://';
+    session_redirect ("$prot$host{$_SERVER['PHP_SELF']}");
   }
-$project = new Project ($group_id);
+$project = group_get_object ($group_id);
 site_project_header ([]);
 
 # Members of this group (little box on the right).
-$res_admin = db_execute ("
-  SELECT
-    user.user_id AS user_id, user.user_name AS user_name,
-    user.realname AS realname
-  FROM user, user_group
-  WHERE
-    user_group.user_id = user.user_id AND user_group.group_id = ?
-    AND user_group.admin_flags = 'A' AND user_group.onduty = 1",
-  [$group_id]
+$res_admin = member_admin_flags_query (
+  $group_id, '= ? AND onduty = 1', MEMBER_FLAGS_ADMIN
 );
 print "\n<div class='indexright'>\n";
 print $HTML->box_top (_("Membership Info"), "", 1);
@@ -114,12 +105,9 @@ print_group_admins ($res_admin);
 
 # Count of members on this group.
 $membersnum = db_fetch_array (db_execute ("
-  SELECT COUNT(*) AS count
-  FROM user_group
-  WHERE
-    group_id = ? AND admin_flags <> 'P' AND admin_flags <> 'SQD'
-    AND user_group.onduty = 1",
-  [$group_id]
+  SELECT COUNT(*) AS count FROM user_group
+  WHERE group_id = ? AND admin_flags NOT IN (?, ?) AND onduty = 1",
+  [$group_id, MEMBER_FLAGS_PENDING, MEMBER_FLAGS_SQUAD]
 ));
 
 $membersnum = $membersnum['count'];
@@ -277,7 +265,7 @@ if ($sys_group_id == $group_id && member_check (0, $group_id, 'A'))
       . '&amp;category_id=1&amp;status_id=1&amp;set=custom',
       $img . _("Pending Group List")
     );
-    $reg_count = number_format (stats_getprojects_pending ());
+    $reg_count = stats_get_pending_groups ();
     print ' ';
     printf (
       ngettext (
