@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/utils.php');
 
@@ -55,8 +55,8 @@ require_once (dirname (__FILE__) . '/utils.php');
 # Unset variables that users are not allowed to set in any cases.
 unset ($feedback_html);
 
-# Fuctions to sanitize user-supplied values.
-# Return 0 when the variable was set, 1 otherwize (the caller will set
+# Functions to sanitize user-supplied values.
+# Return 0 when the variable was set, 1 otherwise (the caller will set
 # the respective item to null).
 # Arguments:
 #   $in: input array
@@ -82,7 +82,7 @@ $sane_sanitizers['specialchars'] = function ($in, &$out, $i, $arg)
 
 # The <input "image" name="submit" /> elements return
 # submit.x and submit.y which don't work when extracting
-# them; this fucntion maps them to submit_x and submit_y.
+# them; this function maps them to submit_x and submit_y.
 function sane_var_names ($values)
 {
   $out = [];
@@ -143,7 +143,7 @@ $sane_sanitizers['xdigits'] = function ($in, &$out, $i, $arg)
 
 # Account name is expected.
 # when $arg is a scalar, it's maximum name length (34 by default),
-# when it's an array, $arg['max_len'] is maxumum name length,
+# when it's an array, $arg['max_len'] is maximum name length,
 # $arg['allow_dots'] is whether the dots are allowed.
 $sane_sanitizers['name'] = function ($in, &$out, $i, $arg)
 {
@@ -252,7 +252,7 @@ $sane_sanitizers['preg'] = function ($in, &$out, $i, $arg)
 $sane_sanitizers['hash'] = function ($in, &$out, $i, $arg)
 {
   global $sane_sanitizers;
-  return $sane_sanitizers['preg'] ($in, $out, $i, '/^[a-f\d]+$/');
+  return $sane_sanitizers['preg'] ($in, $out, $i, '/^[a-f\d;]+$/');
 };
 
 $sane_sanitizers['path'] = function ($in,  &$out, $i, $arg)
@@ -344,16 +344,17 @@ $sane_sanitizers['array'] = function ($in, &$out, $i, $arg)
 
 # Functions to access user input.
 
-function &sane_input_array ($method)
+function &sane_input_array ($input)
 {
-  global $sane_test_input;
+  if (is_array ($input))
+    return $input;
   $arrays = [
     'get' => &$_GET, 'post' => &$_POST, 'cookie' => &$_COOKIE,
-    'files' => &$_FILES, 'test' => &$sane_test_input, 'request' => &$_REQUEST
+    'files' => &$_FILES, 'request' => &$_REQUEST
   ];
-  if (!array_key_exists ($method, $arrays))
-    $method = 'request';
-  return $arrays[$method];
+  if (!array_key_exists ($input, $arrays))
+    $input = 'request';
+  return $arrays[$input];
 }
 
 # Apply function $func to a number of $input items.
@@ -395,7 +396,8 @@ function sane_apply_func ($func, $input, $name, &$values)
 #           'true' => 'insert_group_name', 'hash' => 'rand_hash',
 #           'name' => ['form_full_name', 'form_unix_name']
 #         ]));
-# $method is the way the parameters are passed ('get', 'post', 'cookie'...)
+# $input is the way the parameters are passed ('get', 'post', 'cookie'...)
+# or the input array, directly.
 # $name describes how the parameters are filtered: the keys are indices
 # of functions from $sane_sanitizers[] to apply, the values define the names
 # of parameters to import and additional arguments for the function to use.
@@ -424,10 +426,10 @@ function sane_apply_func ($func, $input, $name, &$values)
 #   ],
 # ]
 # For more examples, see testing/sane.php.
-function sane_import ($method, $names)
+function sane_import ($input, $names)
 {
   $values = [];
-  $input = &sane_input_array ($method);
+  $input = &sane_input_array ($input);
   $in = sane_var_names ($input);
 
   foreach ($names as $fnc => $name)
@@ -440,4 +442,17 @@ function sane_import ($method, $names)
     }
   return $values;
 }
+
+# Extract group or user from REQUEST_URI.  Used in projects.php and users.php.
+function sane_import_name ($msg_fmt)
+{
+  $n = basename (preg_replace ("/\?.*/", "", $_SERVER['REQUEST_URI']));
+  # Historically, a few groups that don't match the 'name' sanitizer exist,
+  # so we use 'path' which seems safe enough both ways.
+  $sanitized = sane_import (['n' => $n], ['path' => 'n']);
+  if (isset ($sanitized['n']))
+    return $sanitized['n'];
+  $msg = sprintf ($msg_fmt, utils_specialchars (rawurldecode ($n)));
+  exit_error (markup_rich ($msg));
+}
 ?>
