@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/../utils.php');
 # Check if the group uses any trackers to search in.
@@ -235,7 +235,7 @@ function search_box ($searched_words = '', $scope = null, $size = 15)
       );
       $ret .= "\n<select name='type'>" . form_option ('', NULL, _("any"));
       $result =
-        db_query ("SELECT type_id, name FROM group_type ORDER BY type_id");
+        db_execute ("SELECT type_id, name FROM group_type ORDER BY type_id");
       while ($eachtype = db_fetch_array ($result))
         $ret .= form_option ($eachtype['type_id'], $type,
           gettext ($eachtype['name'])
@@ -276,7 +276,7 @@ function search_send_header ()
 function print_search_heading ()
 {
   global $words, $type_of_search, $only_group_id;
-  print '<h2 id="results">' . _('Search results') . "</h2>\n";
+  print html_h (2, _('Search results'), 'results');
   if (!($words && $type_of_search))
     return;
   print "<p>";
@@ -400,7 +400,7 @@ function search_run (
   # Accept only to do a search for more than 2 characters.
   # Exit only if we were not told to avoid returning error messages.
   # Note: we tell user we want more than 3 characters, to incitate to
-  # do clever searchs. But it will be ok for only 2 characters (limit
+  # do clever searches.  But it will be ok for only 2 characters (limit
   # that conveniently allow us to search by items numbers).
   if ($keywords && (strlen ($keywords) < 3) && $return_error_messages)
     {
@@ -471,9 +471,7 @@ function search_run (
           $sql .= " AND t.group_id = ? ";
           $sql_params[] = $only_group_id;
         }
-
-      $sql .= " AND t.spamscore < 5
-        GROUP BY bug_id, summary, date, user_name ORDER BY t.date DESC ";
+      $sql .= " AND t.spamscore < 5 ORDER BY t.date DESC ";
     }
 
   $sql .= " LIMIT ?, ?";
