@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,28 +39,44 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-require_once (dirname (__FILE__) . '/utils.php');
-require_once (dirname (__FILE__) . '/gpg.php');
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+foreach (['utils', 'gpg', 'parsemail'] as $h)
+  require_once (dirname (__FILE__) . "/$h.php");
 
+# Return the common signature for all messages, in a list.
 function sendmail_signature ()
 {
   global $int_delayspamcheck, $sys_default_domain, $sys_home, $sys_name;
 
   if (!empty ($int_delayspamcheck))
-    return '';
-  return "\n\n_______________________________________________\n"
+    return [];
+  return ["\n\n_______________________________________________\n"
     # TRANSLATORS: the argument is site name (like Savannah).
     . sprintf (_("Message sent via %s"), $sys_name)
-    . "\nhttps://$sys_default_domain$sys_home\n";
+    . "\nhttps://$sys_default_domain$sys_home\n"];
 }
 
 function sendmail_format_body (&$message, $context)
 {
+  $message['sig'] = [];
   if (!empty ($context['skip_format_body']))
     return;
   $body = $message['body'];
-  $body = wordwrap ($body, 78) . sendmail_signature ();
+  $body = wordwrap ($body, 78);
+  $message['body'] = $body;
+  $message['sig'] = sendmail_signature ();
+}
+
+function sendmail_finalize_body ($uid, $message, $context)
+{
+  if (!empty ($message['bug_ref']))
+    array_unshift (
+      $message['sig'],
+      sendmail_tracker_lines ($uid, $context, $message['bug_ref'])
+    );
+  $body = $message['body'] . join ($message['sig']);
+  if (!empty ($context['skip_format_body']))
+    return $body;
   # Beuc - 20050316
   # That is what I intended to do:
 
@@ -69,7 +85,7 @@ function sendmail_format_body (&$message, $context)
   # $message = preg_replace("/(?<!\r)\n/", "\r\n", $message);
 
   # However the opposite is certainly more Mailman-compliant.
-  $message['body'] = str_replace ("\r\n", "\n", $body);
+  return str_replace ("\r\n", "\n", $body);
 }
 
 function sendmail_x_savane_server_header ()
@@ -89,6 +105,33 @@ function sendmail_x_savane_server_header ()
   return "X-Savane-Server: $ret\n";
 }
 
+function sendmail_cc_addresses ($user_name, $context)
+{
+  if (!sendmail_have_reply_to ($context))
+    return '';
+  $emails = [];
+  foreach ($user_name as $n)
+    foreach ($n as $e)
+      $emails[$e] = 1;
+  return $emails;
+}
+
+function sendmail_reply_to_headers ($uid, $u_email, $context)
+{
+  global $sys_reply_to;
+  if (!sendmail_have_reply_to ($context, $uid))
+    return '';
+  $reply_to = null;
+  $cc = $context['cc'];
+  $context['uid'] = $uid;
+  unset ($cc[$u_email[0]]);
+  $reply_to = sendmail_expand_template ($sys_reply_to, $context);
+  $cc = array_keys ($cc);
+  if ($reply_to !== null)
+    array_unshift ($cc, $reply_to);
+  return "Reply-To: " . sendmail_encode_recipients ($cc) . "\n";
+}
+
 function sendmail_savane_headers ($context)
 {
   # Add a signature for the server (not if delayed, because it will be added
@@ -96,7 +139,9 @@ function sendmail_savane_headers ($context)
   $ret = sendmail_x_savane_server_header ();
 
   # Necessary for proper utf-8 support.
-  $ret .= "MIME-Version: 1.0\nContent-Type: text/plain;charset=UTF-8\n";
+  # These headers are added in parsemail.php when signed email is formed.
+  if (!parsemail_will_sign ())
+    $ret .= "MIME-Version: 1.0\nContent-Type: text/plain;charset=UTF-8\n";
 
   foreach (['group' => 'X-Savane-Project', 'tracker' => 'X-Savane-Tracker',
     'item' => 'X-Savane-Item-ID'] as $k => $h
@@ -171,6 +216,8 @@ function sendmail_build_headers ($from, &$context, &$message)
 
 function sendmail_explode_addr_list ($to)
 {
+  if ($to === null)
+    return [];
   $to = trim ($to);
   if ($to == "")
     return [];
@@ -243,18 +290,62 @@ function sendmail_spamcheck_queue ($context, $u_name, $u_subj, $message)
   );
 }
 
-# Send mails with specific subject line.
-function sendmail_send_to_list ($user_name, $user_subj, &$message, &$context)
+function tracker_instructions_on_replying ($bug_ref, $have_reply_to)
 {
-  global $int_delayspamcheck;
-  $ret = '';
+  $head = "\n    _______________________________________________________\n\n";
+  # Sender's language preferences may differ from recipient's,
+  # so we would have to translate to a language different from
+  # currently selected; meanwhile, we leave the message untranslated.
+  if ($have_reply_to)
+    $note = sprintf (
+      no_i18n (
+       "To reply to this notification, you have two options:
+        * In the Web UI at <%s>.
+        * By email, ONLY IF you sign your email with a GPG key registered
+          in your %s account AND you include the following line
+          in the reply:"),
+      $bug_ref, $GLOBALS['sys_name']
+    );
+  else
+    # Old footer, without the parameter line and long explanation.
+    $note = sprintf (no_i18n ("Reply to this item at:\n\n  <%s>"), $bug_ref);
+  return $head . preg_replace ('/^ {8}/m', '', $note);
+}
+
+# Return the parameter line for bugs/comment.php with comments for the users
+# unless $sys_reply_to is unconfigured or $uid is an email address rather
+# than a number.
+function sendmail_tracker_lines ($uid, $context, $bug_ref)
+{
+  $have_reply_to = sendmail_have_reply_to ($context, $uid);
+  $note = tracker_instructions_on_replying ($bug_ref, $have_reply_to);
+  if ($have_reply_to)
+    $note .= "\n\n  {savane: user = $uid; tracker = {$context['tracker']}; "
+      . "item = {$context['item']}}";
+  return $note;
+}
+
+function sendmail_mail_signed ($to, $subj, $body, $headers)
+{
+  parsemail_sign_message ($body, $headers);
+  return mail ($to, $subj, $body, $headers);
+}
+
+# Send mails with specific subject line.
+function sendmail_send_to_list ($user_name, $user_subj, $message, $context)
+{
+  $ret = true;
+  $context['cc'] = sendmail_cc_addresses ($user_name, $context);
   foreach ($user_subj as $v => $u_subj)
     {
       $u_name = sendmail_encode_recipients ($user_name[$v]);
-      $body = $message['body']; $headers = $message['headers'];
-      if (empty ($int_delayspamcheck))
+      $body = sendmail_finalize_body ($v, $message, $context);
+      $headers = $message['headers'];
+      $headers .= sendmail_reply_to_headers ($v, $user_name[$v], $context);
+      if (empty ($GLOBALS['int_delayspamcheck']))
         {
-          $ret .= mail ($u_name, $u_subj, $body, $headers);
+          if (!sendmail_mail_signed ($u_name, $u_subj, $body, $headers))
+            $ret = false;
           # TRANSLATORS: the argument is a comma-separated list of recipients.
           fb (sprintf (_("Mail sent to %s"), join (', ', $user_name[$v])));
           continue;
@@ -275,8 +366,8 @@ function sendmail_get_squad_ids ($addresses)
     return [];
   $ph = utils_in_placeholders ($uids);
   $result = db_execute (
-    "SELECT user_id AS id FROM user WHERE status = 'SQD' AND user_id $ph",
-    array_keys ($uids)
+    "SELECT user_id AS id FROM user WHERE status = ? AND user_id $ph",
+    array_merge ([USER_STATUS_SQUAD], array_keys ($uids))
   );
   while ($row = db_fetch_array ($result))
     $squads[$row['id']] = true;
@@ -380,16 +471,12 @@ function sendmail_reduce_names_to_uids ($to, $exclude)
    $names = sendmail_extract_account_names ($to, $exclude);
    if (empty ($names))
      return [$to, $exclude];
-   $ph = utils_in_placeholders ($names);
-   $res = db_execute (
-     "SELECT user_id, user_name FROM user WHERE user_name $ph", $names
-   );
-   while ($row = db_fetch_array ($res))
+   foreach (user_get_array ($names) as $row)
      foreach (['to', 'exclude'] as $a)
-       if (!empty ($$a[strtolower ($row['user_name'])]))
+       if (!empty (${$a}[strtolower ($row['user_name'])]))
          {
-           unset ($$a[strtolower ($row['user_name'])]);
-           $$a[$row['user_id']] = true;
+           unset (${$a}[strtolower ($row['user_name'])]);
+           ${$a}[$row['user_id']] = true;
          }
    return [$to, $exclude];
 }
@@ -398,35 +485,53 @@ function sendmail_email_lines ($uids)
 {
   if (empty ($uids))
     return [];
-  $ph = utils_in_placeholders ($uids);
-  $result = db_execute ("
-    SELECT user_id, email, user_name, realname FROM user WHERE user_id $ph",
-    $uids
-  );
   $lines = [];
-  while ($row = db_fetch_array ($result))
-    {
-      $email = $row['email'];
-      $lines[$row['user_id']] =
-        utils_comply_with_rfc822 ($row['realname']) . " <$email>";
-    }
+  foreach (user_get_array ($uids) as $row)
+    $lines[$row['user_id']] =
+      utils_comply_with_rfc822 ($row['realname']) . " <{$row['email']}>";
   return $lines;
 }
 
+function sendmail_have_reply_to ($context, $uid = 289)
+{
+  if (empty ($GLOBALS['sys_reply_to']))
+    return false;
+  if (!sendmail_addr_is_uid ($uid))
+    return false;
+  if (empty ($context['tracker']) || empty ($context['item']))
+    return false;
+  return true;
+}
+
 function sendmail_user_prefs ($uids, $context)
 {
-  if (empty ($uids))
-    return [];
-  $ph = utils_in_placeholders ($uids);
-  $result = db_execute ("
-    SELECT user_id AS id, preference_value AS val FROM user_preferences
-    WHERE preference_name = \"subject_line\" AND user_id $ph", $uids
-  );
-  $subj = [];
-  while ($row = db_fetch_array ($result))
-    $subj[$row['id']] =
-      sendmail_format_subject_line ($row['val'], $context);
-  return $subj;
+  $ret = [];
+  if (empty ($uids) || empty ($context['user_pref']))
+    return $ret;
+  $have_reply_to = sendmail_have_reply_to ($context);
+  foreach ($context['user_pref'] as $id => $arr)
+    if (array_key_exists ('subject_line', $arr))
+      {
+        $pref = $arr['subject_line'];
+        if ($pref !== null)
+          $ret[$id] = sendmail_format_subject_line ($pref, $context);
+        elseif ($have_reply_to)
+          $ret[$id] = '';
+      }
+  return $ret;
+}
+
+# Check for reserved domains, RFC 2606.
+function sendmail_domain_is_reserved ($addr)
+{
+  if (preg_match ('/@savane[.]test$/', $addr)) # Allow for Savane tests.
+    return false;
+  $regexp =
+    '/@('
+        . '(example[.]((com)|(net)|(org)))'
+        . '|([a-z0-9._-]*[.]((test)|(example)|(invalid)|(localhost)))'
+    . ')\b/';
+  return preg_match ($regexp, $addr);
 }
 
 # Forge the real to list, by parsing every item of the $to list.
@@ -445,7 +550,8 @@ function sendmail_make_to_list ($addresses)
     {
       if (sendmail_addr_is_account_name ($v))
         $v = utils_normalize_email ($v);
-      $to1[$v] = true;
+      if (!sendmail_domain_is_reserved ($v))
+        $to1[$v] = true;
     }
   return $to1;
 }
@@ -459,24 +565,33 @@ function sendmail_list_uids ($vals)
   return $ret;
 }
 
-function sendmail_compile_custom_subject_lines ($to, $context)
+function sendmail_uids_emails_prefs ($to, &$context)
 {
-  $recipients = [];
   $uids = sendmail_list_uids ($to);
-  $email_lines = sendmail_email_lines ($uids);
-  $subj_pfx =
-    sendmail_user_prefs (array_keys ($email_lines), $context);
+  $emails = sendmail_email_lines ($uids);
+  if (!empty ($uids))
+    $context['user_pref'] = user_get_preference (['subject_line'], $uids);
+  $subj_pfx = sendmail_user_prefs (array_keys ($emails), $context);
+  return [$uids, $emails, $subj_pfx];
+}
+
+function sendmail_compile_custom_subject_lines ($to, &$context)
+{
+  list ($uids, $emails, $subj_pfx) =
+    sendmail_uids_emails_prefs ($to, $context);
+  $recipients = [];
   foreach ($to as $v => $ignore)
     {
-      if (empty ($email_lines[$v]))
+      if (empty ($emails[$v]))
         {
           $recipients[] = $v;
           continue;
         }
-      if (empty ($subj_pfx[$v]))
-        $recipients[] = $email_lines[$v];
+      if (array_key_exists ($v, $subj_pfx))
+        continue;
+      $recipients[] = $emails[$v];
     }
-  return [$recipients, $subj_pfx, $email_lines];
+  return [$recipients, $subj_pfx, $emails];
 }
 
 function sendmail_add_context_to_subject ($message, $context)
@@ -488,7 +603,7 @@ function sendmail_add_context_to_subject ($message, $context)
     . " #{$context['item']}] $subject";
 }
 
-function sendmail_make_subjects ($to, $message, $context)
+function sendmail_make_subjects ($to, $message, &$context)
 {
   list ($recipients, $subj_pfx, $emails)
     = sendmail_compile_custom_subject_lines ($to, $context);
@@ -497,9 +612,13 @@ function sendmail_make_subjects ($to, $message, $context)
 
   $user_subj = [];
   foreach ($subj_pfx as $k => $v)
-    $user_subj[$k] =  "$v $subject";
+    {
+      if (strlen ($v))
+        $v .= ' ';
+      $user_subj[$k] = "$v$subject";
+    }
   if (empty ($recipients))
-    return [$user_subj, $emails];
+    return [array_map ("sendmail_encode_header", $user_subj), $emails];
   $v = join (', ', $recipients);
   $emails[$v] = $recipients;
   $user_subj[$v] = $subject;
@@ -535,10 +654,10 @@ function sendmail_mail ($addresses, $message, $context = [])
   sendmail_format_from ($addresses);
   $to = sendmail_make_to_list ($addresses);
   sendmail_build_headers ($addresses['from'], $context, $message);
-  list ($recipients, $subj_pfx, $emails) =
-    sendmail_compile_custom_subject_lines ($to, $context);
-  sendmail_debug_override_address ($recipients, $subj_pfx, $message, $emails);
   list ($subj, $emails) = sendmail_make_subjects ($to, $message, $context);
+  foreach (array_keys ($subj) as $k)
+    if (!is_array ($emails[$k]))
+      $emails[$k] = [$emails[$k]];
   return sendmail_send_to_list ($emails, $subj, $message, $context);
 }
 
@@ -618,15 +737,22 @@ function sendmail_form_message ($form_action, $user_id, $cc_me = true)
   print $HTML->box_bottom ();
 }
 
+function sendmail_expand_template ($template, $context)
+{
+  $keys = ['group', 'tracker', 'item', 'uid', 'project', 'server'];
+  $context['server'] = $GLOBALS['sys_default_domain'];
+  foreach ($keys as $k)
+    if (empty ($context[$k]))
+      $context[$k] = '[' . strtoupper ($k) . ']';
+  $context['item'] = '#' . $context['item'];
+  $context['project'] = $context['group'];
+  foreach ($keys as $k)
+    $subst['%' . strtoupper ($k)] = $context[$k];
+  return strtr ($template, $subst);
+}
+
 function sendmail_format_subject_line ($subject_line, $context)
 {
-  foreach (['group', 'tracker', 'item'] as $k)
-    if (empty ($context[$k]))
-      $context[$k] = '';
-  foreach (["%SERVER" => $GLOBALS['sys_default_domain'],
-    "%PROJECT" => $context['group'], "%TRACKER" => $context['tracker'],
-    "%ITEM" => "#{$context['item']}" ] as $k => $v)
-    $subject_line = str_replace ($k, $v, $subject_line);
-  return $subject_line;
+  return sendmail_expand_template ($subject_line, $context);
 }
 ?>
