@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # A note on cookies.
 #
@@ -76,8 +76,8 @@
 # http://www.ietf.org/rfc/rfc2965.txt (status: proposed standard)
 # https://savannah.gnu.org/task/?6800 (don't use a leading dot)
 
-require_once (dirname (__FILE__) . '/sane.php');
-require_once (dirname (__FILE__) . '/account.php');
+foreach (['sane', 'account', 'random-bytes'] as $h)
+  require_once (dirname (__FILE__) . "/$h.php");
 
 $G_SESSION = $G_USER = [];
 
@@ -108,11 +108,11 @@ function session_login_is_sane ($name, $password)
 function session_login_is_disallowed ($status, $allow_pending)
 {
   $GLOBALS['signal_pending_account'] = 0;
-  if ($status == 'A')
+  if ($status == USER_STATUS_ACTIVE)
      return false;
-  if ($status == 'SQD') # Squad account, silently exit.
+  if ($status == USER_STATUS_SQUAD) # Squad account, silently exit.
     return true;
-  if ($status == 'P')
+  if ($status == USER_STATUS_PENDING)
     {
       if ($allow_pending) # If allowpending (in verify.php), then allow.
         return false;
@@ -120,7 +120,7 @@ function session_login_is_disallowed ($status, $allow_pending)
       # We can't rely on $ffeedback because it's cleared after use.
       $GLOBALS['signal_pending_account'] = 1;
     }
-  elseif ($status == 'D' || $status == 'S')
+  elseif (user_status_is_removed ($status))
     fb (_('Account deleted'), 1);
   else
     fb (_('Account not active'), 1);
@@ -129,16 +129,10 @@ function session_login_is_disallowed ($status, $allow_pending)
 
 function session_fetch_login_data ($name)
 {
-  $resq = db_execute (
-    "SELECT user_id, user_pw, status FROM user WHERE user_name = ?",
-    [$name]
-  );
-  if (db_numrows ($resq) < 1)
-    {
-      fb (sprintf (_('User *%s* not found'), $name), 1);
-      return null;
-    }
-  return db_fetch_array ($resq);
+  $ret = user_get_array ($name);
+  if (empty ($ret))
+    fb (sprintf (_('User *%s* not found.'), $name), 1);
+  return $ret;
 }
 
 function session_login_valid (
@@ -149,7 +143,7 @@ function session_login_valid (
     return false;
 
   $usr = session_fetch_login_data ($name);
-  if ($usr === null)
+  if (empty ($usr))
     return false;
 
   if (session_login_is_disallowed ($usr['status'], $allowpending))
@@ -160,6 +154,7 @@ function session_login_valid (
       fb (_('Invalid password'), 1);
       return false;
     }
+  account_upgrade_pw ($usr['user_pw'], $password, $usr['user_id']);
   session_set_new ($usr['user_id'], $cookie_for_a_year);
   return true;
 }
@@ -169,6 +164,13 @@ function session_issecure ()
   return (getenv ('HTTPS') == 'on');
 }
 
+function session_protocol ()
+{
+  if (session_issecure ())
+    return 'https';
+  return 'http';
+}
+
 function session_needsstayinssl ()
 {
   $res = db_execute ("SELECT stay_in_ssl FROM session WHERE session_hash = ?",
@@ -199,6 +201,7 @@ function session_delete_cookie ($n)
 function session_redirect ($loc)
 {
   header ("Location: $loc");
+  utils_output_debug_footer ();
   exit;
 }
 
@@ -244,17 +247,40 @@ function session_setglobals ($user_id)
   $G_USER = [];
   if ($user_id <= 0)
     return;
-  $result = db_execute (
-    "SELECT user_id, user_name FROM user WHERE user_id = ?", [$user_id]
-  );
-  if (db_numrows ($result))
-    $G_USER = db_fetch_array ($result);
+  $G_USER = user_get_array ($user_id);
 }
 
-# Try inserting new session hash in the table.  Return false if the hash already
-# exists; when $tries_left is zero, the error shows up in the browser and
-# terminates the further output (it would be more user-friendly to explain what
-# happened, but the event is unlikely enough to cut the corners).
+function session_hash_parts ($hash)
+{
+  if (preg_match ('/(.*;)(.*)/', $hash, $m))
+    return [$m[2], $m[1]];
+  return [$hash, null];
+}
+
+function session_fetch_data ($uid, $hash)
+{
+  list ($clean_hash, $param) = session_hash_parts ($hash);
+  if (empty ($param))
+    return null;
+  $res = db_execute (
+    'SELECT * FROM session WHERE user_id = ? AND session_hash LIKE ?',
+    [$uid, "$param%"]
+  );
+  while ($row = db_fetch_array ($res))
+    if (session_valid_hash ($row['session_hash'], $clean_hash))
+      {
+        $row['hash_enc'] = $row['session_hash'];
+        $row['session_hash'] = $hash;
+        return $row;
+      }
+  return null;
+}
+
+# Try inserting a new session hash in the table.  Return null if the hash
+# already exists, and the new database row when the insert is successful;
+# when $tries_left is zero, the error shows up in the browser and terminates
+# the further output (it would be more user-friendly to explain what happened,
+# but the event is unlikely enough to cut the corners).
 function session_try_insert_hash ($user_id, $hash, $tries_left)
 {
   $vals = ['session_hash' => $hash, 'user_id' => $user_id,
@@ -266,7 +292,16 @@ function session_try_insert_hash ($user_id, $hash, $tries_left)
   $res = db_autoexecute ('session', $vals);
   db_query_prevent_die (false);
   utils_restore_warnings ($saved);
-  return !empty ($res);
+  if (empty ($res))
+    return null;
+  return $vals;
+}
+
+function session_generate_ticket ()
+{
+  $ret = microtime (true);
+  $ret -= floor ($ret);
+  return (int)($ret * 10000);
 }
 
 function session_generate_hash ($user_id)
@@ -275,11 +310,17 @@ function session_generate_hash ($user_id)
   while ($tries--)
     {
       $hash = random_hash ();
-      if (session_try_insert_hash ($user_id, $hash, $tries))
-        return db_execute (
-          'SELECT * FROM session WHERE session_hash = ?', [$hash]
-        );
-      trigger_error ("duplicate hash $hash detected, tries left: $tries");
+      $ticket = session_generate_ticket ();
+      $hhash = "$ticket;" . account_encryptpw ($hash, true);
+      $vals = session_try_insert_hash ($user_id, $hhash, $tries);
+      if (empty ($vals))
+        {
+          trigger_error ("duplicate hash $hhash detected, tries left: $tries");
+          continue;
+        }
+      $vals['hash_enc'] = $hhash;
+      $vals['session_hash'] = "$ticket;$hash";
+      return $vals;
     }
   return false;
 }
@@ -287,8 +328,9 @@ function session_generate_hash ($user_id)
 function session_set_new ($user_id, $cookie_for_a_year)
 {
   global $G_SESSION, $session_hash;
-  $res = session_generate_hash ($user_id);
-  $G_SESSION = db_fetch_array ($res);
+  $G_SESSION = session_generate_hash ($user_id);
+  if (empty ($G_SESSION))
+    return;
   session_setglobals ($G_SESSION['user_id']);
   $session_hash = $G_SESSION['session_hash'];
 
@@ -296,7 +338,7 @@ function session_set_new ($user_id, $cookie_for_a_year)
   # kill all other sessions.
   if (user_get_preference ("keep_only_one_session"))
     db_execute ("DELETE FROM session WHERE session_hash <> ? AND user_id = ?",
-      [$session_hash, $user_id]
+      [$G_SESSION['hash_enc'], $user_id]
     );
   session_set_new_cookies ($user_id, $cookie_for_a_year);
 }
@@ -305,7 +347,6 @@ function session_set_new ($user_id, $cookie_for_a_year)
 function session_set_new_cookies ($user_id, $cookie_for_a_year = 0)
 {
   $stay_in_ssl = session_stay_in_ssl ();
-  # Concatinate current time, and random seed for MD5 hash.
   # Set a non-secure cookie so that Savane automatically redirects to HTTPS.
   if ($stay_in_ssl)
     session_cookie ('redirect_to_https', 1, $cookie_for_a_year, 0);
@@ -322,34 +363,17 @@ function session_set_new_cookies ($user_id, $cookie_for_a_year = 0)
 function session_set ()
 {
   global $G_SESSION, $G_USER;
-
-  # Assume bad session_hash and session. If all checks work, then allow
-  # otherwise make new session.
-  $id_is_good = 0;
-
-  # Here also check for good hash, set if new session is needed.
   extract (sane_import ('cookie',
     ['hash' => 'session_hash', 'digits' => 'session_uid'])
   );
-  if ($session_hash && $session_uid)
-    {
-      $result = db_execute ("
-        SELECT * FROM session WHERE session_hash = ? AND user_id = ?",
-        [$session_hash, $session_uid]
-      );
-      $G_SESSION = db_fetch_array ($result);
+  if (!($session_hash && $session_uid))
+    return;
+  $G_SESSION = session_fetch_data ($session_uid, $session_hash);
 
-      if (!empty ($G_SESSION['session_hash']))
-        $id_is_good = 1;
-    } # if ($session_hash && $session_uid)
-
-  if ($id_is_good)
-    session_setglobals ($G_SESSION['user_id']);
+  if (empty ($G_SESSION['session_hash']))
+    unset ($G_SESSION, $G_USER);
   else
-    {
-      unset ($G_SESSION);
-      unset ($G_USER);
-    }
+    session_setglobals ($G_SESSION['user_id']);
 }
 
 function session_count ($uid)
@@ -359,23 +383,86 @@ function session_count ($uid)
   ));
 }
 
+function session_valid_hash ($stored_hash, $hash)
+{
+  list ($clean_hash, $ticket) = session_hash_parts ($stored_hash);
+  return account_validpw ($clean_hash, $hash);
+}
+
 function session_exists ($uid, $hash)
 {
-  $res = db_execute (
-    "SELECT NULL FROM session WHERE user_id = ?  AND session_hash = ?",
-    [$uid, $hash]
-  );
-  return db_numrows ($res) == 1;
+  return session_fetch_data ($uid, $hash) !== null;
 }
 
 function session_logout ()
 {
-  # If the session was validated, we can assume that the cookie session_hash
-  # is reliable.
-  extract (sane_import ('cookie', ['xdigits' => 'session_hash']));
-  db_execute ("DELETE FROM session WHERE session_hash = ?", [$session_hash]);
+  db_execute ("DELETE FROM session WHERE session_hash = ?",
+    [$GLOBALS['G_SESSION']['hash_enc']]
+  );
   session_delete_cookie ('redirect_to_https');
   session_delete_cookie ('session_hash');
   session_delete_cookie ('session_uid');
 }
+
+# Check if cookies are enabled.
+function session_check_cookies ($uri, $uri_urlencoded)
+{
+  global $sys_default_domain, $sys_https_url, $sys_home;
+  $url_prefix = "$sys_https_url{$sys_home}account/login.php?uri=";
+  if (isset ($_COOKIE["cookie_probe"]))
+    return;
+  extract (sane_import ('get', ['true' => 'cookie_test']));
+  if (empty ($cookie_test))
+    {
+      # Request a cookie and reload the page to see
+      # if the client actually sends that cookie.
+      session_cookie ('cookie_probe', 1);
+      # $uri used to be not URL-encoded, it caused login problems,
+      # see Savannah sr #108277.
+      session_redirect ("$url_prefix$uri_urlencoded&cookie_test=1");
+    }
+  # TRANSLATORS: the first argument is a domain (like "savannah.gnu.org");
+  # the second argument is a URL ("[URL label]" transforms to a link).
+  $msg = sprintf (
+    _("Savane thinks your cookies are not activated for %s.\nPlease activate "
+      . "cookies in your web browser for this website\nand [%s try to login "
+      . "again]."), $sys_default_domain, "$url_prefix$uri"
+  );
+  fb ($msg, 1);
+}
+
+# Set the theme from user's preferences unless the cookie is already set.
+function session_set_theme ()
+{
+  if (isset ($_COOKIE['SV_THEME']))
+    return;
+  $theme = user_get_field (0, 'theme');
+  if (strlen ($theme) > 0)
+    utils_setcookie ('SV_THEME', $theme, time () + 60 * 60 * 24);
+}
+
+# Log in the 'brother' domain if needed, and return back.
+# Only returns when the action isn't needed; otherwise exits
+# in session_redirect ().
+function session_login_brother ($uri, $uri_urlencoded)
+{
+  global $sys_brother_domain, $sys_home;
+  global $brotherhood, $session_hash, $cookie_for_a_year, $stay_in_ssl;
+  global $from_brother;
+  if (empty ($sys_brother_domain) || empty ($brotherhood))
+    return;
+  # If a brother server exists, login there too, if we are not
+  # already coming from there.
+  $root_url = session_protocol () . "://$sys_brother_domain";
+
+  if ($from_brother)
+    # Redirect back after logging in the 'brother' domain.
+    session_redirect ("$root_url$uri");
+  # Log in the 'brother' domain.
+  session_redirect ("$root_url{$sys_home}account/login.php?"
+    . "session_uid=" . user_getid () . "&session_hash=$session_hash"
+    . "&login=1&cookie_for_a_year=$cookie_for_a_year&from_brother=1"
+    . "&stay_in_ssl=$stay_in_ssl&brotherhood=1&uri=$uri_urlencoded"
+  );
+}
 ?>
