@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # First, initialize some globals var: conffile allows two vals for
 # admin conveniency.
@@ -83,7 +83,7 @@ function spam_flag (
   if (!$reporter_user_id)
     $reporter_user_id = user_getid ();
 
-  # Check if the reported havent flagged the incriminated comment already.
+  # Check if the reported haven't flagged the incriminated comment already.
   $result = db_execute ("
     SELECT id FROM trackers_spamscore
     WHERE
@@ -251,7 +251,7 @@ function spam_get_user_score ($user_id = 0, $set_by_user_id = 0)
   if (!$user_id)
     $user_id = user_getid ();
 
-  # Anonymous get always a score of 3 (requires two users to succesfully
+  # Anonymous get always a score of 3 (requires two users to successfully
   # mark as spam something, only one group member).
   if ($user_id == 100)
     return 3;
@@ -267,7 +267,7 @@ function spam_get_user_score ($user_id = 0, $set_by_user_id = 0)
   # We cannot do a count because it does not allow us to use GROUP BY.
   $userscore = 0;
   $result = db_execute ("
-    SELECT score FROM trackers_spamscore
+    SELECT max(score) AS score FROM trackers_spamscore
     WHERE affected_user_id = ? $set_by_user_id_sql GROUP BY reporter_user_id",
     array_merge ([$user_id], $set_by_user_id_params)
   );
@@ -352,7 +352,7 @@ function spam_add_to_spamcheck_queue (
   # If anonymous, increment priority (yes, it will be meaningless on sites
   # where only anonymous posts are checked):
   # While we may consider giving the priority to logged in users for their
-  # confort, we have to take into account that we need to start with post
+  # comfort, we have to take into account that we need to start with post
   # that are the most likely to contain spams.
   if (!user_isloggedin ())
     $priority++;
