@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,13 +39,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # theme value is fetched by getting the cookie. But we keep in the
 # database the setting, so someone using another computer can easily
 # remember the theme he previously chose.
 
-require_once(dirname(__FILE__).'/utils.php');
+require_once (dirname (__FILE__) . '/utils.php');
 
 # Jump to the next theme available.
 function theme_rotate_jump ($user_theme)
@@ -58,46 +58,43 @@ function theme_rotate_jump ($user_theme)
     theme_get_random (true);
 }
 
-# Return an array with all the themes, but not the special case "rotate"
+function theme_filter_css ($path, $file)
+{
+  global $forbid_theme_regexp;
+  # Ignore symlinks.
+  if (is_link ("$path$file"))
+    return null;
+
+  # Take only css files.
+  if (!preg_match ("/^(.*)\.css$/", $file, $matches))
+    return null;
+
+  # Forbidden themes are ignored.
+  if (preg_match ($forbid_theme_regexp, strtolower ($matches[1])))
+    return null;
+  return $matches[1];
+}
+
+# Return an array with all the themes except the special cases "rotate"
 # and "random".
 function theme_list ()
 {
-  utils_get_content("forbidden_theme");
-
-  # Feed the array.
-  $theme = array();
-  $dir = opendir($GLOBALS['sys_www_topdir']."/css/");
-  while ($file = readdir($dir))
+  utils_get_content ("forbidden_theme");
+  $theme = [];
+  $path = $GLOBALS['sys_www_topdir'] . '/css/';
+  $dir = opendir ($path);
+  while ($file = readdir ($dir))
     {
-      # Ignore symlinks.
-      if (is_link($GLOBALS['sys_www_topdir']."/css/$file"))
-        continue;
-
-      # Take only correct css files.
-      if (!preg_match("/^(.*)\.css$/", $file, $matches))
-        continue;
-
-      # base.css is always ignored
-      # (as of nov 2006, there are in the subdirectory internal, so this
-      # is only here for backward compat).
-      if ($matches[1] == "base")
-        continue;
-
-      # Forbidden themes are also ignored.
-      if (preg_match($GLOBALS['forbid_theme_regexp'], strtolower($matches[1])))
-        continue;
-
-      $theme[] = $matches[1];
+      $entry = theme_filter_css ($path, $file);
+      if ($entry !== null)
+        $theme[] = $entry;
     }
-  closedir($dir);
-
-  # Sort themes - case insensitive.
-  natcasesort($theme);
-
-  # No result? Return only the default theme.
+  closedir ($dir);
+  natcasesort ($theme); # Sort themes - case insensitive.
+  # No result?  Return only the default theme.
   # (If there were no result, there is a problem anyway somewhere in the
   # installation.)
-  if (!count($theme))
+  if (!count ($theme))
     $theme[] = $GLOBALS['sys_themedefault'];
   return $theme;
 }
@@ -106,23 +103,20 @@ function theme_list ()
 function theme_guidelines_check ($theme)
 {
   # Get from the README the latest GUIDELINES number.
-  preg_match("/VERSION: (.*)/",
-             utils_read_file($GLOBALS['sys_www_topdir']."/css/README"),
-             $latest);
+  preg_match ("/VERSION: (.*)/",
+    utils_read_file ($GLOBALS['sys_www_topdir'] . "/css/README"), $latest
+   );
   # Get from the css the current GUIDELINES number.
-  preg_match("/\/\* GUIDELINES VERSION FOLLOWED: (.*) \*\//",
-             utils_read_file($GLOBALS['sys_www_topdir']."/css/".$theme.".css"),
-             $current);
-
-  if ($latest[1] != $current[1])
-    return false;
-  return true;
+  preg_match ("/\/\* GUIDELINES VERSION FOLLOWED: (.*) \*\//",
+    utils_read_file ($GLOBALS['sys_www_topdir'] . "/css/$theme.css"), $current
+  );
+  return $latest[1] == $current[1];
 }
 
 # If the theme is valid, return $user_theme; else return default theme.
 function theme_validate ($user_theme)
 {
-  utils_get_content("forbidden_theme");
+  utils_get_content ("forbidden_theme");
 
   # Disallow going towards filesystem root and other queer paths.
   $forbidden = preg_match (',(/[.]*/|^/|/$|\s),', $user_theme);
@@ -134,15 +128,10 @@ function theme_validate ($user_theme)
     $forbidden = true;
 
   if ($forbidden)
-    {
-      trigger_error ("Forbidden theme '$user_theme', user " . user_getname ());
-      return $GLOBALS['sys_themedefault'];
-    }
+    return $GLOBALS['sys_themedefault'];
 
   if (file_exists ($GLOBALS['sys_www_topdir'] . "/css/$user_theme.css"))
     return $user_theme;
-  if ($user_theme != '')
-    trigger_error ("Invalid theme '$user_theme', user " . user_getname ());
   return $GLOBALS['sys_themedefault'];
 }
 
@@ -295,6 +284,15 @@ function theme_guess ()
   define('SV_THEME', theme_validate($_COOKIE['SV_THEME']));
 }
 
+function theme_actualise_theme ($user_theme)
+{
+  if ($user_theme == "Default")
+    return "";
+  if ($user_theme !== 'rotate' && $user_theme !== 'random')
+    return theme_validate ($user_theme);
+  return $user_theme;
+}
+
 # Select theme.
 function theme_select ()
 {
@@ -307,7 +305,7 @@ function theme_select ()
     {
       # Anonymous user: guess the theme from cookies.
       theme_guess ();
-       return;
+      return;
     }
 
   # When the user is logged in, the theme comes from user's settings.
