@@ -30,7 +30,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -43,22 +43,61 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 namespace {
 
 $dir_name = dirname (__FILE__);
-require_once ("$dir_name/../trackers/transition.php");
-require_once ("$dir_name/../trackers/cookbook.php");
-require_once ("$dir_name/../utils.php");
+foreach (['trackers/transition', 'trackers/cookbook', 'utils', 'group'] as $i)
+  require_once ("$dir_name/../$i.php");
 
-# Get all the possible bug fields for this group both used and unused. If
-# used then show the group specific information about field usage
-# otherwise show the default usage parameter.
-# Make sure array element are sorted by ascending place.
-function trackers_data_get_all_fields ($group_id = false, $reload = false)
+function trackers_data_fetch_all_fields ($group_ids, $group_cond)
+{
+  $art_field = ARTIFACT . '_field';
+  $result = db_execute ("
+    SELECT
+      af.bug_field_id, field_name, display_type, display_size, label,
+      description, scope, required, empty_ok, keep_history, special, custom,
+      group_id, use_it, show_on_add, show_on_add_members, place, custom_label,
+      custom_description, custom_display_size, custom_empty_ok,
+      custom_keep_history, group_id
+    FROM $art_field af JOIN {$art_field}_usage afu
+    WHERE $group_cond AND af.bug_field_id = afu.bug_field_id", $group_ids
+  );
+  $ret = [];
+  while ($field_array = db_fetch_array ($result))
+    $ret[] = $field_array;
+  return $ret;
+}
+
+function trackers_data_rewind_bf_usage ()
 {
   global $BF_USAGE_BY_ID, $BF_USAGE_BY_NAME, $AT_START;
+  reset ($BF_USAGE_BY_ID);
+  reset ($BF_USAGE_BY_NAME);
+  $AT_START = true;
+}
+
+function trackers_data_assign_fields ($all_fields, $group_ids)
+{
+  global $BF_USAGE_BY_ID, $BF_USAGE_BY_NAME;
+  # Put all used fields in a global array for faster access.
+  # Index both by field_name and bug_field_id.
+  # The defaults (group_id = GROUP_NONE) are overridden
+  # with group-specific settings.
+  foreach ($group_ids as $p)
+    foreach ($all_fields as $field_array)
+      if ($field_array['group_id'] == $p)
+        $BF_USAGE_BY_ID[$field_array['bug_field_id']] =
+          $BF_USAGE_BY_NAME[$field_array['field_name']] = $field_array;
+}
+
+# Get all the possible bug fields for this group, both used and unused.
+# If used, then show the group specific information about field usage,
+# otherwise show the default usage parameter.
+function trackers_data_get_all_fields ($group_id, $reload = false)
+{
+  global $BF_USAGE_BY_ID, $BF_USAGE_BY_NAME;
 
   if (!ctype_alnum (strval (ARTIFACT)))
     util_die ("Invalid ARTIFACT name: " . ARTIFACT);
@@ -68,43 +107,11 @@ function trackers_data_get_all_fields ($group_id = false, $reload = false)
     return;
 
   $BF_USAGE_BY_ID = $BF_USAGE_BY_NAME = [];
-
-  # First get the all the defaults.
-  $art_field = ARTIFACT . '_field';
-  $sql = "
-    SELECT
-      af.bug_field_id, field_name, display_type, display_size, label,
-      description, scope, required, empty_ok, keep_history, special, custom,
-      group_id, use_it, show_on_add, show_on_add_members, place, custom_label,
-      custom_description, custom_display_size, custom_empty_ok,
-      custom_keep_history
-    FROM $art_field af, {$art_field}_usage afu
-    WHERE group_id = ? AND af.bug_field_id = afu.bug_field_id";
-
-  $res_defaults = db_execute ($sql, [100]);
-
-  # Put all used fields in a global array for faster access.
-  # Index both by field_name and bug_field_id.
-  while ($field_array = db_fetch_array ($res_defaults))
-    {
-      $BF_USAGE_BY_ID[$field_array['bug_field_id'] ] = $field_array;
-      $BF_USAGE_BY_NAME[$field_array['field_name'] ] = $field_array;
-    }
-
-  # Select all group-specific entries.
-  $res_group = db_execute ($sql, [$group_id]);
-
-  # Override entries in the default array.
-  while ($field_array = db_fetch_array ($res_group))
-    {
-      $BF_USAGE_BY_ID[$field_array['bug_field_id'] ] = $field_array;
-      $BF_USAGE_BY_NAME[$field_array['field_name'] ] = $field_array;
-    }
-
-  # Rewind internal pointer of global arrays.
-  reset ($BF_USAGE_BY_ID);
-  reset ($BF_USAGE_BY_NAME);
-  $AT_START = true;
+  $group_ids = [GROUP_NONE, $group_id];
+  $cond = 'group_id ' . utils_in_placeholders ($group_ids);
+  $all_fields = trackers_data_fetch_all_fields ($group_ids, $cond);
+  trackers_data_assign_fields ($all_fields, $group_ids);
+  trackers_data_rewind_bf_usage ();
 }
 
 function trackers_data_get_item_group ($item_id)
@@ -132,10 +139,10 @@ function trackers_data_get_notification_settings ($group_id, $tracker)
     $result, 0, "{$tracker}_private_exclude_address"
   );
   $cat_field_name = "category_id";
-  # Warning: The hardcoded fiels names: bug_fv_id and bug_field_id will need
+  # Warning: The hardcoded field names bug_fv_id and bug_field_id will need
   # to be changed one day to generic names since they apply to bugs but also
-  # to suuports,tasks and patch-related tables. For now these fileds are
-  # called bug_xxx whatever the service related tables. Too much work to make
+  # to support,tasks and patch-related tables.  For now these fields are
+  # called bug_xxx whatever the service related tables.  Too much work to make
   # all the changes in the code.
   $result = db_execute ("
     SELECT fv.bug_fv_id, fv.value, fv.email_ad, fv.send_all_flag
@@ -170,7 +177,7 @@ function trackers_data_show_notification_settings ($group_id, $tracker)
   $grtrsettings = trackers_data_get_notification_settings (
     $group_id, $tracker
   );
-  if (!(user_ismember ($group_id, 'A')))
+  if (!(user_ismember ($group_id, MEMBER_FLAGS_ADMIN)))
     return;
   $cat_ck = $glob_ck = $both_ck = false;
   if ($grtrsettings['glnotif'] == 0)
@@ -406,8 +413,10 @@ function trackers_data_get_item_notification_info ($item, $artifact, $updated)
     $addr = trackers_data_catnotif ($item, $artifact, $updated, $newad);
   if ($notif > 0)
     $addr = array_merge ($addr, $newad);
-  $addr = trim (join (',', array_unique ($addr)));
-  return [$addr, $addr !== ""];
+  $addr = array_filter (
+    $addr, function ($x) { $x = trim (strval ($x)); return !empty ($x); }
+  );
+  return trim (join (',', array_unique ($addr)));
 }
 
 function trackers_data_cmp ($ar1, $ar2, $field)
@@ -503,16 +512,14 @@ function trackers_data_get_field_predefined_values (
 
   if ($active_only)
     {
+      $status_cond = "status IN (?, ?)";
+      $status_cond_params = [USER_STATUS_ACTIVE, USER_STATUS_PENDING];
       if ($checked && !is_array ($checked))
         {
-          $status_cond = "AND (status IN ('A', 'P') OR value_id = ?) ";
-          $status_cond_params = [$checked];
-        }
-      else
-        {
-          $status_cond = "AND status IN ('A', 'P') ";
-          $status_cond_params = [];
+          $status_cond = "($status_cond OR value_id = ?)";
+          $status_cond_params[] = $checked;
         }
+      $status_cond = "AND $status_cond ";
     }
 
   # The fields value_id and value must be first in the select statement,
@@ -525,7 +532,7 @@ function trackers_data_get_field_predefined_values (
       order_id, status
     FROM " . ARTIFACT . "_field_value
     WHERE group_id = ? AND bug_field_id = ? $status_cond
-    ORDER BY order_id,value ASC";
+    ORDER BY order_id, value ASC";
   $res_value = db_execute (
     $sql, array_merge ([$group_id, $field_id], $status_cond_params)
   );
@@ -572,7 +579,8 @@ function trackers_data_is_custom ($field, $by_field_id = false)
 
 function trackers_data_nonempty ($field, $idx, $by_field_id = false)
 {
-  return !empty (trackers_data_field_val ($field, $idx, $by_field_id));
+  $x = trackers_data_field_val ($field, $idx, $by_field_id);
+  return !empty ($x);
 }
 
 function trackers_data_is_special ($field, $by_field_id = false)
@@ -771,7 +779,7 @@ function trackers_data_get_default_value ($field, $by_field_id = false)
   if ($by_field_id)
     $field = trackers_data_get_field_name ($field);
 
-  $result = db_query ('DESCRIBE ' . ARTIFACT . ' `' . $field . '`');
+  $result = db_execute ('DESCRIBE ' . ARTIFACT . ' `' . $field . '`');
   return (db_result ($result, 0, 'Default'));
 }
 
@@ -813,7 +821,7 @@ function trackers_data_is_value_set_empty (
 }
 
 # Initialize the set of values for a given field for a given group by using
-# the system default (default values belong to group_id 'None'  = 100).
+# the system default (default values belong to group_id  = GROUP_NONE).
 function trackers_data_copy_default_values (
   $field, $group_id, $by_field_id = false
 )
@@ -821,11 +829,11 @@ function trackers_data_copy_default_values (
   if (!$by_field_id)
     $field_id = trackers_data_get_field_id ($field);
 
-  # If group_id is 100 (None), it is a null operation,
-  # because default values belong to group_id 100 by definition.
-  if ($group_id == 100)
+  # If group_id is 100 (GROUP_NONE), it is a null operation,
+  # because default values belong to GROUP_NONE by definition.
+  if ($group_id == GROUP_NONE)
     return;
-  # First delete the exisiting value if any.
+  # First delete the existing value if any.
   $res = db_execute ("
     DELETE FROM " . ARTIFACT . "_field_value
     WHERE bug_field_id = ? AND group_id = ?",
@@ -838,9 +846,8 @@ function trackers_data_copy_default_values (
   # in ANSI SQL to SELECT. So do it by hand !
   $res = db_execute ("
     SELECT value_id, value, description, order_id, status
-    FROM " . ARTIFACT . "_field_value
-    WHERE bug_field_id = ? AND group_id = 100",
-    [$field_id]
+    FROM " . ARTIFACT . "_field_value WHERE bug_field_id = ? AND group_id = ?",
+    [$field_id, GROUP_NONE]
   );
   $rows = db_numrows ($res);
 
@@ -896,14 +903,13 @@ function trackers_data_get_field_value ($item_fv_id)
   );
 }
 
-# See if this field value belongs to group None (100). In this case
+# See if this field value belongs to GROUP_NONE. In this case
 # it is a so called default value.
 function trackers_data_is_default_value ($item_fv_id)
 {
   $res = db_execute ("
     SELECT bug_field_id,value_id FROM " . ARTIFACT . "_field_value
-    WHERE bug_fv_id = ? AND group_id = 100",
-    [$item_fv_id]
+    WHERE bug_fv_id = ? AND group_id = ?", [$item_fv_id, GROUP_NONE]
   );
   return ((db_numrows ($res) >= 1) ? $res : false);
 }
@@ -923,9 +929,9 @@ function trackers_data_create_value (
   if (!$by_field_id)
     $field_id = trackers_data_get_field_id ($field);
 
-  # If group_id = 100 (None), then do nothing,
-  # because no real group should have the group number '100'.
-  if ($group_id == 100)
+  # If group_id = GROUP_NONE, then do nothing,
+  # because no real group should have that number.
+  if ($group_id == GROUP_NONE)
     return;
 
   # If the current value set for this group is empty
@@ -971,7 +977,7 @@ function trackers_data_update_value (
       return;
     }
 
-  # Updating a bug field value that belong to group 100 (None) is
+  # Updating a bug field value that belong to GROUP_NONE is
   # forbidden. These are default values that cannot be changed, so
   # make sure to copy the default values first in the group context first.
 
@@ -987,8 +993,8 @@ function trackers_data_update_value (
     }
   else
     {
-      $where_cond = "bug_fv_id = ? AND group_id <> 100";
-      $where_cond_params = [$item_fv_id];
+      $where_cond = "bug_fv_id = ? AND group_id <> ?";
+      $where_cond_params = [$item_fv_id, GROUP_NONE];
     }
 
   $result = db_autoexecute (
@@ -1009,11 +1015,11 @@ function trackers_data_update_value (
 }
 
 # Reset a field settings to its defaults usage (values are untouched). The defaults
-# always belong to group_id 100 (None) so make sure we don't delete entries for
-# group 100.
+# always belong to group_id GROUP_NONE so make sure we don't delete entries for
+# GROUP_NONE.
 function trackers_data_reset_usage ($field_name, $group_id)
 {
-  if ($group_id == 100)
+  if ($group_id == GROUP_NONE)
     return;
 
   $field_id = trackers_data_get_field_id ($field_name);
@@ -1041,18 +1047,19 @@ function trackers_data_update_usage (
 
   $field_id = trackers_data_get_field_id ($field_name);
 
-  $lbl = isset ($label) ? $label : null;
-  $desc = isset ($description) ? $description : null;
-  $disp_size = isset ($display_size) ? $display_size : null;
-  $emp_ok = isset ($empty_ok) ? $empty_ok : null;
-  $keep_hist = isset ($keep_history) ? $keep_history : null;
+  $lbl = isset ($label)? $label: null;
+  $desc = isset ($description)? $description: null;
+  $disp_size = isset ($display_size)? $display_size: null;
+  $emp_ok = isset ($empty_ok)? $empty_ok: null;
+  $keep_hist = isset ($keep_history)? $keep_history: null;
 
-  if (!isset ($show_on_add))
-    $show_on_add = 0;
-  if (!isset ($show_on_add_members))
-    $show_on_add_members = 0;
-  if (!isset ($transition_default_auth))
-    $transition_default_auth = '';
+  foreach (
+    [ 'show_on_add' => 0, 'show_on_add_members' => 0, 'use_it' => 0,
+      'transition_default_auth' => '']
+    as $var => $val
+  )
+    if (!isset ($$var))
+      $$var = $val;
 
   # See if this field usage exists in the table for this group.
   $result = db_execute ("
@@ -1102,22 +1109,13 @@ function trackers_data_update_usage (
 function trackers_data_get_technicians ($group_id)
 {
   # Get list of members.
-  $members_res = db_execute ("
-    SELECT user.user_id FROM user, user_group
-    WHERE user.user_id = user_group.user_id AND user_group.group_id = ?",
-    [$group_id]
-  );
+  $uids = array_keys (member_get_group_members ($group_id));
+  $uids = member_check_array ($uids, $group_id, 1);
   $sql = "SELECT user_id, user_name FROM user WHERE ";
-  $uids = [];
-  while ($member = db_fetch_array ($members_res))
-    $uids[] = $member['user_id'];
-  $uids = member_check_array (
-    $uids, $group_id, member_create_tracker_flag (ARTIFACT) . '1'
-  );
   if (empty ($uids))
     $sql .= 'NULL'; # Return a valid (but empty) result set.
   else
-    $sql .= 'user_id IN (' . utils_placeholders ($uids) . ')';
+    $sql .= 'user_id ' . utils_in_placeholders ($uids);
   $sql .= " ORDER BY user_name";
   return db_execute ($sql, $uids);
 }
@@ -1160,7 +1158,7 @@ function trackers_data_get_followups ($item_id)
             AND f.field_name = 'comment_type_id'
         ) fv
         ON bhi.type = fv.value_id AND bhi.grp = fv.group_id
-      ORDER BY date", [$item_id]
+      ORDER BY bug_history_id", [$item_id]
     );
 }
 
@@ -1314,14 +1312,7 @@ function trackers_data_add_history (
   # Useless if already considered to be spam.
   if ($spamscore < 5)
     {
-      $result = db_execute ("
-        SELECT group_id FROM $artifact WHERE bug_id = ?",
-        [$item_id]
-      );
-      if (db_numrows ($result))
-        $group_id = db_result ($result, 0, 'group_id');
-      else
-        exit_error (_("Item not found"));
+      $group_id = utils_find_item ($artifact, $item_id)['group_id'];
       spam_add_to_spamcheck_queue (
         $item_id, db_insertid ($result), $artifact, $group_id, $spamscore
       );
@@ -1398,14 +1389,9 @@ namespace trackers_data\handle_update {
 
 function fetch_data ($group_id, $item_id, $vfl)
 {
-  $missing = [];
   # Make sure required fields are not empty.
-  if (!$group_id)
-    $missing[] = 'group_id';
-  if (!$item_id)
-    $missing[] = 'item_id';
-  if (count ($missing))
-    exit_missing_param ($missing);
+  $missing = ['group_id' => $group_id, 'item_id' => $item_id];
+  exit_if_missing (array_keys ($missing), $missing);
 
   if (!trackers_check_empty_fields ($vfl, false))
     return [false, false];
@@ -1649,9 +1635,8 @@ function handle_update (
 
           # Check if we are supposed to send all modifications
           # to an address.
-          list ($address, $sendall) =
-            trackers_data_get_item_notification_info ($dep, $art, 1);
-          if ($sendall && trim ($address) != "")
+          $address = trackers_data_get_item_notification_info ($dep, $art, 1);
+          if (trim ($address) != "")
             $extra_addr[] = $address;
         }
     }
@@ -1689,9 +1674,7 @@ function trackers_data_reassign_item (
   global $group_id;
 
   # Can only be done by a tracker manager.
-  if (
-    !member_check (0, $group_id, member_create_tracker_flag (ARTIFACT) . '2')
-  )
+  if (!member_check (0, $group_id, 2))
     return false;
   # If the new group_id is equal to the current one, nothing need
   # to be done, unless the artifact changed.
@@ -1812,7 +1795,7 @@ function trackers_data_reassign_item (
     . ARTIFACT . " #$item_id\n\n"
     . "Following are the information included in the original report:\n\n";
 
-  $res_show = db_query ("SHOW COLUMNS FROM " . ARTIFACT);
+  $res_show = db_execute ("SHOW COLUMNS FROM " . ARTIFACT);
   $list = array();
   while ($row_show = db_fetch_array ($res_show))
     {
@@ -1869,7 +1852,6 @@ function trackers_data_reassign_item (
        $row_attachment['filename']
       );
       fb ($msg, 1);
-      dbg ("sql: $sql");
     }
 
   # Duplicate CC List.
@@ -1935,15 +1917,12 @@ function trackers_data_reassign_item (
     ],
     DB_AUTOQUERY_INSERT
   );
-
-  # Now send the notification (this must be done here, because we got
-  # here the proper new id, etc).
-  list ($additional_address, $sendall) =
-    trackers_data_get_item_notification_info (
-      $new_item_id, $reassign_change_artifact, 1
-    );
+  $address = '';
+  trackers_append_followup_notif_addresses (
+    $address, $new_item_id, true, $reassign_change_artifact
+  );
   trackers_mail_followup (
-    $new_item_id, $additional_address, false, false, $reassign_change_artifact
+    $new_item_id, $address, false, false, $reassign_change_artifact
   );
 
   # If we get here, assume everything went properly.
@@ -2091,6 +2070,8 @@ function trackers_data_create_item ($group_id, $vfl, &$extra_addresses)
   $insert_fields['details'] = utils_specialchars ($vfl['details']);
   $insert_fields['spamscore'] = $spamscore;
   $insert_fields['ip'] = '127.0.0.1';
+  for ($i = 1; $i <= 10; $i++)
+    $insert_fields["custom_ta$i"] = '';
 
   # Actually insert the entry.
   $result = db_autoexecute (ARTIFACT, $insert_fields, DB_AUTOQUERY_INSERT);
@@ -2177,8 +2158,8 @@ function trackers_data_get_value (
   if (db_numrows ($result) > 0)
     return db_result ($result, 0, 'value');
 
-  # ... if it fails, look for system wide default values (group_id=100)...
-  $args[0] = 100;
+  # If it fails, look for system-wide default values (group_id = GROUP_NONE).
+  $args[0] = GROUP_NONE;
   $result = db_execute ($sql, $args);
   if (db_numrows ($result) > 0)
     return db_result ($result, 0, 'value');
@@ -2197,23 +2178,49 @@ function trackers_data_get_canned_responses ($group_id)
   );
 }
 
-function trackers_data_get_reports ($group_id, $user_id = 100)
+function trackers_data_get_reports ($group_id, $user_id)
 {
-  # Currently, reports are group based.
-  # Print first system reports.
-
-  # OUTDATED: currently personal query forms are deactivated in the code
-  # If user is unknown then get only group-wide and system wide reports
-  # else get personal reports in addition  group-wide and system wide.
-
-  $system_scope = 'S';
-
+  $system_scope = 'S'; $group_scope = 'P'; $personal_scope = 'I';
   $sql = "
-    SELECT report_id,name FROM " . ARTIFACT . "_report
-    WHERE (group_id = ? AND scope = 'P') OR scope = ?
-    ORDER BY scope DESC, report_id ASC";
+    SELECT report_id, name, scope FROM " . ARTIFACT . "_report
+    WHERE
+      (group_id = ? AND scope = ?) OR scope = ?";
+  $params = [$group_id, $group_scope, $system_scope];
+  if ($user_id)
+    {
+      $sql .= "\n      OR (user_id = ? AND group_id = ? AND scope = ?)";
+      array_push ($params, $user_id, $group_id, $personal_scope);
+    }
 
-  return db_execute ($sql, [$group_id, $system_scope]);
+  # List system-wide queries first, then group-defined queries,
+  # and at last personal ones.
+  $sql .= "\n    ORDER BY scope DESC, report_id ASC";
+
+  return db_execute ($sql, $params);
+}
+
+function trackers_data_report_is_editable ($row, $uid, $is_admin)
+{
+  $scope = strtoupper ($row['scope']);
+  if ($scope == 'S' && user_is_super_user ())
+    return true;
+  if ($scope == 'I' && $uid == $row['user_id'])
+    return true;
+  return $scope == 'P' && $is_admin;
+}
+
+function trackers_data_get_editable_reports ($group_id, $is_admin)
+{
+  $user_id = user_getid ();
+  $res = db_execute ("
+    SELECT * FROM " . ARTIFACT . "_report WHERE group_id = ? OR user_id = ?
+    ORDER BY report_id", [$group_id, $user_id]
+  );
+  $ret = [];
+  while ($row = db_fetch_array ($res))
+     if (trackers_data_report_is_editable ($row, $user_id, $is_admin))
+       $ret[$row['report_id']] = $row;
+  return $ret;
 }
 
 function trackers_data_get_notification ($user_id)
@@ -2240,14 +2247,14 @@ function trackers_data_get_notification_with_labels ($user_id)
 
 function trackers_data_get_notification_roles ()
 {
-  return db_query (
+  return db_execute (
     'SELECT * FROM trackers_notification_role ORDER BY rank ASC'
   );
 }
 
 function trackers_data_get_notification_events ()
 {
-  return db_query (
+  return db_execute (
     'SELECT * FROM trackers_notification_event ORDER BY rank ASC'
   );
 }
@@ -2284,20 +2291,24 @@ function trackers_data_insert_notification (
   return db_execute ($sql, $sql_params);
 }
 
+function trackers_data_get_watch ($user_id, $idx)
+{
+  $w = trackers_data_get_watched ($user_id)[$idx];
+  $ret = [];
+  foreach ($w as $gid => $per_group)
+    foreach (array_keys ($per_group) as $m)
+      $ret[$m] = $gid;
+  return $ret;
+}
+
 function trackers_data_get_watchers ($user_id)
 {
-  return db_execute ("
-    SELECT user_id, group_id FROM trackers_watcher WHERE watchee_id = ?",
-    [$user_id]
-  );
+  return trackers_data_get_watch ($user_id, 1);
 }
 
 function trackers_data_get_watchees ($user_id)
 {
-  return db_execute ("
-    SELECT watchee_id, group_id FROM trackers_watcher WHERE user_id = ?",
-    [$user_id]
-  );
+  return trackers_data_get_watch ($user_id, 0);
 }
 
 function trackers_data_add_watchees ($user_id, $watchee_id, $group_id)
@@ -2329,15 +2340,46 @@ function trackers_data_delete_watchees ($user_id, $watchee_id, $group_id)
   );
 }
 
+function trackers_data_fetch_watched ($user_id)
+{
+  $watchees = $watchers = [];
+  $result = db_execute ("
+     SELECT user_id, watchee_id, group_id FROM trackers_watcher
+     WHERE user_id = ? OR watchee_id = ?", [$user_id, $user_id]
+  );
+  while ($row = db_fetch_array ($result))
+    {
+      $gid = $row['group_id']; $wid = $row['watchee_id']; $ret = &$watchees;
+      if ($wid === $user_id)
+        {
+          $ret = &$watchers;
+          $wid = $row['user_id'];
+        }
+      elseif ($row['user_id'] !== $user_id)
+        continue;
+      if (array_key_exists ($gid, $ret))
+        $ret[$gid][$wid] = 1;
+      else
+        $ret[$gid] = [$wid => 1];
+    }
+  return [$watchees, $watchers];
+}
+
+function trackers_data_get_watched ($user_id)
+{
+  static $cache = [];
+  if (!array_key_exists ($user_id, $cache))
+    $cache[$user_id] = trackers_data_fetch_watched ($user_id);
+  return $cache[$user_id];
+}
+
 function trackers_data_is_watched ($user_id, $watchee_id, $group_id)
 {
-  $result = db_execute ("
-     SELECT watchee_id FROM trackers_watcher
-     WHERE user_id = ? AND watchee_id = ? AND group_id = ?",
-    [$user_id, $watchee_id, $group_id]
-  );
-  if (db_numrows ($result))
-    return db_result ($result, 0, 'watchee_id');
+  $row = trackers_data_get_watched ($user_id)[0];
+  if (!empty ($row[$group_id]))
+    $row = $row[$group_id];
+  if (!empty ($row[$watchee_id]))
+    return $watchee_id;
   return null;
 }
 
@@ -2380,7 +2422,7 @@ function trackers_unlink_attachment ($file_id)
   return true;
 }
 
-function trackers_delete_attachment ($file_id)
+function trackers_delete_attachment ($file_id, $item_id)
 {
   if (trackers_unlink_attachment ($file_id))
     return;
@@ -2406,7 +2448,7 @@ function trackers_data_delete_file ($group_id, $item_id, $file_id)
     return;
   if (tracker_data_file_out_of_item ($item_id, $file_id))
     return;
-  trackers_delete_attachment ($file_id);
+  trackers_delete_attachment ($file_id, $item_id);
 }
 
 function trackers_data_count_field_value_usage (
