@@ -4,7 +4,7 @@
 # Copyright (C) 1999, 2000 The SourceForge Crew
 # Copyright (C) 2001, 2002 Laurent Julliard, CodeX Team, Xerox
 # Copyright (C) 2000-2006 Mathieu Roy
-# Copyright (C) 2003-2006 Yves Perrin <yves.perrin--cern.ch>
+# Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
 # Copyright (C) 2013, 2014, 2017-2024 Ineiev
@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,16 +41,15 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $dir_name = dirname (__FILE__);
-require_once ("$dir_name/../calendar.php");
-require_once ("$dir_name/../sendmail.php");
-require_once ("$dir_name/data.php");
-require_once ("$dir_name/format.php");
-require_once ("$dir_name/../utils.php");
+foreach (
+  ['../calendar', '../sendmail', 'data', 'format', '../utils', '../group'] as $i
+)
+  require_once ("$dir_name/$i.php");
 
-# Generate URL arguments from a variable wether scalar or array.
+# Generate URL arguments from a variable whether scalar or array.
 function trackers_convert_to_url_arg ($varname, $var)
 {
   if (is_array ($var))
@@ -65,11 +64,13 @@ function trackers_convert_to_url_arg ($varname, $var)
 
 function trackers_check_artifact ($group_id)
 {
+  if ($group_id == GROUP_NONE && user_is_super_user ())
+    return;
   $project = project_get_object ($group_id);
 
   foreach (['bugs', 'support', 'task', 'patch'] as $art)
     if (ARTIFACT == $art && !$project->Uses ($art))
-      exit_error (_("This project has turned off this tracker."));
+      exit_error (_("This group has turned off this tracker."));
 }
 
 function trackers_header ($params, $prefix = '')
@@ -127,17 +128,14 @@ function trackers_list_all_fields ($sort_func = false, $by_field_id = false)
   # list of field names.
   $idx = $by_field_id? 'bug_field_id': 'field_name';
 
-  if (current ($BF_USAGE_BY_ID) !== false)
+  if (current ($BF_USAGE_BY_ID) === false)
     {
-      $field_array = current ($BF_USAGE_BY_ID);
-      next ($BF_USAGE_BY_ID);
-      return $field_array[$idx];
+      trackers_data_rewind_bf_usage ();
+      return false;
     }
-  # Rewind internal pointer for the next time.
-  reset ($BF_USAGE_BY_ID);
-  reset ($BF_USAGE_BY_NAME);
-  $AT_START = true;
-  return false;
+  $field_array = current ($BF_USAGE_BY_ID);
+  next ($BF_USAGE_BY_ID);
+  return $field_array[$idx];
 }
 
 function trackers_field_label_display (
@@ -169,7 +167,7 @@ function trackers_field_label_display (
 # - field_name: name of the column.
 # - group_id: the group id.
 # - value: the current value stored in this field (for select boxes type of field
-#     it is the value_id actually. It can also be an array with mutliple values.
+#     it is the value_id actually. It can also be an array with multiple values.
 # - break: true if a break line is to be inserted between the field label
 #     and the field value.
 # - label: if true display the field label.
@@ -400,7 +398,7 @@ function trackers_field_textarea (
     . "\" rows=\"$rows\" cols=\"$cols\" wrap='soft'>$value</textarea>";
 }
 
-# Return a select box populated with field values for this project.
+# Return a select box populated with field values for this group.
 # If box_name is given, then impose this name in the select box
 # of the  HTML form otherwise use the field_name.
 function trackers_field_box (
@@ -494,7 +492,7 @@ function trackers_field_box (
   );
 }
 
-# Return a multiple select box populated with field values for this project.
+# Return a multiple select box populated with field values for this group.
 # If box_name is given then impose this name in the select box
 # of the  HTML form otherwise use the field_name.
 function trackers_multiple_field_box (
@@ -516,8 +514,8 @@ function trackers_multiple_field_box (
   );
 }
 
-# Returns the list of field names in the HTML Form corresponding to a
-# field used by this project
+# Return the list of field names in the HTML Form corresponding
+# to a field used by this group.
 function trackers_extract_field_list ($post_method = true)
 {
   global $BF_USAGE_BY_NAME;
@@ -559,7 +557,6 @@ function trackers_extract_field_list ($post_method = true)
         }
       $k = print_r ($key, true);
       $v = print_r ($val, true);
-      dbg ("Rejected key = $k val = $v");
     }
   return $vfl;
 }
@@ -735,9 +732,8 @@ function trackers_trim_email ($email)
 function trackers_get_cc_list ($artifact, $item_id)
 {
   return db_execute ("
-    SELECT email, added_by FROM {$artifact}_cc
-    WHERE bug_id = ? GROUP BY email LIMIT 150",
-    [$item_id]
+    SELECT email, min(added_by) AS added_by FROM {$artifact}_cc
+    WHERE bug_id = ? GROUP BY email LIMIT 150", [$item_id]
   );
 }
 
@@ -866,7 +862,7 @@ function trackers_build_notification_list ($item, $changes, $artifact)
       $added_by = $row['added_by'];
       $email = trackers_trim_email ($row['email']);
       $skip_it = trackers_filter_notif_address (
-        $added_by,  $email, $changes, $addreses, $addresses_to_skip
+        $added_by,  $email, $changes, $addresses, $addresses_to_skip
       );
       if (!$skip_it)
         $addresses[$email] = true;
@@ -906,16 +902,17 @@ function trackers_build_mail ($artifact, $res, $item_id, $changes)
   global $int_delayspamcheck_comment_id;
   # Text of the mail must not be localized.
   $bug_ref = trackers_mail_bug_ref ($artifact, $item_id);
+  $msg = [];
   if ($changes)
-    $body = format_item_changes ($changes, $item_id, $res['group_id']) . "\n";
+    $msg['body'] = format_item_changes ($changes, $item_id, $res) . "\n";
   else
-    $body = format_item_summary ($res, $bug_ref, $artifact);
-  $body .= format_message_trailer ($bug_ref);
-  $subject = utils_specialchars_decode ($res['summary'], ENT_QUOTES);
+    $msg['body'] = format_item_summary ($res, $bug_ref, $artifact);
+  $msg['bug_ref'] = $bug_ref;
+  $msg['subject'] = utils_specialchars_decode ($res['summary'], ENT_QUOTES);
   # Necessary to mention the comment id (for delayed mails).
   if ($int_delayspamcheck_comment_id)
     $item_id .= ":$int_delayspamcheck_comment_id";
-  return [$body, $subject, $item_id];
+  return [$msg, $item_id];
 }
 
 function trackers_exclude_list ($artifact, $group_id, $force_exclude, $privacy)
@@ -951,6 +948,11 @@ function trackers_followup_mail_addresses (
   return [$from, $to, $exclude];
 }
 
+function trackers_item_is_public ($privacy, $group_id)
+{
+  return $privacy != 2 && group_get_object ($group_id)->isPublic ();
+}
+
 function trackers_reported_subject ($subject)
 {
   global $sys_new_user_watch_days, $sys_watch_anon_posts;
@@ -974,6 +976,8 @@ function trackers_reported_subject ($subject)
 function trackers_send_followup ($addresses, $message, $context)
 {
   global $sys_mail_admin, $sys_mail_domain;
+  if (empty ($context['group']))
+    $context['group'] = group_getunixname ($context['group_id']);
   sendmail_mail ($addresses, $message, $context);
   $addresses['to'] = "$sys_mail_admin@$sys_mail_domain";
   $message['subject'] = trackers_reported_subject ($message['subject']);
@@ -982,43 +986,58 @@ function trackers_send_followup ($addresses, $message, $context)
   sendmail_mail ($addresses, $message, $context);
 }
 
-function trackers_mail_followup (
-  $item_id, $more_addresses = '', $changes = false,
-  $exclude_list = false, $artifact = null
+function trackers_append_followup_notif_addresses (
+  &$addresses, $item_id, $updated = true, $tracker = null
 )
 {
-  $res = trackers_mail_fetch_followup ($artifact, $item_id);
+  $tracker = $tracker !== null? $tracker: ARTIFACT;
+  $additional_address =
+    trackers_data_get_item_notification_info ($item_id, $tracker, $updated);
+  if ($additional_address !== '' && trim ($addresses) !== '')
+    $addresses .= ', ';
+  $addresses .= $additional_address;
+}
+
+function trackers_mail_followup (
+  $item_id, $addresses = '', $changes = false,
+  $exclude_list = false, $tracker = null
+)
+{
+  $tracker = $tracker !== null? $tracker: ARTIFACT;
+  $res = trackers_mail_fetch_followup ($tracker, $item_id);
   if ($res === false)
     return;
 
-  list ($body, $subject, $item_id) =
-    trackers_build_mail ($artifact, $res, $item_id, $changes);
+  list ($msg, $item) =
+    trackers_build_mail ($tracker, $res, $item_id, $changes);
   list ($from, $to, $exclude) =
     trackers_followup_mail_addresses (
-      $res, $more_addresses, $exclude_list, $artifact, $changes
+      $res, $addresses, $exclude_list, $tracker, $changes
     );
-  $group = group_getunixname ($res['group_id']);
   trackers_send_followup (
-    ['from' => $from, 'to' => $to, 'exclude' => $exclude],
-    ['subject' => $subject, 'body' => $body],
-    ['group' => $group, 'tracker' => $artifact, 'item' => $item_id]
+    ['from' => $from, 'to' => $to, 'exclude' => $exclude], $msg,
+    ['group_id' => $res['group_id'], 'tracker' => $tracker, 'item' => $item]
   );
 }
 
 function trackers_add_file ($item_id, $file, $file_description, &$changes)
 {
   if (empty ($file['tmp_name']))
-    return '';
-  if ($file['error'] != UPLOAD_ERR_OK)
-    return '';
+    return null;
+  if (empty ($file['emailed']) && $file['error'] != UPLOAD_ERR_OK)
+    return null;
 
   $id = trackers_attach_file ($item_id, $file, $file_description, $changes);
   if (!$id)
-    return '';
+    {
+      if (is_file ($file['tmp_name']))
+        unlink ($file['tmp_name']);
+      return null;
+    }
   $changes['attach'][] = [
     'name' => $file['name'], 'size' => $file['size'], 'id' => $id
   ];
-  return "file #$id, ";
+  return "file #$id";
 }
 
 # Wrapper for trackers_attach_file that will find out if one or more files
@@ -1027,167 +1046,182 @@ function trackers_attach_several_files ($item_id, $group_id, &$changes)
 {
   # Reset the global used to count the current upload size.
   $GLOBALS['current_upload_size'] = 0;
-
-  $changed = false;
-  $comment = '';
-
-  $filenames = [];
+  $comment = $filenames = [];
   for ($i = 1; $i < 5; $i++)
     $filenames[] = "input_file$i";
   $files = sane_import ('files', ['pass' => $filenames]);
   extract (sane_import ('post', ['specialchars' => 'file_description']));
   foreach ($files as $file)
-    $comment .= trackers_add_file (
+    $comment[] = trackers_add_file (
       $item_id, $file, $file_description, $changes
     );
-
-  if ($comment)
-    {
-      $changed = true;
-      $comment = "\n\n(" . rtrim ($comment, ", ") . ")";
-    }
-
-  # Trash the used global.
   unset ($GLOBALS['current_upload_size']);
-  return [$changed, $comment];
+  $comment = array_filter ($comment);
+  if (empty ($comment))
+    return [false, ''];
+  return [true, "\n\n(" . join (', ', $comment) . ")"];
 }
 
-function trackers_attach_file ($item_id, $file, $file_description, &$changes)
+function trackers_upload_size_invalid ($file_name)
+{
+  if ($GLOBALS['current_upload_size'] >= 0)
+    return false;
+  # TRANSLATORS: the argument is file name.
+  $msg = sprintf (
+    _("Unexpected error, disregarding file %s attachment"), $file_name
+  );
+  fb ($msg, 1);
+  return true;
+}
+
+function trackers_upload_size_notice ()
+{
+  $size = $GLOBALS['current_upload_size'];
+  if (!$size)
+    return '';
+  # Explanation added when an upload is refused,
+  # if the upload count is involved.
+  return ' ' . sprintf (
+    ngettext (
+      "You already uploaded %s kilobyte.", "You already uploaded %s kilobytes.",
+      $size),
+    $size
+  );
+}
+
+function trackers_attachments_dir_unaccessible ()
 {
   global $sys_trackers_attachments_dir;
-  $input_file = $file['tmp_name'];
-  $input_file_name = preg_replace ('/[&<\s"\';?!*]/', '@', $file['name']);
-  $input_file_type = $file['type'];
-  $input_file_size = $file['size'];
-  $user_id = (user_isloggedin ()? user_getid (): 100);
-
-  if (!is_writable ($sys_trackers_attachments_dir))
-    {
-      $msg = sprintf (
-        _("The upload directory '%s' is not writable."),
-        $sys_trackers_attachments_dir
-      );
-      fb ($msg, 1);
-      return false;
-    }
-  if (!is_uploaded_file ($input_file))
-    {
-      $msg = sprintf (
-        _("File %s not attached: unable to open it"),
-        $input_file_name
-      );
-      fb ($msg, 1);
-      return false;
-    }
-
-  # Found of the previous upload count.
-  # It could not be inferior to 0. If it is, someone obviously find a way
-  # to tamper, ignore the file.
-  $current_upload_size = $GLOBALS['current_upload_size'];
-  $current_upload_size_comment = '';
-  if ($current_upload_size < 0)
-    {
-      # TRANSLATORS: the argument is file name.
-      fb (
-        sprintf (
-          _("Unexpected error, disregarding file %s attachment"),
-          $input_file_name),
-        1
-      );
-      return false;
-    }
-  if ($current_upload_size > 0)
-    {
-      # Explanation added when an upload is refused, if the upload count
-      # is involved.
-      $current_upload_size_comment = ' '
-        . sprintf (
-            ngettext (
-              "You already uploaded %s kilobyte.",
-              "You already uploaded %s kilobytes.",
-              $current_upload_size),
-            $current_upload_size
-          );
-    }
-
-  # Check file size.
-  # Note: in english, use the expression kilobytes, and not kB, because
-  # feedback is in lowercase for the whole string.
-  # We always add the current upload count.
-  $filesize = round (filesize ($input_file) / 1024);
-  $uploadsize = $filesize + $current_upload_size;
-  if ($uploadsize > $GLOBALS['sys_upload_max'])
-    {
-      $msg_not_attached =
-        sprintf (ngettext (
-          "File %s not attached: its size is %s kilobyte.",
-          "File %s not attached: its size is %s kilobytes.",
-          $filesize), $input_file_name, $filesize
-        );
-      $msg_max_size =
-        sprintf (ngettext (
-          "Maximum allowed file size is %s kilobyte,\n"
-          . "after escaping characters as required.",
-          "Maximum allowed file size is %s kilobytes,\n"
-          . "after escaping characters as required.",
-          $GLOBALS['sys_upload_max']),
-          $GLOBALS['sys_upload_max']
-        );
-      fb ("$msg_not_attached $msg_max_size$current_upload_size_comment", 1);
-      return false;
-    }
-  if (filesize ($input_file) == 0)
-    {
-      fb (
-        sprintf (_("File %s is empty."), $input_file_name)
-        . $current_upload_size_comment,
-        1
-      );
-      return false;
-    }
-
-  # Update the upload count value (before the actual database insert, safer).
-  $GLOBALS['current_upload_size'] = $uploadsize;
-
-  $res = db_autoexecute (
-    'trackers_file',
-    [
-      'item_id' => $item_id, 'artifact' => ARTIFACT,
-      'submitted_by' => $user_id, 'date' => time (),
-      'description' => $file_description, 'filename' => $input_file_name,
-      'filesize' => $input_file_size, 'filetype' => $input_file_type
-    ],
-    DB_AUTOQUERY_INSERT
+  if (is_writable ($sys_trackers_attachments_dir))
+    return false;
+  $msg = sprintf (
+    _("The upload directory '%s' is not writable."),
+    $sys_trackers_attachments_dir
   );
+  fb ($msg, 1);
+  return true;
+}
 
-  if (!$res)
-    {
-      fb (sprintf (_("Error while attaching file %s"), $input_file_name), 1);
-      return false;
-    }
-  $file_id = db_insertid ($res);
+function trackers_not_uploaded_file ($input_file, $file_name)
+{
+  if (is_uploaded_file ($input_file))
+    return false;
+  $msg = sprintf (
+    _("File %s not attached: unable to open it"), $file_name
+  );
+  fb ($msg, 1);
+  return true;
+}
+
+function trackers_get_upload_size ($file)
+{
+  $filesize = round (filesize ($file) / 1024);
+  return $filesize + $GLOBALS['current_upload_size'];
+}
+
+function trackers_file_too_big ($file, $file_name, $comment)
+{
+  $filesize = round (filesize ($file) / 1024);
+  if (trackers_get_upload_size ($file) <= $GLOBALS['sys_upload_max'])
+    return false;
+  $msg_not_attached = sprintf (ngettext (
+    "File %s not attached: its size is %s kilobyte.",
+    "File %s not attached: its size is %s kilobytes.",
+    $filesize), $file_name, $filesize
+  );
+  $msg_max_size = sprintf (ngettext (
+    "Maximum allowed file size is %s kilobyte,\n"
+    . "after escaping characters as required.",
+    "Maximum allowed file size is %s kilobytes,\n"
+    . "after escaping characters as required.",
+    $GLOBALS['sys_upload_max']), $GLOBALS['sys_upload_max']
+  );
+  fb ("$msg_not_attached $msg_max_size$comment", 1);
+  return true;
+}
+
+function trackers_file_is_empty ($file, $file_name, $size_comment)
+{
+  clearstatcache (true, $file);
+  if (filesize ($file))
+    return false;
+  fb (sprintf (_("File %s is empty."), $file_name) . $size_comment, 1);
+  return true;
+}
+
+function trackers_check_upload_size ($file, $file_name)
+{
+  if (trackers_upload_size_invalid ($file_name))
+    return true;
+  $size_comment = trackers_upload_size_notice ();
+  if (trackers_file_is_empty ($file, $file_name, $size_comment))
+    return true;
+  return trackers_file_too_big ($file, $file_name, $size_comment);
+}
+
+function trackers_check_upload_params ($file, $file_name)
+{
+  if (trackers_attachments_dir_unaccessible ())
+    return true;
   if (
-    !move_uploaded_file ($input_file, "$sys_trackers_attachments_dir/$file_id")
+    empty ($file['emailed'])
+    && trackers_not_uploaded_file ($file['tmp_name'], $file_name)
   )
-    {
-      fb (
-        sprintf (_("Error while saving file %s on disk"), $input_file_name),
-        1
-      );
-      return false;
-    }
+    return true;
+  return trackers_check_upload_size ($file['tmp_name'], $file_name);
+}
 
-  $file_id = db_insertid ($res);
+function trackers_insert_attachment ($params)
+{
+  $params['artifact'] = ARTIFACT;
+  $params['date'] = time ();
+  $params['filetype'] = $params['file']['type'];
+  $params['filesize'] = $params['file']['size'];
+  unset ($params['file']);
+  $res = db_autoexecute ('trackers_file', $params, DB_AUTOQUERY_INSERT);
+  if ($res)
+    return db_insertid ($res);
+  fb (sprintf (_("Error while attaching file %s"), $file_name), 1);
+  return false;
+}
+
+function trackers_move_attachment ($file, $file_id, $file_name)
+{
+  global $sys_trackers_attachments_dir;
+  if (rename ($file, "$sys_trackers_attachments_dir/$file_id"))
+    return false;
+  fb (sprintf (_("Error while saving file %s on disk"), $file_name), 1);
+  return true;
+}
+
+function trackers_post_attach ($item_id, $file_id, $file_name)
+{
   # TRANSLATORS: the argument is file id (a number).
   fb (sprintf (_("file #%s attached"), $file_id));
-
   trackers_data_add_history (
-    "Attached File", "-", "Added $input_file_name, #$file_id", $item_id,
-    0, 0, 1
+    "Attached File", "-", "Added $file_name, #$file_id", $item_id, 0, 0, 1
   );
-  # Add the guy in CC.
   if (user_isloggedin () && !user_get_preference ("skipcc_updateitem"))
     trackers_add_cc ($item_id, user_getid (), "-UPD-");
+}
+
+function trackers_attach_file ($item_id, $file, $description, &$changes)
+{
+  global $current_upload_size;
+  $file_name = preg_replace ('/[&<\s"\';?!*]/', '@', $file['name']);
+  $user_id = (user_isloggedin ()? user_getid (): 100);
+  if (trackers_check_upload_params ($file, $file_name))
+    return false;
+  $current_upload_size = trackers_get_upload_size ($file['tmp_name']);
+  $file_id = trackers_insert_attachment ([
+    'item_id' => $item_id, 'submitted_by' => $user_id,
+    'description' => $description, 'filename' => $file_name, 'file' => $file
+  ]);
+  if (!$file_id)
+    return false;
+  if (trackers_move_attachment ($file['tmp_name'], $file_id, $file_name))
+    return false;
+  trackers_post_attach ($item_id, $file_id, $file_name);
   return $file_id;
 }
 
@@ -1281,8 +1315,7 @@ function trackers_delete_cc (
       # - the CC name is the current user
       # - the CC email address matches the one of the current user
       # - the current user is the person who added a given name in CC list
-      $mem_f = member_create_tracker_flag (ARTIFACT) . '2';
-      if (!member_check (0, $group_id, $mem_f))
+      if (!member_check (0, $group_id, 2))
         if (
           $user_id != $email && $user_id != $added_by
           && user_getname ($user_id) != $email
@@ -1343,7 +1376,7 @@ function trackers_delete_dependency (
   # But well, consequences would be small an it will be easy to identify
   # the criminal.
 
-  if (member_check (0, $group_id, member_create_tracker_flag (ARTIFACT) . '1'))
+  if (member_check (0, $group_id, 1))
     $result = db_execute ("
       DELETE FROM " . ARTIFACT . "_dependencies
       WHERE
@@ -1389,9 +1422,8 @@ function trackers_isvarany ($var)
   return false;
 }
 
-# Check is a sort criteria is already in the list of comma
-# separated criterias. If so invert the sort order, if not then
-# simply add it.
+# Check is a sort criteria is already in the list of comma-separated
+# criteria. If so, invert the sort order, else simply add it.
 function trackers_add_sort_criteria ($criteria_list, $order, $msort)
 {
   $found = false;
@@ -1455,7 +1487,7 @@ function trackers_criteria_list_to_query ($criteria_list)
   # Undo the uid->user_name trick to avoid "Column 'submitted_by' in
   # order clause is ambiguous" error. This is pretty ugly. Also check
   # trackers_data_is_username_field().
-  $criteria_list = str_replace(
+  $criteria_list = str_replace (
     'submitted_by ', 'user_submitted_by.user_name ', $criteria_list
   );
   $criteria_list = str_replace (
@@ -1678,4 +1710,39 @@ function trackers_get_msgid ($artifact, $item_id)
     $msg_ids[] = "<{$row['msg_id']}>";
   return $msg_ids;
 }
+
+function trackers_fetch_item_access_data ($item_id, $user_id)
+{
+  $fields = utils_find_item (
+    ARTIFACT, $item_id, ['privacy', 'discussion_lock', 'submitted_by'],
+    function ($x) { return null; }
+  );
+  if ($fields === null)
+    return null;
+
+  $group = project_get_object ($fields['group_id']);
+  if ($group->isError ())
+    exit_no_group ();
+
+  $fields['is_trackeradmin'] = member_check ($user_id, $fields['group_id'], 2);
+  return $fields;
+}
+
+function trackers_may_user_comment ($user_id, $item_id)
+{
+  $fields = trackers_fetch_item_access_data ($item_id, $user_id);
+  if (empty ($fields))
+    return [false, $fields];
+  if (
+    $fields['privacy'] == 2 && !member_check ($user_id, $fields['group_id'])
+    && $fields['submitted_by'] != $user_id
+  )
+    return [false, $fields];
+  if ($fields['discussion_lock'] && !$fields['is_trackeradmin'])
+    return [false, $fields];
+  $ret = group_restrictions_check (
+    $fields['group_id'], ARTIFACT, TRACKER_EVENT_COMMENT
+  );
+  return [$ret, $fields];
+}
 ?>
