@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/cookbook.php');
 require_once (dirname (__FILE__) . '/../utils.php');
@@ -332,7 +332,7 @@ function show_item_details (
   );
   if ($item_no < 1)
     return $text;
-  $ctl = form_submit (_('Reverse comment order'), 'revert_bis');
+  $ctl = form_submit (_('Reverse comment order'), 'reverse_bis');
   if ($item_no > 5 && !$comment_order)
     {
       $jumpto_text = _("Jump to the original submission");
@@ -346,9 +346,9 @@ function show_item_details (
   return "<p>$ctl</p>\n$text";
 }
 
-function show_item_attached_files ($item_id, $group_id, $ascii = false)
+function show_item_attached_files ($item_id, $group_id, $public)
 {
-  print format_item_attached_files ($item_id, $group_id, $ascii);
+  print format_item_attached_files ($item_id, $group_id, false, $public);
 }
 
 function show_item_cc_list ($item_id, $group_id)
@@ -369,9 +369,7 @@ function show_dependent_item ($item_id, $dependson = 0)
   global $group_id;
 
   $artifacts = ["support", "bugs", "task", "patch"];
-  $is_manager = member_check (
-    0, $group_id, member_create_tracker_flag (ARTIFACT) . '1'
-  );
+  $is_manager = member_check (0, $group_id, 1);
   if (!$dependson)
     $title = _("Items that depend on this one");
   else
@@ -510,9 +508,7 @@ function show_dependent_item ($item_id, $dependson = 0)
       # rights, being project member is enough).
       if (!array_key_exists ($current_group_id, $allowed_to_see))
         $allowed_to_see[$current_group_id] =
-          member_check (
-            0, $current_group_id, member_create_tracker_flag (ARTIFACT) . '2'
-          );
+          member_check (0, $current_group_id, 2);
 
       if ($content[$key]['privacy'] == "2"
           && !$allowed_to_see[$current_group_id]
