@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 extract (sane_import ('request',
   ['array' => [['prefill', [null, 'specialchars']]]]
@@ -68,9 +68,9 @@ $res_preamble = db_execute (
 
 $preamble = db_result ($res_preamble, 0, $grp_field);
 if ($preamble)
-  print '<h2>' . _("Preamble") . "</h2>\n" . markup_rich ($preamble);
+  print html_h (2, _("Preamble")) . markup_rich ($preamble);
 
-print '<h2>' . _("Details") . "</h2>\n";
+print html_h (2, _("Details"));
 
 # Beginning of the submission form with fixed fields.
 print form_header (
@@ -82,8 +82,7 @@ print "\n<table cellpadding='0' width='100%'>";
 # Now display the variable part of the field list (depending on the project).
 $i = 0;
 $j = 0;
-$is_trackeradmin =
-  member_check (0, $group_id, member_create_tracker_flag (ARTIFACT) . '2');
+$is_trackeradmin = member_check (0, $group_id, 2);
 
 while ($field_name = trackers_list_all_fields ())
   {
@@ -198,14 +197,14 @@ print
 
 if ($preview)
   {
-    print '<h2>' . _('Preview') . "</h2>\n\n";
+    print html_h (2, _('Preview'));
     if (isset ($details))
       print markup_full (utils_specialchars ($details));
   }
 
 
 print "<p>&nbsp;</p>\n";
-print '<h2>' . _("Attached Files") . "</h2>\n";
+print html_h (2, _("Attached Files"));
 printf (
   _("(Note: upload size limit is set to %s kB, after insertion of\n"
     . "the required escape characters.)"),
@@ -232,7 +231,7 @@ print '<span class="preinput">' . _("Comment:") . "</span><br />\n"
 if ($is_trackeradmin)
   {
     print "<p>&nbsp;</p>\n";
-    print '<h2>' . _("Mail Notification CC") . "</h2>\n";
+    print html_h (2, _("Mail Notification CC"));
 
     # TRANSLATORS: the argument is site name (like Savannah).
     print '<p>';
