@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 $dirname = dirname (__FILE__);
 require_once ("$dirname/../trackers/show.php");
@@ -88,10 +88,11 @@ if ($preview)
 $item_name = utils_get_tracker_prefix (ARTIFACT) . " #$item_id";
 $item_link = utils_link ("?$item_id", $item_name);
 
+$privacy = $res_arr['privacy'];
 # Check whether this item is private or not. If it is private, show only to
 # the submitter or to people that got the right to see private items
 $private_intro = '';
-if ($res_arr['privacy'] == "2")
+if ($privacy == "2")
   {
     if (member_check_private (0, $group_id))
       {
@@ -121,20 +122,18 @@ trackers_header (
 );
 
 # Check if the user have a specific role.
-$check_member = function ($role)
+function check_member ($role)
 {
-  global $group_id;
-  $flag = member_create_tracker_flag (ARTIFACT) . $role;
-  return member_check (0, $group_id, $flag);
-};
+  return member_check (0, $GLOBALS['group_id'], $role, true);
+}
 $member_help = function ($title, $arr)
 {
   print '<p>';
   print help ($title, $arr);
   print "</p>\n";
 };
-$is_manager = $check_member ('3');
-if ($check_member ('2'))
+$is_manager = member_check (0, $group_id, 3);
+if (check_member (2))
   $member_help (
     _("You are both technician and manager for this tracker."),
     [
@@ -143,7 +142,7 @@ if ($check_member ('2'))
      _("manager") => _("fully manage the items"),
     ]
   );
-elseif ($check_member ('1'))
+elseif (check_member (1))
   $member_help (
     _("You are technician for this tracker."),
     [
@@ -152,7 +151,7 @@ elseif ($check_member ('1'))
           . "items, change priority, open nor close")
     ]
   );
-elseif ($is_manager)
+elseif (check_member (3))
   $member_help (
     _("You are manager for this tracker."),
     [
@@ -162,7 +161,6 @@ elseif ($is_manager)
           . "groups, changing priority, opening and closing items")
     ]
   );
-unset ($check_member);
 
 if (!empty ($private_intro))
   print "<p>$private_intro</p>\n";
@@ -170,19 +168,18 @@ if (!empty ($private_intro))
 $class = utils_get_priority_color (
   $res_arr['priority'], $res_arr['status_id']
 );
+$caption = "<i>$item_link</i>: {$res_arr['summary']}";
+print html_h (1, $caption, ['class' => $class]);
 
-print "<h1 class=\"$class\"><i>$item_link</i>: ";
-print $res_arr['summary'] . "</h1>\n";
+print form_header (
+  null, "post", 'enctype="multipart/form-data" name="item_form"'
+);
+$hidden = [
+  "group_id" => $group_id, "item_id" => $item_id,
+  'func' => $enable_comments? 'postmoditem': 'detailitem'
+];
+print form_hidden ($hidden);
 
-if ($enable_comments)
-  {
-    print form_header (
-      null, "post", 'enctype="multipart/form-data" name="item_form"'
-    );
-    print form_hidden (
-      ["func" => "postmoditem", "group_id" => $group_id, "item_id" => $item_id]
-    );
-  }
 # Colspan explanation:
 #
 #  We want the following, twice much space for the value than for the label:
@@ -516,8 +513,8 @@ if ($item_discussion_lock)
     print "</p>\n";
   }
 print html_hidsubpart_header ("discussion", _("Discussion"));
-if ($revert_comment_order)
-  print form_hidden (['revert_order' => 1]);
+if ($reverse_comment_order)
+  print form_hidden (['reverse_order' => 1]);
 
 $new_comment = [];
 if ($preview)
@@ -585,7 +582,8 @@ if ($enable_comments)
 
 print "<p>";
 
-show_item_attached_files ($item_id, $group_id);
+$public = trackers_item_is_public ($privacy, $group_id);
+show_item_attached_files ($item_id, $group_id, $public);
 
 print "</p>\n<p>&nbsp;</p>\n";
 print html_hidsubpart_footer ();
@@ -627,7 +625,7 @@ if ($is_trackeradmin)
         form_option ($option_value, $depends_search_only_artifact, $text);
     $tracker_select .= "</select>\n";
 
-    $group_select = '<select title="' . _("Wether to search in any group")
+    $group_select = '<select title="' . _("Whether to search in any group")
       . '" name="depends_search_only_group">';
 
     # By default, search restricted to the group (lighter for the CPU,
@@ -929,8 +927,9 @@ if ($enable_comments)
       . form_submit (_("Submit changes and browse items"), "submit", 'class="bold"')
       . ' '
       . form_submit (_("Submit changes and return to this item"), "submitreturn")
-      . "</div>\n</form>\n";
+      . "</div>\n";
   }
+print "</form>\n";
 print html_hidsubpart_header ("history", _("History"));
 show_item_history ($item_id, $group_id);
 print html_hidsubpart_footer ();
