@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,13 +39,23 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/member.php');
+define ('USER_STATUS_ACTIVE', 'A');
+define ('USER_STATUS_PENDING', 'P');
+define ('USER_STATUS_SQUAD', MEMBER_FLAGS_SQUAD);
+define ('USER_STATUS_SUSPENDED', 'S');
+define ('USER_STATUS_DELETED', 'D');
 
-# Unset these globals until register_globals if off everywhere.
-unset ($USER_IS_SUPER_USER);
-$USER_RES = [];
+# User records in this array can be accessed by user_id as well as by user_name.
+# the user_name keys are in the lower case, user names are case-insensitive.
+$USER_ARR = [];
+
+function user_status_is_removed ($status)
+{
+  return $status == USER_STATUS_SUSPENDED || $status == USER_STATUS_DELETED;
+}
 
 function user_isloggedin ()
 {
@@ -53,19 +63,23 @@ function user_isloggedin ()
   return !empty ($G_USER['user_id']);
 }
 
-function user_can_be_super_user ()
+function user_can_be_super_user ($user_id = 0)
 {
-  global $USER_IS_SUPER_USER;
-  if (isset ($USER_IS_SUPER_USER))
+  static $USER_IS_SUPER_USER = null;
+  if ($USER_IS_SUPER_USER !== null)
     return $USER_IS_SUPER_USER;
   $USER_IS_SUPER_USER = false;
-  if (!user_isloggedin ())
-    return false;
+  if (empty ($user_id))
+    {
+      if (!user_isloggedin ())
+        return false;
+      $user_id = user_getid ();
+    }
   # Admins of sys_group_id are admins and have superuser privs site-wide.
   $result = db_execute ("
     SELECT * FROM user_group
     WHERE user_id = ? AND group_id = ? AND admin_flags = 'A'",
-    [user_getid (), $GLOBALS['sys_group_id']]
+    [$user_id, $GLOBALS['sys_group_id']]
   );
   $USER_IS_SUPER_USER = (db_numrows ($result) >= 1);
   return $USER_IS_SUPER_USER;
@@ -90,7 +104,7 @@ function user_check_ismember ($user_id, $group_id, $type = 0)
   return member_check ($user_id, $group_id, $type);
 }
 
-# Get the groups to which a user belongs.
+# Get the ids of the groups the user belongs to.
 function user_groups ($uid)
 {
   $result = db_execute ("SELECT * FROM user_group WHERE user_id = ", [$uid]);
@@ -100,61 +114,60 @@ function user_groups ($uid)
   return $arr;
 }
 
+# Return arrays of ids and names of the groups the user belogs to.
+function user_group_names ($uid)
+{
+  $result = db_execute ("
+    SELECT g.group_id, g.unix_group_name, g.group_name
+    FROM `groups` g JOIN user_group u ON g.group_id = u.group_id
+    WHERE u.user_id = ? AND g.status = 'A' ORDER BY g.unix_group_name", [$uid]
+  );
+  $ids = $names = [];
+  while ($row = db_fetch_array ($result))
+    {
+      $uname = $row['unix_group_name'];
+      $names[$uname] = $row['group_name'];
+      $ids[$uname] = $row['group_id'];
+    }
+  return [$ids, $names];
+}
+
 # Get the email of a user.
 function user_get_email ($uid)
 {
-  $result = db_execute ("SELECT * FROM user WHERE user_id = ?", [$uid]);
-  $val = db_fetch_array ($result);
-  return $val['email'];
+  return user_get_field ($uid, 'email');
 }
 
-function user_getname ($user_id = 0, $getrealname = 0)
+function user_getrealname ($user_id = 0, $rfc822_compliant = 0)
 {
-  global $G_USER, $USER_NAMES;
-
-  # Use current user if one is not passed in.
-  if (!$user_id && $getrealname)
-    $user_id = user_getid ();
-
-  $prefix = 'realname';
-  $column = 'realname';
-  if (!$getrealname)
+  $uids = user_get_uids_array ($user_id);
+  $ret = user_get_field ($uids, 'realname');
+  if ($rfc822_compliant)
+    $ret = array_map ('utils_comply_with_rfc822', $ret);
+  foreach ($uids as $u)
     {
-      $prefix = 'user';
-      $column = 'user_name';
+      if (array_key_exists ($u, $ret))
+        continue;
+      if (empty ($u))
+        $ret[$u] = _("anonymous");
+      else
+        $ret[$u] = '<b>' . _("Invalid User ID") . '</b>';
     }
-  if (!$user_id)
-   {
-      if ($getrealname)
-        return _("anonymous");
-      if ($G_USER)
-        return $G_USER['user_name'];
-      # TRANSLATORS: "Not applicable".
-      return _("NA");
-   }
-
-  # Lookup name.
-  if (!empty ($USER_NAMES["{$prefix}_$user_id"]))
-    return $USER_NAMES["{$prefix}_$user_id"];
-  # Fetch the user name and store it for future reference.
-  $result = db_execute ("
-    SELECT user_id, user_name, realname FROM user WHERE user_id = ?",
-    [$user_id]
-  );
-  if (db_numrows ($result))
-    {
-      # Valid user - store and return.
-      $USER_NAMES["{$prefix}_$user_id"] = db_result ($result, 0, $column);
-      return $USER_NAMES["{$prefix}_$user_id"];
-    }
-  $uid = "#$user_id";
-  if ($getrealname)
-    $uid = _("Invalid User ID");
-  $USER_NAMES["{$prefix}_$user_id"] = "<strong>$uid</strong>";
-  return $USER_NAMES["{$prefix}_$user_id"];
+  return utils_return_val ($user_id, $ret);
 }
 
-function user_getid  ($username = 0)
+function user_getname ($user_id = 0)
+{
+  $uids = user_get_uids_array ($user_id);
+  $ret = user_get_field ($uids, 'user_name');
+  foreach ($uids as $u)
+    if (!array_key_exists ($u, $ret))
+      # TRANSLATORS: "Not applicable".
+      $ret[$u] = empty ($u)? _("NA"): "<b>#$u</b>";
+  return utils_return_val ($user_id, $ret);
+}
+
+function user_getid ($username = 0)
 {
   if (!$username)
     {
@@ -164,27 +177,26 @@ function user_getid  ($username = 0)
         return $G_USER['user_id'];
       return 0;
     }
-  $result = db_execute (
-    "SELECT user_id FROM user WHERE user_name = ?", [$username]
-  );
-  if (db_numrows ($result) > 0)
-    return db_result ($result, 0, "user_id");
-  return 0;
+  $user = user_get_array ($username);
+  if (empty ($user))
+    return 0;
+  return $user['user_id'];
 }
 
 function user_squad_exists ($user_id)
 {
-  return $user_id && user_get_field ($user_id, 'status') == 'SQD';
+  return $user_id && user_get_field ($user_id, 'status') == USER_STATUS_SQUAD;
 }
 
 function user_exists ($user_id)
 {
-  return $user_id && !empty (user_get_field ($user_id, 'user_id'));
+  $a = user_get_field ($user_id, 'user_id');
+  return $user_id && !empty ($a);
 }
 
 function user_is_active ($user_id)
 {
-  return user_get_field ($user_id, 'status') == 'A';
+  return user_get_field ($user_id, 'status') == USER_STATUS_ACTIVE;
 }
 
 function user_fetch_name ($user_id)
@@ -193,51 +205,144 @@ function user_fetch_name ($user_id)
   return ($name === null)? '': $name;
 }
 
-function user_getrealname ($user_id = 0, $rfc822_compliant = 0)
-{
-  $ret = user_getname($user_id, 1);
-  if ($rfc822_compliant)
-    return utils_comply_with_rfc822 ($ret);
-  return $ret;
-}
-
-function user_getemail($user_id=0)
+function user_getemail ($user_id = 0)
 {
   return user_get_field ($user_id, 'email');
 }
 
-# Fetch a row from user table by user_id unless already cached,
-# put it to $USER_RES[$user_id].
-function user_get_result_set ($user_id)
+function user_get_uids_array ($u)
 {
-  global $USER_RES;
-  if (empty ($user_id))
-    return null;
-  if (empty ($USER_RES[$user_id]))
-    $USER_RES[$user_id] =
-      db_execute ("SELECT * FROM user WHERE user_id = ?", [$user_id]);
-  return $USER_RES[$user_id];
+  return utils_make_arg_array ($u, user_getid ());
 }
 
 function user_get_field ($user_id, $field)
 {
-  if (!$user_id)
-    $user_id = user_getid ();
-  $result = user_get_result_set ($user_id);
-  if (db_numrows ($result) > 0)
-    return db_result ($result, 0, $field);
-  return null;
+  $uids = user_get_uids_array ($user_id);
+  $u = user_get_array ($uids);
+  $ret = [];
+  foreach ($u as $k => $v)
+    $ret[$k] = $v[$field];
+  return utils_return_val ($user_id, $ret);
 }
 
-# Fetch a row from user table by user_name, put it to $USER_RES[$user_id].
-function user_get_result_set_from_user_name ($user_name)
+function user_get_fields ($fields, $user_id = 0)
 {
-  global $USER_RES;
-  $res = db_execute ("SELECT * FROM user WHERE user_name = ?", [$user_name]);
-  if (!db_numrows ($res))
-    return null;
-  $USER_RES[db_result ($res, 0, 'user_id')] = $res;
-  return $res;
+  if (!$user_id)
+    $user_id = user_getid ();
+  $arr = user_get_array ([$user_id]);
+  foreach ($arr as $row)
+    {
+      $ret = [];
+      foreach ($fields as $f)
+        if (array_key_exists ($f, $row))
+          $ret[$f] = $row[$f];
+      return $ret;
+    }
+  return [];
+}
+
+function user_get_gpg_key ($user_id = 0, $minified = false)
+{
+  $uids = user_get_uids_array ($user_id);
+  $user_key = user_get_field ($uids, 'gpg_key');
+  $pref_key = user_get_preference ('gpg_key', $uids);
+  $ret = [];
+  foreach ($uids as $u)
+    {
+      if (empty ($user_key[$u]))
+        $user_key[$u] = $pref_key[$u];
+      if ($minified || empty ($pref_key[$u]))
+        $ret[$u] = $user_key[$u];
+      else
+        $ret[$u] = $pref_key[$u];
+    }
+  return utils_return_val ($user_id, $ret);
+}
+
+function user_set_gpg_key ($key)
+{
+  if (!user_isloggedin ())
+    return;
+  list ($minified, $error) = gpg_minify_key ($key);
+  if ($error)
+    $minified = '';
+  $res = db_autoexecute (
+    'user', ['gpg_key' => $minified], DB_AUTOQUERY_UPDATE,
+    "user_id = ?", [user_getid ()]
+  );
+  $ret = user_set_preference ('gpg_key', $key);
+  return $res && $ret;
+}
+
+function user_filter_missing ($users)
+{
+  global $USER_ARR;
+  $missing = $uids = $names = [];
+  foreach ($users as $u)
+    if (!array_key_exists ($u, $USER_ARR))
+      $missing[] = $u;
+  if (empty ($missing))
+    return [$names, $uids];
+  foreach ($missing as $u)
+    if (ctype_digit (strval ($u)))
+      $uids[] = $u;
+    else
+      $names[] = $u;
+  return [$names, $uids];
+}
+
+# Fetch a row from user table by user_id or user_name unless already cached,
+# put it to $USER_ARR under both user_id and user_name.
+function user_fetch_data ($users)
+{
+  global $USER_ARR;
+  $lower_users = array_map ('strtolower', $users);
+  list ($names, $uids) = user_filter_missing ($lower_users);
+  $arg = array_merge ($names, $uids);
+  if (empty ($arg))
+    return;
+  $cond = [];
+  if (!empty ($names))
+    $cond[] = "user_name " . utils_in_placeholders ($names);
+  if (!empty ($uids))
+    $cond[] = "user_id " . utils_in_placeholders ($uids);
+  $res = db_execute ("SELECT * FROM user WHERE " . join (' OR ', $cond), $arg);
+  while ($arr = db_fetch_array ($res))
+    {
+      $idx = strtolower ($arr['user_name']);
+      $USER_ARR[$arr['user_id']] = $USER_ARR[$idx] = $arr;
+    }
+  db_free_result ($res);
+}
+
+function user_refetch_data ($users = null)
+{
+  global $USER_ARR;
+  if ($users === null)
+    $users = [user_getid ()];
+  foreach ($users as $u)
+    if (array_key_exists ($u, $USER_ARR))
+      unset ($USER_ARR[$u]);
+  user_fetch_data ($users);
+}
+
+# Return user data array; when $user is an array, return an array of
+# records with the indices from the input array.
+function user_get_array ($user)
+{
+  global $USER_ARR;
+  if (empty ($user))
+    return [];
+  $users = user_get_uids_array ($user);
+  user_fetch_data ($users);
+  $ret = [];
+  foreach ($users as $name)
+    {
+      $idx = strtolower ($name);
+      if (!empty ($USER_ARR[$idx]))
+        $ret[$name] = $USER_ARR[$idx];
+    }
+  return utils_return_val ($user, $ret);
 }
 
 function user_get_timezone ()
@@ -247,12 +352,23 @@ function user_get_timezone ()
   return user_get_field (user_getid (), 'timezone');
 }
 
+function user_normalize_pref_name ($name)
+{
+  return strtolower (trim ($name));
+}
+
+function user_normalize_preference_names ($names)
+{
+  $ret = array_map ('user_normalize_pref_name', utils_make_arg_array ($names));
+  return utils_return_val ($names, $ret);
+}
+
 function user_set_preference ($preference_name, $value)
 {
   global $user_pref;
   if (!user_isloggedin ())
     return false;
-      $preference_name = strtolower(trim($preference_name));
+  $preference_name = user_normalize_pref_name ($preference_name);
   $res = db_execute ("
     SELECT NULL FROM user_preferences
     WHERE user_id = ? AND preference_name = ?",
@@ -279,7 +395,7 @@ function user_unset_preference ($preference_name)
   global $user_pref;
   if (!user_isloggedin ())
     return false;
-  $preference_name = strtolower (trim ($preference_name));
+  $preference_name = user_normalize_pref_name ($preference_name);
   $result = db_execute ("
     DELETE FROM user_preferences
     WHERE user_id = ? AND preference_name = ? LIMIT 1",
@@ -290,48 +406,66 @@ function user_unset_preference ($preference_name)
   return true;
 }
 
+function user_fetch_preferences ($uids, $pref_names = [])
+{
+  $pref_cond = '';
+  if (!empty ($pref_names))
+    $pref_cond = "\n      preference_name "
+      . utils_in_placeholders ($pref_names) . "\n      AND ";
+  $res = db_execute ("
+    SELECT preference_name, preference_value, user_id FROM user_preferences
+    WHERE {$pref_cond}user_id " . utils_in_placeholders ($uids),
+    array_merge ($pref_names, $uids)
+  );
+  $ret = [];
+  foreach ($uids as $u)
+    $ret[$u] = [];
+  if (!empty ($pref_names))
+    foreach ($uids as $u)
+      foreach ($pref_names as $p)
+        $ret[$u][$p] = null;
+  while ($row = db_fetch_array ($res))
+    $ret[$row['user_id']][$row['preference_name']] = $row['preference_value'];
+  return $ret;
+}
+
+# Both $preference_names and $user_ids may be either scalars or lists;
+# the return value is structured respectively; when the result is
+# two-dimensional, the user ID is the first index: $ret[$uid][$pref_name].
+function user_get_preference_by_id ($preference_names, $user_ids)
+{
+  $uids = user_get_uids_array ($user_ids);
+  $pref_names = utils_make_arg_array ($preference_names);
+  $prefs = user_fetch_preferences ($uids, $pref_names);
+  if (is_array ($preference_names))
+    return utils_return_val ($user_ids, $prefs);
+  # Scalar $preference_names: exclude the redundant preference name
+  # from the result, just like utils_return_val does.
+  $ret = [];
+  foreach ($prefs as $uid => $arr)
+    foreach ($arr as $name => $val)
+      $ret[$uid] = $val;
+  return utils_return_val ($user_ids, $ret);
+}
+
 function user_get_preference ($preference_name, $user_id = false)
 {
   global $user_pref;
-
-  if ($user_id)
-    {
-      # Looking for information without being the user.
-      $res = db_execute ("
-        SELECT preference_value FROM user_preferences
-        WHERE user_id = ? AND preference_name = ?",
-        [$user_id, $preference_name]
-      );
-      if (db_numrows ($res) > 0)
-        return db_result ($res, 0, 'preference_value');
-      return null;
-    }
+  if ($user_id !== false)
+    return user_get_preference_by_id ($preference_name, $user_id);
 
   if (!user_isloggedin ())
     return false;
-  $preference_name = strtolower (trim ($preference_name));
-  # First check to see if we have already fetched the preferences.
-  if ($user_pref)
-    {
-      if (!empty ($user_pref[$preference_name]))
-        return $user_pref[$preference_name];
-      return false;
-    }
-  # We haven't returned prefs - go to the DB.
-  $result = db_execute ("
-    SELECT preference_name, preference_value
-    FROM user_preferences WHERE user_id = ?", [user_getid ()]
-  );
-  if (db_numrows ($result) < 1)
-    return false;
-  for ($i = 0; $i < db_numrows ($result); $i++)
-    {
-      $user_pref[db_result ($result, $i, 'preference_name')] =
-        db_result ($result, $i, 'preference_value');
-    }
-  if (isset ($user_pref["$preference_name"]))
-    return $user_pref["$preference_name"];
-  return false;
+  $uid = user_getid ();
+  if (empty ($user_pref))
+    foreach (user_fetch_preferences ([$uid])[$uid] as $name => $value)
+      $user_pref[$name] = $value;
+  $pref = utils_make_arg_array ($preference_name);
+  $pref = user_normalize_preference_names ($pref);
+  $ret = [];
+  foreach ($pref as $p)
+    $ret[$p] = isset ($user_pref[$p])? $user_pref[$p]: false;
+  return utils_return_val ($preference_name, $ret);
 }
 
 # Find out if the user use the vote, very similar to
@@ -432,12 +566,53 @@ function user_has_history ($user_id)
   return db_numrows ($result) > 0;
 }
 
+# Delete personal query forms.
+function user_delete_query_forms ($user_id, $art)
+{
+  $res = db_execute (
+    "SELECT report_id FROM {$art}_report WHERE scope = ? AND user_id = ?",
+    ['I', $user_id]
+  );
+  $report_ids = [];
+  while ($row = db_fetch_array ($res))
+    $report_ids[] = $row['report_id'];
+  if (empty ($report_ids))
+    return;
+  db_execute ("
+    DELETE FROM {$art}_report
+    WHERE report_id " . utils_in_placeholders ($report_ids), $report_ids
+  );
+  db_execute ("
+    DELETE FROM {$art}_report_field
+    WHERE report_id " . utils_in_placeholders ($report_ids), $report_ids
+  );
+}
+
+function user_get_squads ($user_id = 0)
+{
+  static $squads = [];
+  if (empty ($user_id))
+    $user_id = user_getid ();
+  if (array_key_exists ($user_id, $squads))
+    return $squads[$user_id];
+
+  $result = db_execute (
+    "SELECT squad_id FROM user_squad WHERE user_id = ?", [$user_id]
+  );
+  $squads[$user_id] = [];
+  while ($row = db_fetch_array ($result))
+    $squads[$user_id][] = $row['squad_id'];
+  return [$squads[$user_id], empty ($squads[$user_id])];
+}
+
 # Delete user's data that are not stored in the 'user' table.
 function user_delete_aux_data ($user_id)
 {
   foreach (['user_group', 'user_squad', 'user_bookmarks', 'user_preferences',
     'user_votes', 'session'] as $table)
     db_execute ("DELETE FROM $table WHERE user_id = ?", [$user_id]);
+  foreach (utils_get_tracker_list () as $a)
+    user_delete_query_forms ($user_id, $a);
 }
 
 # Completely remove account from the database; should only be done
@@ -469,7 +644,7 @@ function user_rename ($user_id, $new_name)
         "UPDATE user SET user_name = ? WHERE user_id = ?",
         [$old_name, $user_id]
       );
-      return sprintf ('User <%s> alredy exists', $new_name);
+      return sprintf ('User <%s> already exists', $new_name);
     }
   db_execute ("
     UPDATE group_history set old_value = ?
@@ -483,7 +658,7 @@ function user_rename ($user_id, $new_name)
 # This function should always be used in a secure context, when user_id
 # is 100% sure.
 # Best is to not to pass the user_id argument unless necessary.
-function user_delete ($user_id = false, $confirm_hash = false)
+function user_delete ($user_id = false)
 {
   if (!$user_id)
     $user_id = user_getid ();
@@ -493,15 +668,6 @@ function user_delete ($user_id = false, $confirm_hash = false)
   if (!user_is_super_user () && $user_id != user_getid ())
     exit_permission_denied ();
 
-  $confirm_hash_test = '';
-  $confirm_hash_param = [];
-  # If self-destruct, the correct confirm_hash must be provided.
-  if (!user_is_super_user ())
-    {
-      $confirm_hash_test = " confirm_hash = ? AND ";
-      $confirm_hash_param = [$confirm_hash];
-    }
-
   if (!user_has_history ($user_id))
     {
       user_purge ($user_id);
@@ -514,12 +680,11 @@ function user_delete ($user_id = false, $confirm_hash = false)
     $new_realname = '-';
   $success = db_autoexecute ('user',
     [ 'user_pw' => '!', 'realname' => $new_realname, 'status' => 'S',
-      'email' => 'idontexist@example.net', 'confirm_hash' => '',
+      'email' => 'idontexist@savane.test', 'confirm_hash' => null,
       'authorized_keys' => '', 'people_view_skills' => '0',
       'people_resume' => '', 'timezone' => 'GMT', 'theme' => '',
       'gpg_key' => '', 'email_new' => ''],
-    DB_AUTOQUERY_UPDATE, "$confirm_hash_test user_id = ?",
-    array_merge ($confirm_hash_param, [$user_id])
+    DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
   );
   if (!$success)
     {
@@ -539,15 +704,20 @@ function user_delete ($user_id = false, $confirm_hash = false)
 function user_list_groups_with_history ($uid, $skip)
 {
   return db_execute ("
-    SELECT
-      g.group_name, g.group_id, g.unix_group_name, g.is_public,
-      u.admin_flags, h.date
-    FROM groups g, user_group u, group_history h, user a
-    WHERE
-      g.group_id = u.group_id AND h.group_id = u.group_id $skip
-      AND u.user_id = ? AND a.user_id = ? AND h.old_value = a.user_name
-      AND g.status = 'A' AND h.field_name IN ('Added User', 'Approved User')
-    GROUP BY g.group_id ORDER BY g.group_id", [$uid, $uid]
+    SELECT g.group_name, g.unix_group_name, g.is_public,ug.*
+    FROM
+      ( SELECT u.group_id, u.admin_flags, max(h.date) AS date
+        FROM
+          group_history h JOIN user a ON h.old_value = a.user_name
+          JOIN user_group u
+            ON h.group_id = u.group_id AND u.user_id = a.user_id
+        WHERE
+          u.user_id = ? $skip
+          AND h.field_name IN ('Added User', 'Approved User')
+          GROUP BY u.group_id, u.admin_flags ORDER BY u.group_id
+      ) ug
+      JOIN `groups` g ON g.group_id = ug.group_id
+    ORDER BY ug.group_id", [$uid]
   );
 }
 
@@ -557,9 +727,9 @@ function user_list_groups_without_history ($uid, $skip)
   return db_execute ("
     SELECT g.group_name, g.group_id, g.unix_group_name, g.is_public,
       u.admin_flags, 0 as date
-    FROM groups g, user_group u
+    FROM `groups` g, user_group u
     WHERE g.group_id = u.group_id AND u.user_id = ? AND g.status = 'A' $skip
-    GROUP BY g.group_id ORDER BY g.group_id", [$uid]
+    ORDER BY g.group_id", [$uid]
   );
 }
 
@@ -580,7 +750,9 @@ function user_list_merge_row (&$ret, &$row)
 # Auxiliary function used in user_list_groups.
 function user_enumerate_groups ($uid, $skip_pending)
 {
-  $skip = $skip_pending? "AND u.admin_flags != 'P'": "";
+  $skip = '';
+  if ($skip_pending)
+    $skip = "AND u.admin_flags != '" . MEMBER_FLAGS_PENDING . "'";
   $ret = [];
   $result = user_list_groups_with_history ($uid, $skip);
   while ($row = db_fetch_array ($result))
@@ -604,8 +776,8 @@ function user_list_groups_to_hide (&$ret)
   $gid_in = utils_in_placeholders ($private);
   $result = db_execute ("
     SELECT group_id FROM user_group
-    WHERE user_id = ? AND group_id $gid_in AND admin_flags != 'P'",
-    array_merge ([user_getid ()], $private)
+    WHERE user_id = ? AND admin_flags != ? AND group_id $gid_in",
+    array_merge ([user_getid (), MEMBER_FLAGS_PENDING], $private)
   );
   $allowed = [];
   while ($row = db_fetch_array ($result))
