@@ -7,6 +7,7 @@
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
 # Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2022, 2024 Bob Proulx
 #
 # This file is part of Savane.
 #
@@ -27,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 namespace {
 # Clean initialization for globals.
 $GLOBALS['feedback_count'] = 0;
@@ -109,7 +110,7 @@ function utils_link (
   return "$return>$title$closing_tag";;
 }
 
-# Make an clean email link depending on the authentification level of the user.
+# Make an clean email link depending on the authentication level of the user.
 # Don't use this on normal text, just on field where only an email address is
 # expected.  This may corrupt the text and does extensive search.
 function utils_email ($address, $nohtml = 0)
@@ -425,20 +426,17 @@ function utils_result_column_to_array ($result, $col = 0, $localize = false)
   return $arr;
 }
 
-function utils_user_link ($username, $realname = false, $noneisanonymous = false)
+function utils_user_link ($username, $realname = false)
 {
   global $sys_home;
 
-  if ($username == 'None' || empty($username))
-    {
-      # Would be nice to always return _("Anonymous"); but in some cases it is
-      # really none (assigned_to).
-      if (!$noneisanonymous)
-        # TRANSLATORS: Displayed when no user is selected.
-        return _('None');
-      # TRANSLATORS: anonymous user.
-      return _("Anonymous");
-    }
+  if ($username === 'None')
+    # TRANSLATORS: Displayed when no user is selected.
+    return _('None');
+
+  if (empty ($username))
+    # TRANSLATORS: anonymous user.
+    return _("Anonymous");
   $re = "<a href=\"{$sys_home}users/$username\">";
   if ($realname)
     $re .= "$realname &lt;$username&gt;";
@@ -550,7 +548,7 @@ function utils_show_result_set (
   $rows = db_numrows ($result);
   $cols = db_numfields ($result);
 
-  print "<h$level>$title</h$level>\n";
+  print html_h ($level, $title);
   print "<table border='0' width='100%' summary=\"$title\">\n";
 
   print "<tr>\n";
@@ -665,65 +663,17 @@ function utils_is_valid_filename ($file)
   return true;
 }
 
-# Add debugging information.
-function util_debug ($msg)
-{
-  if (!$GLOBALS['sys_debug_on'])
-    return;
-  $backtrace = debug_backtrace ();
-  $location = '';
-  if (isset ($backtrace[1]))
-    $location = $backtrace[1]['function'];
-  else
-    {
-      $relative_path = str_replace (
-        $GLOBALS['sys_www_topdir'] . '/', '', $backtrace[0]['file']
-      );
-      $location = "$relative_path:{$backtrace[0]['line']}";
-    }
-  $GLOBALS['debug'] .= "($location) $msg<br />";
-}
-
-function dbg ($msg)
-{
-  util_debug ($msg);
-}
-
-# Temporary debug.
-# Use it instead of 'echo' so you can easily spot and remove them later after
-# debugging is done.
-function temp_dbg ($msg)
-{
-  print '<pre>';
-  debug_print_backtrace ();
-  var_dump ($msg);
-  print '</pre>';
-}
-
 # Die with debug information.
 function util_die ($msg)
 {
   trigger_error ($msg);
-  $msg = utils_specialchars ($msg);
-  if (empty ($GLOBALS['sys_debug_on']))
-    die ($msg);
-  print "<strong>Fatal error:</strong> $msg<br />";
-  print '<pre>';
-  debug_print_backtrace ();
-  print '</pre>';
-  die ();
+  die (utils_specialchars ($msg));
 }
 
 # Add feedback information.
 function fb ($msg, $error = 0)
 {
   $GLOBALS['feedback_count']++;
-
-  if ($GLOBALS['sys_debug_on'])
-    {
-      $msg .= ' [#' . $GLOBALS['feedback_count'] . ']';
-      dbg("Add feedback #" . $GLOBALS['feedback_count']);
-    }
   $msg .= "\n";
   if ($error)
     $GLOBALS['ffeedback'] .= $msg;
@@ -743,6 +693,14 @@ function fb_dbsuccess ()
   fb (_("Database successfully updated"));
 }
 
+function fb_dbresult ($success)
+{
+  if ($success)
+    fb_dbsuccess ();
+  else
+    fb_dberror ();
+}
+
 function utils_help ($text, $explanation_array)
 {
   return help ($text, $explanation_array);
@@ -763,20 +721,25 @@ function help ($text, $explanation_array)
   return $text;
 }
 
-function utils_setcookie ($name, $value, $expire, $secure = false)
+# Remove old $domain-based cookies, see Savannah sr #111062.
+# After 2025-05-13 all such cookies will expire, and this function
+# may be removed.
+function utils_clear_spurious_cookie ($name, $value, $secure = false)
 {
   global $sys_home, $sys_default_domain;
   $domain = preg_replace ('/:\d*$/', '', $sys_default_domain);
-
-  # The domain argument is used in order to pass cookies to sys_file_domain,
-  # which is assumed to be a subdomain of sys_default_domain.
-  # When the session cookies don't come to sys_file_domain, attachments
-  # to private items are inaccessible.
   setcookie (
-    $name, $value, $expire, $sys_home, $domain, $secure, true
+    $name, $value, time () - 3600, $sys_home, $domain, $secure, true
   );
 }
 
+function utils_setcookie ($name, $value, $expire, $secure = false)
+{
+  global $sys_home;
+  utils_clear_spurious_cookie ($name, $value, $secure);
+  setcookie ($name, $value, $expire, $sys_home, '', $secure, true);
+}
+
 function utils_set_csp_headers ()
 {
   global $skip_csp_headers;
@@ -920,18 +883,20 @@ function utils_mktemp ($template, $type = 'file')
       $type = 'dir';
       $cmd .= "-d ";
     }
-  $is_func = "is_$type";
+  if (substr ($template, -3) !== 'XXX')
+    $template .= '.XXXXXXXXX';
   $cmd .= $template;
   $res = utils_run_proc ($cmd, $out, $err);
   if ($res)
     {
-      trigger_error ("$cmd failed, $res: $err");
+      trigger_error ("'$cmd' failed, $res: $err");
       return null;
     }
   $out = trim ($out);
+  $is_func = "is_$type";
   if ($is_func ($out))
     return $out;
-  trigger_error ("$cmd failed: no $out $type");
+  trigger_error ("'$cmd' failed: no $out $type");
   return null;
 }
 
@@ -1000,13 +965,13 @@ function utils_update_decimal_separator ()
   return $decimal_separator;
 }
 
-# RFC822 requires some characters to be escaped. We usually care about this
-# compliance only in email headers.
+# RFC822 Section 3.3. requires some characters to only be in quoted strings.
+# We usually care about this compliance only in email headers.
 function utils_comply_with_rfc822 ($string)
 {
-  if (preg_match ("#\.|\,|\@|\/|\\|\||\;|\!#", $string))
-    return "\"$string\"";
-  return $string;
+  if (!preg_match ('#[()<>@,;:\\\".[\]]#', $string))
+    return $string;
+  return '"' . str_replace (['\\', '"'], ['\\\\', '\"'], $string) . '"';
 }
 
 # Work-around PHP 8.1 deprecation of null parameter #1 and change of defaults
@@ -1042,5 +1007,314 @@ function utils_disable_cache ()
   header ("Cache-Control: no-cache", false);
   header ("Cache-Control: must-revalidate", false);
 }
+function utils_public_file_url ($file)
+{
+  global $sys_home;
+  return "{$sys_home}file/{$file['name']}?file_id={$file['id']}";
+}
+
+# Find an item in a tracker, fetch the fields listed in $fields
+# (but group_id in any case), return the array of fields on success,
+# invoke $error_handler ('Item not found') on fail.
+function utils_find_item (
+  $tracker, $item_id, $fields = [], $error_handler = 'exit_error'
+)
+{
+  if (!in_array ('group_id', $fields))
+    $fields[] = 'group_id';
+  $field_list = join (', ', $fields);
+  $result = db_execute (
+    "SELECT $field_list FROM $tracker WHERE bug_id = ?", [$item_id]
+  );
+  if (!db_numrows ($result))
+    return $error_handler (sprintf (_("Item #%s not found."), $item_id));
+  $arr = db_fetch_array ($result);
+  $ret = [];
+  foreach ($fields as $k)
+    $ret[$k] = $arr[$k];
+  return $ret;
+}
+
+# Get and acquire a semaphore for $file; return the semaphore on success,
+# else return null.
+function utils_sem_acquire ($file)
+{
+  $tok = ftok ($file, 'a');
+  if ($tok === -1)
+    {
+      trigger_error ("ftok failed");
+      return null;
+    }
+  $sem = sem_get ($tok);
+  if ($sem === false)
+    {
+      trigger_error ("sem_get failed");
+      return null;
+    }
+  if (sem_acquire ($sem))
+    return $sem;
+  trigger_error ("can't acquire semaphore");
+  return null;
+}
+
+# Acquire a semaphore for $file, return it unless $callback isn't null,
+# in which case return $callback ($arg) and release the semaphore;
+# on any failures with acquiring semaphore, return null.
+function utils_run_lock ($file, $callback = null, $arg = null)
+{
+  $sem = utils_sem_acquire ($file);
+  if ($sem === null)
+    return null;
+  if ($callback === null)
+    return $sem;
+  $ret = $callback ($arg);
+  sem_release ($sem);
+  return $ret;
+}
+
+function utils_no_i18n ($s)
+{
+  return $s;
+}
+
+function utils_debug_output_tv (&$ru, $var, $label)
+{
+  if (!array_key_exists ("ru_$var.tv_sec", $ru))
+    return '';
+  $sec = $ru["ru_$var.tv_sec"] + $ru["ru_$var.tv_usec"] / 1000000;
+  $ret = sprintf (utils_no_i18n ("%s: %.6f s\n"), $label, $sec);
+  unset ($ru["ru_$var.tv_sec"]);
+  unset ($ru["ru_$var.tv_usec"]);
+  return $ret;
+}
+
+function utils_debug_output_val (&$ru, $var, $label)
+{
+  $var = "ru_$var";
+  if (!array_key_exists ($var, $ru))
+    return '';
+  $ret = "$label: {$ru[$var]}\n";
+  unset ($ru[$var]);
+  return $ret;
+}
+
+function utils_debug_output_rusage ($ru)
+{
+  $ret = '  ' . utils_no_i18n ("Resource usage:\n\n");
+  $ret .= utils_debug_output_tv ($ru, 'utime', utils_no_i18n ('user time'));
+  $ret .= utils_debug_output_tv ($ru, 'stime', utils_no_i18n ('system time'));
+  $vars = [
+    'oublock' => 'block outputs', 'inblock' => 'block inputs',
+    'msgsnd' => 'IPC messages sent', 'msgrcv' => 'IPC messages received',
+    'maxrss' => 'maximum resident set size',
+    'ixrss' => 'integral shared memory size',
+    'idrss' => 'integral unshared data size',
+    'minflt' => 'page reclaims (soft page faults)',
+    'majflt' => 'page faults (hard page faults)',
+    'nsignals' => 'signals received', 'nswap' => 'swaps',
+    'nvcsw' => 'voluntary context switches',
+    'nivcsw' => 'involuntary context switches'
+  ];
+  foreach ($vars as $v => $l)
+    $ret .= utils_debug_output_val ($ru, $v, $l);
+  if (!count ($ru))
+    return $ret;
+  $ret .= utils_no_i18n ("Other values:");
+  foreach ($ru as $k => $v);
+    $ret .= preg_replace ('/^ru_/', '', $k) . ": $v\n";
+  return $ret;
+}
+
+function utils_format_timestamp ($t)
+{
+  return sprintf ("%.3f ms", $t);
+}
+
+function utils_debug_output_db_queries ($list_includes)
+{
+  global $db_queries_filed;
+  if (empty ($db_queries_filed))
+    return '';
+  $head[1] = '-multi-';
+  $head[0] = preg_replace ('/./', '-', $head[1]);
+  $ret = '  ' . utils_no_i18n ('DB queries:') . "\n";
+  $ts_total = 0;
+  foreach ($db_queries_filed as $entry)
+    {
+      $ts_total += $entry[3];
+      $ts = utils_format_timestamp ($entry[3]);
+      $ret .= $head[$entry[1]? 1: 0] . " Elapsed time: $ts\n";
+      if ($list_includes)
+        $ret .= "{$entry[2]}\n";
+      $ret .= preg_replace ("/^\n*/", '', $entry[0]) . "\n";
+    }
+  $ts_total = utils_format_timestamp ($ts_total);
+  $footer = "Total number of DB queries: " . count ($db_queries_filed)
+   . "\nTotal time elapsed in DB queries: $ts_total";
+  return "$ret$head[0]\n$footer\n\n";
+}
+
+function utils_debug_footer ($list_includes = false)
+{
+  $ts = utils_format_timestamp (error_timestamp () - $GLOBALS['TIMESTAMP_START']);
+  $ru = getrusage ();
+  $msg = utils_no_i18n ("PHP run summary") . "\n  "
+    . "Elapsed time: $ts\n";
+  if ($list_includes)
+    {
+      $msg .= "  " . utils_no_i18n ("Included files:") . "\n";
+      foreach (get_included_files () as $f)
+        $msg .= error_relative_source_path ($f) . "\n";
+    }
+  $msg .= "\n";
+  $msg .= utils_debug_output_db_queries ($list_includes);
+  return $msg . utils_debug_output_rusage ($ru);
+}
+
+function utils_output_debug_footer ()
+{
+  if (!empty ($GLOBALS['sys_debug_footer']))
+    trigger_error (utils_debug_footer (true));
+}
+
+# Unify the argument that may be scalar or array (return an array);
+# when $def_val isn't null, replace all empty entries with it.
+function utils_make_arg_array ($arg, $def_val = null)
+{
+  if (!is_array ($arg))
+    $arg = [$arg];
+  if ($def_val === null)
+    return $arg;
+  $ret = [];
+  foreach ($arg as $a)
+    $ret[] = empty ($a)? $def_val: $a;
+  return $ret;
+}
+
+# Convert $ret to the original type passed to utils_make_arg_array ().
+function utils_return_val ($arg, $ret)
+{
+  if (is_array ($arg))
+    return $ret;
+  foreach ($ret as $val)
+    return $val;
+}
+function utils_team_signature ()
+{
+  global $sys_name;
+  # TRANSLATORS: the argument is site name (like Savannah).
+  return sprintf (_("-- the %s team."), $sys_name) . "\n";
+}
+
+# Fetch current idNumber (that may or may not be NULL) and the next
+# idNumber to assign.
+function utils_get_next_idNumber ($id, $params)
+{
+  list ($min_id, $tbl, $col, $id_col) = $params;
+  $result = db_execute ("
+    SELECT
+      IFNULL(MAX(`$col`) + 1, 0) AS `next_id`,
+      (SELECT `$col` FROM `$tbl` WHERE `$id_col` = ?) AS `id`
+    FROM `$tbl`", [$id]
+  );
+  $row = db_fetch_array ($result);
+  if (empty ($row))
+    {
+      trigger_error ("Can't fetch $col data; probably DB connection issue");
+      return [0, 0];
+    }
+  if ($row['next_id'] < $min_id)
+    $row['next_id'] = $min_id;
+  return [$row['id'], $row['next_id']];
+}
+
+function utils_idNumber_is_unique ($id, $idN, $params)
+{
+  list ($min_id, $tbl, $col, $id_col) = $params;
+  $res = db_execute (
+    "SELECT COUNT(`$id_col`) AS `cnt` FROM `$tbl` WHERE `$col` = ?", [$idN]
+  );
+  $row = db_fetch_array ($res);
+  if (empty ($row))
+    return false;
+  if ($row['cnt'] == 1)
+    return true;
+  trigger_error (
+    "$col $idN assignment failed for $type #$id ({$row['cnt']} entities found)"
+  );
+  # Probably someone assigned the same idNumber to a different row meanwhile.
+  # Revert our change.
+  if ($row['cnt'] > 1)
+    db_execute (
+      "UPDATE `$tbl` SET `$col` = ? WHERE `$id_col` = ?", [null, $id]
+    );
+  return false;
+}
+
+function utils_idNumber_matches ($id, $idN, $params)
+{
+  list ($min_id, $tbl, $col, $id_col) = $params;
+  $res = db_execute (
+    "SELECT `$col` AS `id` FROM `$tbl` WHERE `$id_col` = ?", [$id]
+  );
+  $row = db_fetch_array ($res);
+  if (empty ($row))
+    return false;
+  return $row['id'] == $idN;
+}
+
+# Values to switch between updating uidNumber and gidNumber
+# in utils_assign_idNumber ().
+function utils_get_assign_idNumber_params ($type)
+{
+  global $sys_min_uidNumber, $sys_min_gidNumber;
+  $cases = [
+    'user'  => [$sys_min_uidNumber, 'user', 'uidNumber', 'user_id'],
+    'group' => [$sys_min_gidNumber, 'groups', 'gidNumber', 'group_id'],
+  ];
+  return $cases[$type];
+}
+
+# Try to assign a new idNumber to the database entity $id (in the context
+# of $type).  Return the new idNumber in case of success, else return zero.
+function utils_try_assign_idNumber ($id, $type)
+{
+  $params = utils_get_assign_idNumber_params ($type);
+  list ($cur_id, $idN) = utils_get_next_idNumber ($id, $params);
+  if (!empty ($cur_id))
+    return $cur_id;
+  if (empty ($idN))
+    return 0;
+  list ($min_id, $tbl, $col, $id_col) = $params;
+  db_execute ("UPDATE `$tbl` SET `$col` = ? WHERE `$id_col` = ?", [$idN, $id]);
+  if (!utils_idNumber_is_unique ($id, $idN, $params))
+    return 0;
+  if (!utils_idNumber_matches ($id, $idN, $params))
+    return 0;
+  return $idN;
+}
+
+# Assign a new idNumber depending on $type ('user' or 'group'),
+# return its value on success and zero on failure.
+function utils_assign_idNumber ($id, $type = 'user')
+{
+  for ($i = 0; $i < 0x11; $i++)
+    {
+      # Hopefully we don't run multiple instances of frontend installed
+      # in separate locations, so we may pass __FILE__ to utils_sem_acquire ().
+      # The competition from the backend scripts is eliminated because
+      # we first assign idNumber, then update the status of the entity.
+      $sem = utils_sem_acquire (__FILE__);
+      # Concurrent runs aren't probable anyway, so we just proceed even
+      # in the unlikely case when we fail to acquire a semaphore.
+      $uid = utils_try_assign_idNumber ($id, $type);
+      if ($sem)
+        sem_release ($sem);
+      if ($uid)
+        return $uid;
+    }
+  trigger_error ("Can't assign idNumber for $type #$id");
+  return 0;
+}
 } # namespace {
 ?>
