@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once (dirname (__FILE__) . '/vcs/git.php');
 
@@ -60,9 +60,19 @@ function vcs_save_sorting ($vcs, $group_id, &$repos)
   group_set_preference ($group_id, "vcs:$vcs:repo-order", $order);
 }
 
-function vcs_get_repo_pref ($vcs, $group_id, $pref, $repo)
+function vcs_get_repo_pref ($vcs, $group_id, $pref_name, $repo)
 {
-  return group_get_preference ($group_id, "vcs:$vcs:$pref:$repo");
+  $repos = utils_make_arg_array ($repo);
+  $names = $ret = [];
+  foreach ($repos as $r)
+    $names[] = "vcs:$vcs:$pref_name:$r";
+  $pref_array = group_get_preference ($group_id, $names);
+  foreach ($pref_array as $name => $val)
+    {
+      $n = str_replace ("vcs:$vcs:$pref_name:", "", $name);
+      $ret[$n] = $val;
+    }
+  return utils_return_val ($repo, $ret);
 }
 function vcs_set_repo_pref ($vcs, $group_id, $pref, $repo, $val)
 {
@@ -99,12 +109,14 @@ function vcs_get_repo_readme ($vcs, $group_id, $name)
 # Use descriptions from database when available.
 function vcs_override_descriptions ($vcs, $group_id, $repos)
 {
-  $ret = [];
+  $ret = $names = $desc = [];
+  foreach ($repos as $r)
+    $names[] = $r['name'];
+  $desc = vcs_get_repo_description ($vcs, $group_id, $names);
   foreach ($repos as $r)
     {
-      $desc = vcs_get_repo_description ($vcs, $group_id, $r['name']);
-      if (!empty ($desc))
-        $r['desc'] = $desc;
+      if (!empty ($desc[$r['name']]))
+        $r['desc'] = $desc[$r['name']];
       $ret[] = $r;
     }
   return $ret;
@@ -131,14 +143,13 @@ function vcs_sort_repos ($vcs, $group_id, $repos)
   return $ret;
 }
 
-# Get array of repository descriptions.
-function vcs_get_repos ($vcs, $group_id)
+function vcs_fetch_repos ($vcs, $group_id)
 {
   global $sys_vcs_dir;
   $func = "{$vcs}_list_repos";
   if (!function_exists ($func))
     return [];
-  $group = project_get_object ($group_id);
+  $group = group_get_object ($group_id);
   $group_name = $group->getUnixName ();
   if (empty ($sys_vcs_dir) || !is_array ($sys_vcs_dir))
     return [];
@@ -154,6 +165,18 @@ function vcs_get_repos ($vcs, $group_id)
   return vcs_sort_repos ($vcs, $group_id, $repos);
 }
 
+# Get array of repository descriptions.
+function vcs_get_repos ($vcs, $group_id, $reload = false)
+{
+  static $cache = [];
+  if (array_key_exists ($vcs, $cache)
+    && array_key_exists ($group_id, $cache[$vcs]) && !$reload
+  )
+    return $cache[$vcs][$group_id];
+  $cache[$vcs][$group_id] = vcs_fetch_repos ($vcs, $group_id);
+  return $cache[$vcs][$group_id];
+}
+
 function vcs_print_browsing_preface ($vcs_name)
 {
   # TRANSLATORS: The argument is a name of VCS (like Arch, CVS, Git).
@@ -242,7 +265,7 @@ function vcs_page ($vcs, $group_id)
   if ($vcs_name === null)
     exit_error ();
 
-  $group = project_get_object ($group_id);
+  $group = group_get_object ($group_id);
   if (!$group->Uses ($vcs) && !$group->UsesForHomepage ($vcs))
     exit_error (_("This group doesn't use this tool."));
 
