@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 require_once('include/init.php');
 require_once('include/markup.php');
 require_once('include/trackers/general.php');
@@ -50,7 +50,7 @@ extract (sane_import ('post',
 ));
 $text = $comment;
 # The request doesn't commit any changes in the database, so we could
-# use GET rather than POST, without checking form_id; howerver,
+# use GET rather than POST, without checking form_id; however,
 # the user may submit a longer text than GET can pass.
 form_check ($submit_buttons);
 
@@ -104,7 +104,7 @@ if ($rich)
   {
     print "</div>\n</td>\n";
     print '<td class="boxlightextra">'
-      . utils_user_link (user_getname (), user_getname (0, 1));
+      . utils_user_link (user_getname (), user_getrealname ());
     print "</td>\n</tr>\n";
     print "</table>\n";
   }
