@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 session_require (['isloggedin' => '1']);
@@ -53,35 +53,46 @@ extract (sane_import ('request',
   ['preg' => [['cancel', '/^(\d+|any)$/']]]
 ));
 
+function cancel_all ($tr_cc, $cc_set)
+{
+  return
+    db_execute ("DELETE FROM $tr_cc WHERE $tr_cc.email IN (?, ?, ?)", $cc_set);
+}
+
+function list_bugs_ccing ($tracker, $tr_cc, $group, $cc_set)
+{
+  $result = db_execute ("
+     SELECT DISTINCT t.bug_id
+     FROM $tracker t JOIN $tr_cc c ON t.bug_id = c.bug_id
+     WHERE t.group_id = ? AND c.email IN (?, ?, ?)",
+     array_merge ([$group], $cc_set)
+  );
+  $bug_ids = [];
+  while ($entry = db_fetch_array ($result))
+    $bug_ids[] = $entry['bug_id'];
+  return $bug_ids;
+}
+
+function cancel_in_tracker ($tracker, $group, $cc_set)
+{
+  $tr_cc = "{$tracker}_cc";
+  if ($group == 'any')
+    return cancell_all ($tr_cc, $cc_set);
+  $bug_ids = list_bugs_ccing ($tracker, $tr_cc, $group, $cc_set);
+  if (empty ($bug_ids))
+    return true;
+  return db_execute ("
+    DELETE FROM $tr_cc
+    WHERE email IN (?, ?, ?) AND bug_id " . utils_in_placeholders ($bug_ids),
+    array_merge ($cc_set, $bug_ids)
+  );
+}
+
 if (!empty ($cancel))
-  {
-    $whichgroup = $cancel;
-    foreach ($trackers as $tracker)
-      {
-        $tr_cc = "{$tracker}_cc";
-        if ($whichgroup == 'any')
-          {
-            # If it all groups, go the easy way
-            db_execute (
-              "DELETE FROM $tr_cc WHERE $tr_cc.email IN (?, ?, ?)",
-              [$user_id, $user_email, $user_name]
-            );
-            continue;
-          }
-        # If we need to remove items only for a given group, we first need
-        # to get this list of items.
-        $result = db_execute (
-          "SELECT bug_id FROM $tracker WHERE group_id = ?", [$whichgroup]
-        );
-        while ($entry = db_fetch_array ($result))
-          db_execute (
-            "DELETE FROM $tr_cc WHERE bug_id = ? AND $tr_cc.email IN (?, ?, ?)",
-            [$entry['bug_id'], $user_id, $user_email, $user_name]
-          );
-      }
-    # Not much crosscheck here, so no feedback (the result should be obvious
-    # anyway).
-  }
+  foreach ($trackers as $tracker)
+    cancel_in_tracker (
+      $tracker, $cancel, [$user_id, $user_email, $user_name]
+    );
 
 # Actually prints the HTML page.
 site_user_header (
@@ -95,23 +106,19 @@ print '<p>'
       . "lists\nof any items of the selected groups.")
   . "<p>\n";
 
-# Find all CC the users is registered to receive, list them per groups.
+# Find all CC the user is registered to receive, list them per groups.
 $groups_with_cc = $groups_with_cc_gid = [];
-foreach ($trackers as $tracker)
+foreach ($trackers as $tr)
   {
     $result = db_execute ("
-      SELECT g.unix_group_name, g.group_name, t.group_id
-      FROM groups g, $tracker t, {$tracker}_cc cc
-      WHERE
-        g.group_id = t.group_id AND t.bug_id = cc.bug_id
-        AND cc.email IN (?, ?, ?)
-      GROUP BY g.group_name",
-      [$user_id, $user_email, $user_name]
+      SELECT g.unix_group_name, g.group_name, g.group_id
+      FROM
+        `groups` g JOIN $tr t ON g.group_id = t.group_id
+        JOIN {$tr}_cc cc ON t.bug_id = cc.bug_id
+      WHERE cc.email IN (?, ?, ?)", [$user_id, $user_email, $user_name]
     );
-    while ($entry = db_fetch_array($result))
+    while ($entry = db_fetch_array ($result))
       {
-        if (isset ($groups_with_cc[$entry['group_id']]))
-          continue;
         $groups_with_cc[$entry['unix_group_name']] = $entry['group_name'];
         $groups_with_cc_gid[$entry['unix_group_name']] = $entry['group_id'];
       }
@@ -125,9 +132,7 @@ if (!count ($groups_with_cc))
     exit;
   }
 
-print $HTML->box_top (
-  _("Groups to which belong items you are in Carbon Copy for")
-);
+print $HTML->box_top (_("Groups with items you are in Carbon Copy for"));
 ksort ($groups_with_cc);
 $i = 0;
 foreach ($groups_with_cc as $thisunixname => $thisname)
