@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,45 +39,57 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-foreach (['init', 'account', 'sendmail'] as $inc)
+foreach (['init', 'account', 'sendmail', 'my/admin/general'] as $inc)
   require_once ("../../include/$inc.php");
 session_require (['isloggedin' => 1]);
 
 extract (sane_import ('post',
-  ['true' => 'update', 'array' => [['form_keys', ['digits', 'no_quotes']]]]
+  [
+    'true' => ['update', 'form_quiet_ssh'],
+    'array' => [['form_keys', ['digits', 'no_quotes']]]
+  ]
 ));
 
 $key_limit = 25; # Maximum key number to register.
-$min_keys = 5; # Minumum key fields to show.
+$min_keys = 5; # Minimum key field number to show.
 
-$keys = account_get_authorized_keys ();
 $user_id = user_getid ();
 form_check ('update');
 
+$keys = account_get_authorized_keys ();
+function add_new_key ($form_keys, $i, &$keys)
+{
+  if (!isset ($form_keys[$i]))
+    return;
+  $k = $form_keys[$i];
+  # Remove useless blank spaces.
+  $k = str_replace ("\n", "", trim ($k));
+  if ($k !== '' && !in_array ($k, $keys, true))
+    {
+      fb (sprintf (_("Key #%s seen"), $i + 1));
+      $keys[] = $k;
+    }
+}
+function update_ssh_keys ($old_keys, $form_keys, $user_id)
+{
+  global $key_limit, $min_keys;
+  $keys = [];
+  for ($i = 0; $i < $key_limit; $i++)
+    add_new_key ($form_keys, $i, $keys);
+  $new_keys = array_diff ($keys, $old_keys);
+
+  if (count ($new_keys))
+    account_new_keys_alert ($user_id);
+  account_register_keys ($keys, $user_id);
+  return $keys;
+}
+
 if ($update)
   {
-    $orig_keys = $keys;
-    $keys = [];
-    for ($i = 0; $i < $key_limit; $i++)
-      {
-        if (!isset ($form_keys[$i]))
-          continue;
-        $k = $form_keys[$i];
-        # Remove useless blank spaces.
-        $k = str_replace ("\n", "", trim ($k));
-        if ($k !== '' && !in_array ($k, $keys, true))
-          {
-            fb (sprintf (_("Key #%s seen"), $i + 1));
-            $keys[] = $k;
-          }
-      }
-    $new_keys = array_diff ($keys, $orig_keys);
-
-    if (count ($new_keys))
-      account_new_keys_alert ($user_id);
-    account_register_keys ($keys, $user_id);
+    $keys = update_ssh_keys ($keys, $form_keys, $user_id);
+    my_sync_preference ('quiet_ssh');
   }
 
 # Not valid registration, or first time to page.
@@ -85,13 +97,13 @@ site_user_header (
   ['title' => _("Change Authorized Keys"), 'context' => 'account']
 );
 print form_header ();
-print '<h2>' . _("Authorized keys") . "</h2>\n";
+print html_h (2, _("Authorized keys"));
 utils_get_content ("account/editsshkeys");
 print '<p>'
  . _("Fill the text fields below with the public keys for each key you want "
      . "to\nregister. After submitting, verify that the number of keys "
      . "registered is what\nyou expected.")
- . "</p>\n";
+ . "</p><p>\n";
 
 $n = count ($keys);
 if ($n < $min_keys)
@@ -104,11 +116,13 @@ for ($i = 0; $i < $n; $i++)
     $k = '';
     if (isset ($keys[$i]))
       $k = $keys[$i];
-    print "<span class=\"preinput\"><label for=\"form_keys[$i]\">";
-    printf (_("Key #%s:"), $i + 1);
-    print "</label></span>\n<input type='text' size='60' id='form_keys[$i]' "
+    print "<span class=\"preinput\">"
+      . html_label ("form_keys[$i]", sprintf (_("Key #%s:"), $i + 1));
+    print "</span>\n<input type='text' size='60' id='form_keys[$i]' "
       . "name='form_keys[$i]'\n  value='$k' /><br />\n";
   }
+print "</p>\n";
+my_quiet_ssh_control ();
 print "<br />\n" . form_footer (_("Update"));
 site_user_footer ([]);
 ?>
