@@ -25,7 +25,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -38,13 +38,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-foreach (['sane', 'utils', 'form'] as $i)
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+foreach (['sane', 'utils', 'form', 'my/admin/general'] as $i)
   require_once ("../../include/$i.php");
 $pref_list = ['email_encrypted', 'nonfixed_feedback', 'keep_only_one_session',
   'quiet_ssh', 'reverse_comments_order', 'stone_age_menu', 'use_bookmarks'
 ];
-$pref_fb_on_set = [
+$fb_on_set = [
  'stone_age_menu' =>
    _("Stone age menu activated, it will be effective the next time "
      . "a page is loaded")
@@ -58,64 +58,56 @@ extract (sane_import ('post',
     'no_quotes' => ['form_timezone', 'user_theme']
   ]
 ));
+# Define actions to do before selecting theme.
+function update_theme ()
+{
+  global $update;
+  form_check ('update');
+  if (!$update)
+    return;
+  my_update_theme ();
+  update_user ();
+}
 
-if ($update)
-  {
-    # Define actions to do before selecting theme.
-    function update_theme ()
-    {
-      global $user_theme, $theme_rotate_jump, $form_timezone, $form_email_hide;
+function update_user ()
+{
+  global $user_theme, $theme_rotate_jump, $form_timezone, $form_email_hide;
 
-      # Update theme.
-      if ($user_theme == "Default")
-        $user_theme = "";
-      elseif ($user_theme !== 'rotate' && $user_theme !== 'random')
-        $user_theme = theme_validate ($user_theme);
+  if ($form_timezone == 100)
+    $form_timezone = "GMT";
+  if (empty ($form_email_hide))
+    $email_hide = '0';
+  else
+    $email_hide = '1';
 
-      if ($theme_rotate_jump == "1")
-        theme_rotate_jump ($user_theme);
+  $success = db_autoexecute (
+    'user',
+     [
+       'email_hide' => $email_hide,
+       'theme' => $user_theme, 'timezone' => $form_timezone,
+     ],
+     DB_AUTOQUERY_UPDATE, "user_id = ?", [user_getid ()]
+  );
+  fb_dbresult ($success);
+  user_refetch_data ();
+}
 
-      if ($form_timezone == 100)
-        $form_timezone = "GMT";
-
-      $success = db_autoexecute (
-        'user',
-         [
-           'email_hide' => ($form_email_hide ? "1" : "0"),
-           'theme' => $user_theme, 'timezone' => $form_timezone,
-         ],
-         DB_AUTOQUERY_UPDATE, "user_id = ?", [user_getid ()]
-      );
-      if ($success)
-        fb (_("Database successfully updated"));
-      else
-        fb (_("Failed to update the database"),1);
-    }
-  }
 # FIXME init.php has to be included after parsing POST variables and defining
 # update_theme that may be used in theme.php that is included in init.php.
 # This isn't the most clear possible way to do these things.
 require_once ('../../include/init.php');
 require_once ('../../include/timezones.php');
-form_check ('update');
+
 extract (sane_import ('request', ['pass' => 'feedback']));
 session_require (['isloggedin' => 1]);
 
-function sync_preference ($pref)
-{
-  global $pref_fb_on_set;
-  if ($GLOBALS["form_$pref"] == "1")
-    {
-      user_set_preference ($pref, 1);
-      if (array_key_exists ($pref, $pref_fb_on_set))
-        fb ($pref_fb_on_set[$pref]);
-    }
-  else
-    user_unset_preference ($pref);
-}
 if ($update)
-  foreach ($pref_list as $p)
-    sync_preference ($p);
+  foreach ($pref_list as $pref)
+    {
+      global $fb_on_set;
+      $fb = array_key_exists ($pref, $fb_on_set)? $fb_on_set[$pref]: null;
+      my_sync_preference ($pref, $fb);
+    }
 
 # Print form and links.
 site_user_header (['context' => 'account']);
@@ -123,7 +115,10 @@ site_user_header (['context' => 'account']);
 $res_user = db_execute (
   "SELECT * FROM user WHERE user_id = ?", [user_getid ()]
 );
-$row_user = db_fetch_array ($res_user);
+$row_user = user_get_fields ([
+  'user_id', 'user_name', 'add_date', 'email', 'realname', 'email_hide',
+  'timezone', 'theme'
+]);
 
 print '<p>' . _("You can change all of your account features from here.")
   . "</p>\n";
@@ -132,7 +127,7 @@ print form_tag ();
 print html_h (2, _("Significant Arrangements"));
 
 print $HTML->box_top (_('Authentication Setup'));
-print '<p><a href="change.php?item=password">' . _("Change Password")
+print '<p><a href="change.php?item=password">' . _("Change password")
   . "</a></p>\n";
 print '<p class="smaller">'
   . _("This password gives access to the web interface.");
@@ -152,13 +147,13 @@ print '<p><a href="editsshkeys.php">';
 if ($keynum > 0)
   printf (
     ngettext (
-      "Edit the %d SSH Public Key registered",
-      "Edit the %d SSH Public Keys registered",
+      "Edit the %d SSH public key registered",
+      "Edit the %d SSH public keys registered",
       $keynum),
     $keynum
   );
 else
-  print _("Register an SSH Public Key");
+  print _("Register SSH public keys");
 
 print "</a></p>\n<p class='smaller'>";
 utils_get_content ("account/index_ssh");
@@ -166,7 +161,7 @@ print "</p>\n";
 
 $i++;
 print_box_next_item ();
-print '<p><a href="change.php?item=gpgkey">' . _("Edit GPG Key")
+print '<p><a href="change.php?item=gpgkey">' . _("Edit GPG key")
   . "</a></p>\n";
 print '<p class="smaller">';
 utils_get_content ("account/index_gpg");
@@ -185,15 +180,7 @@ printf (
 print "</a></p>\n";
 print_box_next_item ();
 
-function pref_cbox ($name, $label, $checked = null)
-{
-  if ($checked === null)
-    $checked = user_get_preference ($name);
-  print "<p>" .form_checkbox ("form_$name", $checked) . "\n";
-  print "<label for=\"form_$name\">$label</label></p>\n";
-}
-
-pref_cbox (
+my_pref_cbox (
   'keep_only_one_session',
   _("Keep only one session opened at a time")
 );
@@ -212,26 +199,26 @@ print $HTML->box_top (_('Identity Record'));
 print '<p>';
 printf (_("Account #%s"), $row_user['user_id']);
 print "</p>\n<p class='smaller'>";
-printf (_("Your login is %s."), "<strong>{$row_user['user_name']}</strong>");
+printf (_("Your login is %s."), "<b>{$row_user['user_name']}</b>");
 # TRANSLATORS: the argument is registration date.
 printf (
   ' ' . _("You registered your account on %s."),
-  '<strong>' . utils_format_date ($row_user['add_date']) . '</strong>'
+  '<b>' . utils_format_date ($row_user['add_date']) . '</b>'
 );
 print "</p>\n";
 
 $i = 0;
 print_box_next_item ();
-print '<p><a href="change.php?item=realname">' . _("Change Real Name")
+print '<p><a href="change.php?item=realname">' . _("Change display name")
   . "</a></p>\n";
 print '<p class="smaller">';
 # TRANSLATORS: the argument is full name.
-printf (_("You are %s."), "<strong>{$row_user['realname']}</strong>");
+printf (_("You are %s."), "<b>{$row_user['realname']}</b>");
 print "</p>\n";
 
 $i++;
 print_box_next_item ();
-print '<p><a href="resume.php">' . _("Edit Resume and Skills") . "</a></p>\n";
+print '<p><a href="resume.php">' . _("Edit resume and skills") . "</a></p>\n";
 print '<p class="smaller">'
   . _("Details about your experience and skills may be of interest to other "
       . "users\nor visitors.")
@@ -240,7 +227,7 @@ print '<p class="smaller">'
 $i++;
 print_box_next_item ();
 print "<p><a href=\"{$sys_home}users/{$row_user['user_name']}\">"
-  . _("View your Public Profile") . "</a></p>\n";
+  . _("View your public profile") . "</a></p>\n";
 print '<p class="smaller">' . _("Your profile can be viewed by everybody.")
  . "</p>\n";
 
@@ -248,13 +235,13 @@ print $HTML->box_bottom ();
 
 print $HTML->box_top (_('Mail Setup'));
 
-print '<p><a href="change.php?item=email">' . _("Change Email Address")
+print '<p><a href="change.php?item=email">' . _("Change email address")
   . "</a></p>\n";
 print '<p class="smaller">';
 printf (
   _("Your current address is %s. It is essential to us that this\n"
     . "address remains valid. Keep it up to date."),
-  "<strong>{$row_user['email']}</strong>"
+  "<b>{$row_user['email']}</b>"
 );
 print "</p>\n";
 
@@ -262,7 +249,7 @@ $i = 0;
 print_box_next_item ();
 
 print '<p><a href="change_notifications.php">'
-  . _("Edit Personal Notification Settings") . "</a></p>\n";
+  . _("Edit personal notification settings") . "</a></p>\n";
 print '<p class="smaller">'
   . _("Here is defined when the trackers should send email notifications. It\n"
       . "permits also to configure the subject line prefix of sent mails.")
@@ -270,7 +257,7 @@ print '<p class="smaller">'
 
 $i++;
 print_box_next_item ();
-print '<p><a href="cc.php">' . _("Cancel Mail Notifications") . "</a></p>\n";
+print '<p><a href="cc.php">' . _("Cancel mail notifications") . "</a></p>\n";
 print '<p class="smaller">' . _("Here, you can cancel all mail notifications.")
   . "</p>\n";
 
@@ -280,7 +267,7 @@ print html_h (2, _("Secondary Arrangements"));
 
 print $HTML->box_top (_('Optional Features'));
 
-pref_cbox ("use_bookmarks", _("Use integrated bookmarks"));
+my_pref_cbox ("use_bookmarks", _("Use integrated bookmarks"));
 print '<p class="smaller">'
   . _("By default, integrated bookmarks are deactivated to avoid redundancy "
       . "with\nthe bookmark feature provided by most modern web browsers. "
@@ -292,7 +279,7 @@ print '<p class="smaller">'
 $i = 0;
 print_box_next_item ();
 
-pref_cbox (
+my_pref_cbox (
   "email_hide", _("Hide email address from your account information"),
   $row_user['email_hide']
 );
@@ -305,7 +292,7 @@ print '<p class="smaller">'
 
 $i++;
 print_box_next_item ();
-pref_cbox ('email_encrypted', _("Encrypt emails when resetting password"));
+my_pref_cbox ('email_encrypted', _("Encrypt emails when resetting password"));
 print '<p class="smaller">'
   . _("When checked, Savannah will encrypt email messages\nwith your "
       . "registered public GPG key when resetting password is requested.\n"
@@ -314,10 +301,7 @@ print '<p class="smaller">'
 
 $i++;
 print_box_next_item ();
-pref_cbox ('quiet_ssh', _("Quiet SSH member shell"));
-print '<p class="smaller">'
-  . _("Suppress the message that offers Savane source code when using SSH.")
-  . "</p>\n";
+my_quiet_ssh_control ();
 
 print $HTML->box_bottom ();
 print $update_btn;
@@ -343,10 +327,9 @@ print "<p>";
 html_select_theme_box ("user_theme", $row_user['theme']);
 print ' ' . _("Theme") . "</p>";
 
-if ("rotate" === $row_user['theme'] || 'random' === $row_user['theme'])
+if (in_array ($row_user['theme'], ['rotate', 'random']))
   print "<p>\n" . form_checkbox ('theme_rotate_jump')
-    . '<label for="theme_rotate_jump">' . _("Jump to the next theme")
-    . "</label></p>\n";
+    . html_label ("theme_rotate_jump", _("Jump to the next theme")) . "</p>\n";
 print '<p class="smaller">'
   . _("Not satisfied with the default color theme of the interface?")
   . "</p>\n";
@@ -380,7 +363,7 @@ if (!theme_guidelines_check (SV_THEME))
 $i++;
 print_box_next_item ();
 
-pref_cbox (
+my_pref_cbox (
   "reverse_comments_order",
   _("Print comments from the oldest to the latest")
 );
@@ -394,7 +377,7 @@ print '<p class="smaller">'
 $i++;
 print_box_next_item ();
 
-pref_cbox ("stone_age_menu", _("Use the Stone Age menu"));
+my_pref_cbox ("stone_age_menu", _("Use the Stone Age menu"));
 print '<p class="smaller">'
   . _("By default, the top menu includes links to all relevant pages\n"
       . "context (group area, personal area) in dropdown submenus. However,\n"
@@ -407,7 +390,7 @@ print '<p class="smaller">'
 $i++;
 print_box_next_item ();
 
-pref_cbox ("nonfixed_feedback", _("Show feedback in relative position"));
+my_pref_cbox ("nonfixed_feedback", _("Show feedback in relative position"));
 print '<p class="smaller">'
   . _("By default, the feedback box appear as a fixed box on top of the "
       . "window.\nIf you check this option, the feedback will\n"
@@ -426,7 +409,7 @@ printf (
   _("If you are no longer member of any project and do not intend to use\n"
     . "%s further, you may want to delete your account. This action cannot "
     . "be undone\nand your current login will be forever lost."),
-  "<strong>$sys_name</strong>"
+  "<b>$sys_name</b>"
 );
 print "</p>\n";
 
