@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 require_once ('../../include/form.php');
@@ -77,11 +77,10 @@ if (db_numrows ($result))
 if (!$allow_resume)
   {
     $result = db_execute ("
-      SELECT groups.group_id FROM user_group, groups
-      WHERE
-        groups.group_id = user_group.group_id AND groups.status = 'A'
-        AND user_id=? AND admin_flags != 'P'
-      LIMIT 1", [user_getid ()]
+      SELECT g.group_id
+      FROM user_group u JOIN groups g ON g.group_id = u.group_id
+      WHERE status = 'A' AND user_id = ? AND admin_flags != ?
+      LIMIT 1", [user_getid (), MEMBER_FLAGS_PENDING]
     );
     $allow_resume = db_numrows ($result);
   }
