@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/database.php');
@@ -93,20 +93,13 @@ function send_pending_user_email ($group_id, $user_id, $user_message)
   if (db_numrows ($res_grp) < 1)
     return 0;
   $row_grp = db_fetch_array ($res_grp);
-  $res_admins = db_execute ("
-    SELECT user.user_name FROM user, user_group
-    WHERE
-      user.user_id = user_group.user_id
-      AND user_group.group_id = ?
-      AND user_group.admin_flags = 'A'", [$group_id]
-  );
-
-  if (db_numrows ($res_admins) < 1)
+  $res = member_admin_flags_query ($group_id, '= ?', MEMBER_FLAGS_ADMIN);
+  if (db_numrows ($res) < 1)
     return 0;
   # Send one email per admin, in one command line comma-separated.
-  $admin_list = '';
-  while ($row_admins = db_fetch_array ($res_admins))
-    $admin_list .= ($admin_list ? ',' : '') . $row_admins['user_name'];
+  $admin_list = [];
+  while ($row_admins = db_fetch_array ($res))
+    $admin_list[] = $row_admins['user_name'];
 
   $message = approval_user_gen_email (
     $row_grp['group_name'], $row_grp['unix_group_name'],
@@ -115,7 +108,7 @@ function send_pending_user_email ($group_id, $user_id, $user_message)
   );
 
   sendmail_mail (
-    ['from' => user_getname (), 'to' => $admin_list],
+    ['from' => user_getname (), 'to' => join (',', $admin_list)],
     # TRANSLATORS: the argument is group name.
     [ 'subject' =>
         sprintf (_("Membership request for group %s"), $row_grp['group_name']),
@@ -135,7 +128,7 @@ form_check ($submits);
 
 if ($update)
   {
-    $result_upd = db_query ("
+    $result_upd = db_execute ("
       SELECT group_id FROM groups WHERE status = 'A' AND is_public = '1'
       ORDER BY group_id"
     );
@@ -143,7 +136,7 @@ if ($update)
       {
         if (!isset ($form_groups[$val['group_id']]))
           continue;
-        # If not in group, add user with admin_flag "P"
+        # If not in group, add user with admin_flag MEMBER_FLAGS_PENDING
         # (not very sensible, but this way we avoid changing
         # the table layout).
         if (member_check_pending ($row_user['user_id'], $val['group_id']))
@@ -159,7 +152,9 @@ if ($update)
             );
             continue;
           }
-        if (!member_add ($row_user['user_id'], $val['group_id'], 'P'))
+        if (!member_add (
+          $row_user['user_id'], $val['group_id'], MEMBER_FLAGS_PENDING
+        ))
           continue;
         send_pending_user_email (
           $val['group_id'], $row_user['user_id'], $form_message
@@ -182,10 +177,10 @@ utils_get_content ("my/groups");
 # Right part.
 print html_splitpage (1);  # Watching other users.
 print $HTML->box_top (_("Watched Partners"));
-$result_w = trackers_data_get_watchees (user_getid ());
-$rows_w = db_numrows ($result_w);
+$w_arr = trackers_data_get_watchees (user_getid ());
+$w_num = count ($w_arr);
 
-if (!$result_w || $rows_w < 1)
+if (!$w_num)
   {
     print '<p>' . _("You are not watching any partners.") . "</p>\n<p>";
     print _("To watch someone, follow the &ldquo;Watch partner&rdquo; link\n"
@@ -197,18 +192,18 @@ if (!$result_w || $rows_w < 1)
 else
   {
     print '<table>';
-    for ($i = 0; $i < $rows_w; $i++)
+    $i = 0;
+    user_getname (array_keys ($w_arr)); # Pre-fetch all user names.
+    foreach ($w_arr as $uid => $gid)
       {
-        $wa_res = db_result ($result_w, $i, 'watchee_id');
-        $gr_res = db_result ($result_w, $i, 'group_id');
-        print '<tr class="' . utils_altrow ($i) . '"><td width="99%"><strong>'
-          . utils_user_link (user_getname ($wa_res), user_getrealname($wa_res))
-          . '</strong> <span class="smaller">[' . group_getname ($gr_res) . ']'
+        print '<tr class="' . utils_altrow ($i++) . '"><td width="99%"><strong>'
+          . utils_user_link (user_getname ($uid), user_getrealname ($uid))
+          . '</strong> <span class="smaller">[' . group_getname ($gid) . ']'
           . "</span>\n";
 
         print "</td>\n"
           . "<td><a href=\"$php_self?func=delwatchee&amp;group_id="
-          . "$gr_res&amp;watchee_id=$wa_res" . '" onClick="return confirm(\''
+          . "$gid&amp;watchee_id=$uid" . '" onClick="return confirm(\''
           . _("Stop watching this user") . '\')">'
           . html_image_trash (['alt' => _("Stop watching this user")])
           . "</a></td></tr>\n";
@@ -216,38 +211,28 @@ else
     print "</table>\n";
   }
 
-$result_w = trackers_data_get_watchers (user_getid ());
-$watchers = '';
-$watchers_num = 0;
-while ($row_watcher = db_fetch_array ($result_w))
-  {
-    $watchers_num += 1;
-    $watchers .= "\n"
-      . utils_user_link (
-          user_getname ($row_watcher['user_id']),
-          user_getrealname ($row_watcher['user_id'])
-        )
-      . ' <span class="smaller">[' . group_getname ($row_watcher['group_id'])
-      . ']</span>, ';
-  }
+$w_arr = trackers_data_get_watchers (user_getid ());
+$w_num = count ($w_arr);
+$watchers = [];
+user_getname (array_keys ($w_arr)); # Pre-fetch all user names.
+foreach ($w_arr as $uid => $gid)
+  $watchers[] = utils_user_link (user_getname ($uid), user_getrealname ($uid))
+    . ' <span class="smaller">[' . group_getname ($gid) . ']</span>';
 
-if ($watchers)
+if ($w_num)
   {
-    $watchers = substr ($watchers, 0, -2); # Remove extra comma at the end.
-    $watchers .= ".";
+    $watchers = join (",\n", $watchers);
 
     print '<p>';
     # TRANSLATORS: the message is selected according to number of watchers
     # listed in the first argument; the second argument is comma-separated
     # list of watchers.
-    printf (
-      ngettext (
-        'I am currently watched by %1$s user: %2$s.',
-        'I am currently watched by %1$s users: %2$s.',
-        $watchers_num
-      ),
-      $watchers_num, $watchers
+    $msg = ngettext (
+      'I am currently watched by %1$s user: %2$s.',
+      'I am currently watched by %1$s users: %2$s.',
+      $w_num
     );
+    printf ($msg, $w_num, $watchers);
     print "</p>\n";
   }
 else
@@ -268,19 +253,20 @@ print
     . "this form.");
 print "</p>\n\n";
 
+extract (sane_import ('request', ['specialchars' => 'words']));
 print form_tag () . form_hidden (['searchgroup' => '1']);
-print form_input ('text', 'words', $words,
-  "title=\"" . _("Group to look for") . "\" size='35'");
+print html_label ('words', _("Group to look for"))
+  . ' ' . form_input ('text', 'words', $words, "size='35'");
 print "<br /><br />\n";
 $int_trapisset = true;
 print form_submit (_("Search Groups"), "Submit");
 print "</form>\n\n</div><!-- end boxitem -->\n";
 
-extract (sane_import ('request', ['pass' => 'words']));
 if (!is_scalar ($words))
   $words = '';
 if ($words)
   {
+    $offset = 0; $max_rows = 33; $type = null;
     # Avoid to big search by asking for more than 1 characters.
     # Restricting to more than 2 chars skips a great deal of group names
     # (eg: gv, gdb).
@@ -348,15 +334,15 @@ if (count ($group_list) < 1)
     exit;
   }
 $titles = [
-  'A' => [
+  MEMBER_FLAGS_ADMIN => [
     _("Groups I'm administrator of"),
     _("I am not administrator of any groups"),
     _("Quit this group")],
-  '' => [
+  MEMBER_FLAGS_MEMBER => [
     _("Groups I'm contributor of"),
     _("I am not contributor member of any groups"),
     _("Quit this group")],
-  'P' => [
+  MEMBER_FLAGS_PENDING => [
     _("Requests for inclusion waiting for approval"),
     _("None found"),
     _("Discard this request")]
