@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 require_once ('../include/init.php');
 require_once ('../include/session.php');
 require_once ('../include/sane.php');
@@ -66,7 +66,7 @@ if ($cancel)
 function notify_admins ($quitting_group_id)
 {
   global $sys_mail_replyto, $sys_mail_domain, $sys_mail_admin;
-  $user_name = user_getname (0, 1);
+  $user_name = user_getrealname ();
   $login = user_getname ();
   $group_name = group_getname ($quitting_group_id);
   $res_admin = db_execute ("
