@@ -24,7 +24,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -37,10 +37,9 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-require_once ('../include/init.php');
-require_once ('../include/form.php');
-require_once ('../include/news/general.php');
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
+foreach (['init', 'form', 'trackers/general', 'news/general'] as $i)
+  require_once ("../include/$i.php");
 
 extract (sane_import ('request',
   ['pass' => 'feedback', 'digits' => ['id', 'limit']])
