@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once('../../include/init.php');
 require_once('../../include/form.php');
@@ -95,7 +95,7 @@ if ($people_cat)
     # Show categories and blank row.
     site_header (['title' => _('Change Categories')]);
     # List of possible categories for this group.
-    $result = db_query ("SELECT category_id, name FROM people_job_category");
+    $result = db_execute ("SELECT category_id, name FROM people_job_category");
     if (db_numrows ($result) > 0)
       utils_show_result_set (
         $result, _("Existing Categories"), 'people_cat', '2'
@@ -118,7 +118,7 @@ elseif ($people_skills)
     # Show people_groups and blank row.
     site_header (['title' => _('Change People Skills')]);
     # List of possible people_groups for this group.
-    $result = db_query ("SELECT skill_id, name FROM people_skill");
+    $result = db_execute ("SELECT skill_id, name FROM people_skill");
     print "<p>";
     if (db_numrows ($result) > 0)
       utils_show_result_set (
