@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,25 +39,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 extract (sane_import ('get', ['digits' => 'user_id']));
 
-if (empty ($user_id))
-  exit_missing_param (['user_id']);
-
-$result = db_execute (
-  "SELECT user_name, gpg_key FROM user WHERE user_id = ?", [$user_id]
-);
-
-if (db_numrows ($result) < 1)
-  exit_error (_("User not found."));
-
-$user_name = db_result ($result, 0, 'user_name');
-$gpg_key = db_result ($result, 0, 'gpg_key');
-
-if (!$gpg_key)
+exit_if_missing ('user_id');
+if (!user_exists ($user_id))
+  exit_error (sprintf (_("User #%s not found."), $user_id));
+$user_name = user_getname ($user_id);
+$key = user_get_gpg_key ($user_id);
+if (empty ($key))
   exit_error (_("This user hasn't registered a GPG key."));
 
 header ('Content-Type: application/pgp-keys');
@@ -65,5 +57,5 @@ header ("Content-Disposition: attachment; filename=$user_name-key.gpg");
 # TRANSLATORS: the argument is user's name.
 $description = sprintf (_('GPG Key of the user %s'), $user_name);
 header ("Content-Description: $description");
-print $gpg_key;
+print $key;
 ?>
