@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 require_directory ("trackers");
@@ -56,8 +56,7 @@ if (!user_ismember ($group_id, 'A'))
 
 if ($update && !in_array ($from_group_id, [0, 100]))
   {
-    if (empty ($artifact))
-      exit_missing_param (['artifact']);
+    exit_if_missing ('artifact');
     trackers_conf_copy ($group_id, $artifact, $from_group_id);
   }
 
