@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 require_once ('../../include/account.php');
@@ -60,7 +60,7 @@ extract (sane_import ('post',
   ]
 ));
 
-session_require (['group' => $group_id, 'admin_flags' => 'A']);
+session_require (['group' => $group_id, 'admin_flags' => MEMBER_FLAGS_ADMIN]);
 
 if (!$group_id)
   exit_no_group ();
@@ -73,9 +73,8 @@ function finish_page ()
   exit (0);
 }
 
-if (account_realname_valid ($form_realname))
-  $form_realname = account_sanitize_realname ($form_realname);
-else
+$form_realname = account_sanitize_realname ($form_realname);
+if (!account_realname_valid ($form_realname))
   $form_realname = '';
 
 function update_squad_name ($new_name, $squad_id, $group_id, &$current_name)
@@ -121,15 +120,9 @@ if ($squad_id)
   {
     # A squad passed? Allow to add and remove member, to
     # change the squad name or to delete it.
-    $sql = "
-      SELECT user.user_name, user.realname, user.user_id
-      FROM user, user_group
-      WHERE
-        user.user_id = ? AND user_group.group_id = ?
-        AND user_group.user_id = user.user_id
-        AND user_group.admin_flags = 'SQD'
-      ORDER BY user.user_name";
-    $result = db_execute ($sql, [$squad_id, $group_id]);
+    $result = member_admin_flags_query ($group_id,
+      '= ? AND u.user_id = ?', [MEMBER_FLAGS_SQUAD, $squad_id]
+    );
     if (!db_numrows ($result))
       exit_error (_("Squad not found"));
 
@@ -175,21 +168,20 @@ if ($squad_id)
 
     print form_header ();
     print form_hidden (["group_id" => $group_id, "squad_id" => $squad_id]);
-    print '<p><span class="preinput"><label for="form_realname">'
-          . _("Real Name:") . "</label></span><br />\n&nbsp;&nbsp;";
+    print '<p><span class="preinput">'
+      . html_label ("form_realname", _("Display name:"))
+      . "</span><br />\n&nbsp;&nbsp;";
     print form_input ("text", "form_realname", $realname)
       . " &lt;$squad_name&gt;</p>\n";
     print form_submit (_("Update"), "update_general") . ' '
-      . form_submit (_("Delete Squad"), "update_delete_step1") . "</form>\n";
+      . form_submit (_("Delete squad"), "update_delete_step1") . "</form>\n";
 
     print html_h (2, _("Removing members"));
 
     $result_delusers = db_execute ("
-      SELECT user.user_id, user.user_name, user.realname
-      FROM user, user_squad
-      WHERE user.user_id = user_squad.user_id AND user_squad.squad_id = ?
-      ORDER BY user.user_name",
-      [$squad_id]
+      SELECT u.user_id, user_name, realname
+      FROM user u JOIN user_squad s ON u.user_id = s.user_id
+      WHERE s.squad_id = ?  ORDER BY user_name", [$squad_id]
    );
 
     print "<p>"
@@ -220,15 +212,9 @@ if ($squad_id)
       print $users_none_found;
     print form_submit (_("Remove Members"), "remove_from_squad") . "</form>\n";
     print html_h (2, _("Adding members"));
-    $result_addusers =  db_execute ("
-      SELECT user.user_id, user.user_name, user.realname
-      FROM user, user_group
-      WHERE
-        user.user_id=user_group.user_id AND user_group.group_id = ?
-        AND admin_flags <> 'P' AND admin_flags <> 'SQD'
-      ORDER BY user.user_name", [$group_id]
+    $result_addusers =  member_admin_flags_query ($group_id,
+      'NOT IN (?, ?)', [MEMBER_FLAGS_PENDING, MEMBER_FLAGS_SQUAD]
     );
-
     print "<p>"
       . _("To add members to the squad, select their name and push the button "
           . "below.")
@@ -293,9 +279,9 @@ function create_squad (&$form_loginname, &$form_realname, $group)
     'user',
     [
       'user_name' => strtolower ($group . "-" . $form_loginname),
-      'user_pw' => 'ignored', 'realname' => $form_realname,
+      'user_pw' => '!', 'realname' => $form_realname, 'people_resume' => '',
       'email' => "{$sys_mail_replyto}@{$sys_mail_domain}",
-      'add_date' => time (), 'status' => 'SQD', 'email_hide' => 1,
+      'add_date' => time (), 'status' => USER_STATUS_SQUAD, 'email_hide' => 1,
     ],
     DB_AUTOQUERY_INSERT
   );
@@ -306,7 +292,7 @@ function create_squad (&$form_loginname, &$form_realname, $group)
     }
   fb (_("Squad created"));
   $created_squad_id = db_insertid ($result);
-  member_add ($created_squad_id, $group_id, 'SQD');
+  member_add ($created_squad_id, $group_id, MEMBER_FLAGS_SQUAD);
 
   # Clear variables so the form below will be empty.
   $form_loginname = $form_realname = null;
@@ -329,16 +315,9 @@ if ($update)
 if ($update_delete_step2 && $deletionconfirmed == "yes")
   {
     $squad_id_to_delete = $squad_id_to_delete;
-    $delete_result = db_execute ("
-      SELECT user.user_name, user.realname, user.user_id
-      FROM user, user_group
-      WHERE
-        user.user_id = ? AND user_group.group_id = ?
-        AND user_group.user_id = user.user_id
-        AND user_group.admin_flags = 'SQD'
-      ORDER BY user.user_name", [$squad_id_to_delete, $group_id]
+    $delete_result = member_admin_flags_query ($group_id,
+       '= ? AND u.user_id = ?', [MEMBER_FLAGS_SQUAD, $squad_id_to_delete]
     );
-
     if (!db_numrows ($delete_result))
       exit_error (_("Squad not found"));
 
@@ -346,14 +325,7 @@ if ($update_delete_step2 && $deletionconfirmed == "yes")
     member_remove ($squad_id_to_delete, $group_id);
   }
 
-$result = db_execute ("
-  SELECT user.user_name, user.realname, user.user_id
-  FROM user, user_group
-  WHERE
-    user.user_id = user_group.user_id AND user_group.group_id = ?
-    AND user_group.admin_flags = 'SQD'
-  ORDER BY user.user_name", [$group_id]
-);
+$result = member_admin_flags_query ($group_id, '= ?', MEMBER_FLAGS_SQUAD);
 $rows = db_numrows ($result);
 
 site_project_header (
@@ -385,21 +357,20 @@ else
 # important).
 print html_h (2, _("Create a New Squad"));
 
-$result = db_execute ("
-  SELECT user_id FROM user_group
-  WHERE group_id = ? AND admin_flags <> 'P' AND admin_flags <> 'SQD'",
-  [$group_id]
+$result = member_admin_flags_query (
+  $group_id, 'NOT IN (?, ?)', [MEMBER_FLAGS_PENDING, MEMBER_FLAGS_SQUAD]
 );
 if ($rows < db_numrows ($result))
   {
     print form_header ("$php_self#form");
     print form_hidden (["group_id" => $group_id]);
-    print '<p><span class="preinput"><label for="form_loginname">'
-      . _("Squad Login Name:") . "</label></span>\n<br />&nbsp;&nbsp;";
+    print '<p><span class="preinput">'
+      . html_label ("form_loginname", _("Squad login name:"))
+      . "</span>\n<br />&nbsp;&nbsp;";
     print "$group-" . form_input ("text", "form_loginname", $form_loginname)
       . "</p>\n";
     print '<p><span class="preinput">'
-      . html_label ("form_realname", _("Squad Full Name:"))
+      . html_label ("form_realname", _("Squad full name:"))
       . "</span>\n<br />&nbsp;&nbsp;";
     print form_input ("text", "form_realname", $form_realname) . "</p>\n";
     print form_footer ();
