@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 require_once ('../../include/init.php');
 require_once ('../../include/form.php');
 require_once ('../../include/sendmail.php');
@@ -168,7 +168,7 @@ function look_for_non_members ($group_id, $words)
         (select * from user_group ug WHERE group_id = ?) ug
       ON u.user_id = ug.user_id
     WHERE $kw_sql AND status = 'A' AND ug.onduty IS NULL
-    GROUP BY u.user_id ORDER BY user_name LIMIT 26", $sql_params
+    ORDER BY user_name LIMIT 26", $sql_params
     );
 }
 
@@ -231,10 +231,9 @@ if ($action == 'approve_for_group' && $user_ids)
            sprintf (
              "You've been approved as a member of the group %s on %s,\n"
              . "where you are registered as %s.",
-            group_getname ($group_id), $sys_name,
-            user_getname ($user)
-          )
-          . "\n\n" . sprintf ("-- the %s team.", $sys_name) . "\n";
+             group_getname ($group_id), $sys_name, user_getname ($user)
+           )
+         . "\n\n" . utils_team_signature ();
 
         sendmail_mail (
           ['to' => $email], ['subject' => $title, 'body' => $message]
@@ -246,24 +245,16 @@ site_project_header (
   ['title' => _("Manage Members"), 'group' => $group_id, 'context' => 'ahome']
 );
 
-function user_admin_query ($cond)
-{
-  global $group_id;
-  return db_execute ("
-    SELECT u.user_id, u.user_name, u.realname
-    FROM user u, user_group g
-    WHERE u.user_id = g.user_id AND g.group_id = ? AND $cond
-    ORDER BY u.user_name", [$group_id]
-  );
-}
+$res_pend = member_admin_flags_query ($group_id, "= ?", MEMBER_FLAGS_PENDING);
 
-$res_pending = user_admin_query ("admin_flags = 'P'");
-
-show_pending_users_list ($res_pending, $group_id);
-db_data_seek ($res_pending);
+show_pending_users_list ($res_pend, $group_id);
+db_data_seek ($res_pend);
 print "<br />\n";
-$res_active = user_admin_query ("NOT admin_flags IN ('A', 'P', 'SQD')");
-show_all_users_remove_list ($res_active, $res_pending, $group_id);
+$flags = [MEMBER_FLAGS_ADMIN, MEMBER_FLAGS_PENDING, MEMBER_FLAGS_SQUAD];
+$res_act = member_admin_flags_query (
+  $group_id, "NOT " . utils_in_placeholders ($flags), $flags
+);
+show_all_users_remove_list ($res_act, $res_pend, $group_id);
 print "<br />\n";
 show_all_users_add_searchbox ($group_id, $words);
 show_all_users_add_list ($group_id, $words);
