@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../../include/init.php');
 session_require (['group' => $group_id, 'admin_flags' => 'A']);
@@ -80,7 +80,7 @@ function effective_permissions ($squad_perm, $user_perm)
   if ($user_perm == "9")
     return [true, $squad_perm];
 
-  # If user's perm and squad perm are 1 (techician) and  3 (manager), assume
+  # If user's perm and squad perm are 1 (technician) and  3 (manager), assume
   # that the user should be both technician and manager.
   if (($user_perm == "1" && $squad_perm == "3")
       || ($squad_perm == "1" && $user_perm == "3"))
@@ -176,17 +176,14 @@ function no_member_changes ($uid, &$fields, $is_squad)
   return true;
 }
 
-function compile_member_fields_values (&$permissions)
+function compile_member_fields_values ($permissions)
 {
   global $trackers;
-  $fields = [
-    'admin_flags' => $permissions['admin'],
-    'privacy_flags' => $permissions['privacy'],
-    'onduty' => $permissions['onduty'],
-  ];
-
-  foreach ($trackers as $art)
-    $fields[$art . '_flags'] = $permissions[$art];
+  $fields['admin_flags'] = $permissions['admin'];
+  $fields['onduty'] = $permissions['onduty'] !== null;
+  foreach (array_merge ($trackers, ['privacy']) as $k)
+    $fields[$k . '_flags'] =
+      $permissions[$k] === 'NULL'? null: $permissions[$k];
   return $fields;
 }
 
@@ -200,10 +197,11 @@ function import_member_permissions ($row)
   $checkbox_names = ["onduty_user_$uid"];
   $checkbox_names[] = "privacy_user_$uid";
 
+  $flags = [MEMBER_FLAGS_ADMIN, MEMBER_FLAGS_SQUAD, MEMBER_FLAGS_PENDING];
   $imported = sane_import ('post',
     [
       'preg' => [$names], 'true' => $checkbox_names,
-      'strings' =>  [["admin_user_$uid", ['A', 'SQD', 'P']]],
+      'strings' =>  [["admin_user_$uid", $flags]],
     ]
   );
   $len = strlen ("_user_$uid");
@@ -215,11 +213,13 @@ function import_member_permissions ($row)
 function fetch_user_list  ($group_id)
 {
   global $user_list, $squad_list;
-  $res = query_members ($group_id, "NOT g.admin_flags IN ('P', 'SQD')");
+  $flags = [MEMBER_FLAGS_PENDING, MEMBER_FLAGS_SQUAD];
+  $cond = "NOT g.admin_flags " . utils_in_placeholders ($flags);
+  $res = query_members ($group_id, $cond, $flags);
   $user_list = [];
   while ($row = db_fetch_array ($res))
     $user_list[$row['user_id']] = $row;
-  $res = query_members ($group_id, "g.admin_flags = 'SQD'");
+  $res = query_members ($group_id, "g.admin_flags = ?", [MEMBER_FLAGS_SQUAD]);
   $squad_list = [];
   while ($row = db_fetch_array ($res))
     $squad_list[$row['user_id']] = $row;
@@ -240,16 +240,16 @@ function sanitize_permissions ($uid, $is_squad)
   # Admins are not allowed to turn off their own admin flag.
   # It is too dangerous---set it back to 'A'.
   if (user_getid () == $uid && !user_is_super_user ())
-    $permissions['admin'] = 'A';
+    $permissions['admin'] = MEMBER_FLAGS_ADMIN;
   # Squads flag cannot be changed, squads should not be turned into normal
   # users.
   if ($is_squad)
-    $permissions['admin'] = 'SQD';
+    $permissions['admin'] = MEMBER_FLAGS_SQUAD;
   if (empty ($permissions['admin']))
-    $permissions['admin'] = '';
+    $permissions['admin'] = MEMBER_FLAGS_MEMBER;
 
   # Admins have the access to the private items.
-  if ($permissions['admin'] == "A")
+  if ($permissions['admin'] == MEMBER_FLAGS_ADMIN)
     $permissions['privacy'] = '1';
 }
 
@@ -257,7 +257,7 @@ function change_member ($row_dev, &$permissions, &$squad_permissions)
 {
   global $group_id, $feedback_squad_override, $feedback_able, $squad_flags;
   $uid = $row_dev['user_id']; $name = $row_dev['user_name'];
-  $is_squad = $row_dev['admin_flags'] == 'SQD';
+  $is_squad = $row_dev['admin_flags'] == MEMBER_FLAGS_SQUAD;
   $permissions = import_member_permissions ($row_dev);
   sanitize_permissions ($uid, $is_squad);
 
@@ -343,13 +343,14 @@ function group_field_reference ()
   return $reference;
 }
 
-function no_group_changes (&$fields)
+function no_group_changes (&$fields, &$reference)
 {
-  $reference = group_field_reference ();
+  $f = $fields;
+  foreach ($f as $k => $v)
+    if ($v === 'NULL')
+      $fields[$k] = 0;
   foreach ($fields as $k => $v)
     {
-      if ($v === 'NULL')
-        $v = 0;
       if (!array_key_exists ($k, $reference))
         $reference[$k] = 0;
       if ($v != $reference[$k])
@@ -360,7 +361,8 @@ function no_group_changes (&$fields)
 
 function update_group (&$fields, $group_id, $titles)
 {
-  if (no_group_changes ($fields))
+  $reference = group_field_reference ();
+  if (no_group_changes ($fields, $reference))
     return;
   $result = db_autoexecute ('groups_default_permissions',
     $fields, DB_AUTOQUERY_UPDATE, "group_id = ?", [$group_id]
@@ -410,8 +412,8 @@ function compile_posting_restrictions ()
   $fields = [];
   foreach ($trackers as $art)
     {
-      $var = $art . '_flags';
-      $fields[$art . '_rflags'] = $GLOBALS[$var];
+      $var = $GLOBALS[$art . '_flags'];
+      $fields[$art . '_rflags'] = $var === 'NULL'? 0: $var;
     }
   return $fields;
 }
@@ -439,7 +441,7 @@ function append_tracker_titles (&$titles, $project)
     $titles[] = $title;
 }
 
-function query_members ($group_id, $cond)
+function query_members ($group_id, $cond, $arg)
 {
   global $trackers;
   $tr = '';
@@ -450,7 +452,8 @@ function query_members ($group_id, $cond)
       u.user_name, u.realname, u.user_id, g.admin_flags, g.onduty,
       g.onduty as onduty_flags,$tr g.privacy_flags
     FROM user u JOIN user_group g ON u.user_id = g.user_id
-    WHERE g.group_id = ? AND $cond ORDER BY u.user_name", [$group_id]
+    WHERE g.group_id = ? AND $cond ORDER BY u.user_name",
+    array_merge ([$group_id], $arg)
   );
 }
 
@@ -709,6 +712,7 @@ if ($update)
     update_posting_restrictions ($fields, $group_id);
 
     fetch_user_list ($group_id);
+    group_get_default_permissions ($group_id, true);
     fetch_group_data ($group_id);
   } # if ($update)
 
