@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/form.php');
@@ -49,7 +49,7 @@ require_directory ("trackers");
 
 function is_squad ($m)
 {
-  return $m['admin_flags'] === 'SQD';
+  return $m['admin_flags'] === MEMBER_FLAGS_SQUAD;
 }
 
 # Return lists of members sorted by the onduty flag
@@ -58,15 +58,14 @@ function fetch_member_list ($group_id)
 {
   $flag_select = '';
   foreach (['admin', 'bugs', 'task', 'patch', 'news', 'support'] as $f)
-    $flag_select .= " g.{$f}_flags AS {$f}_flags,";
+    $flag_select .= " g.{$f}_flags,";
   $res = db_execute ("
     SELECT
       u.user_name, u.user_id, u.realname, u.add_date, u.people_view_skills,
-      u.email,$flag_select g.onduty AS onduty
-    FROM user u, user_group g
-    WHERE u.user_id = g.user_id AND g.group_id = ? AND g.admin_flags <> 'P'
-    ORDER BY u.user_name",
-    [$group_id]
+      u.email,$flag_select g.onduty
+    FROM user u JOIN user_group g ON u.user_id = g.user_id
+    WHERE g.group_id = ? AND g.admin_flags <> ? ORDER BY u.user_name",
+    [$group_id, MEMBER_FLAGS_PENDING]
   );
 
   $members = [0 => [], 1 => []];
@@ -121,13 +120,13 @@ function print_in_td ($s)
 function role_icon ($flags, $group_id)
 {
   global $sys_group_id;
-  if ($flags == 'A')
+  if ($flags == MEMBER_FLAGS_ADMIN)
     {
       if ($group_id == $sys_group_id)
         return ["site-admin",  _("Site Administrator")];
       return ["project-admin", _("Group administrator")];
     }
-  if ($flags == 'SQD')
+  if ($flags == MEMBER_FLAGS_SQUAD)
     return ["people", _("Squad")];
   return ["project-member", _("Group member")];
 }
@@ -233,8 +232,8 @@ $duty_titles = [
 ];
 foreach ([1, 0] as $onduty)
   {
-    print "<h2>{$duty_titles[$onduty]}</h2>\n";
-    echo html_build_list_table_top ($title_arr);
+    print html_h (2, $duty_titles[$onduty]);
+    print html_build_list_table_top ($title_arr);
 
     $mem = $members[$onduty];
     $cnt = count ($mem);
@@ -242,7 +241,7 @@ foreach ([1, 0] as $onduty)
       {
         $m = $mem[$i];
         $color = utils_altrow ($i);
-        $is_admin = $m['admin_flags'] == 'A';
+        $is_admin = $m['admin_flags'] == MEMBER_FLAGS_ADMIN;
         if ($is_admin)
           $color = "boxhighlight";
         print "\n\t<tr class=\"$color\">\n";
