@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # Tricks for automatically opening a task in the tracker (sigh..).
 define ('ARTIFACT', 'task');
@@ -57,7 +57,7 @@ require_once ('../include/sendmail.php');
 # GPLQuickForm validation callback.
 # Make sure the name is not already taken, ignoring incomplete
 # registrations: risks of a name clash seems near 0, while not doing that
-# require maintainance, since some people interrupt registration and
+# require maintenance, since some people interrupt registration and
 # try to redoit later with another name.
 # And even if a name clash happens, admins will notice it during approval.
 function group_does_not_already_exist ($unix_name)
@@ -335,13 +335,7 @@ sendmail_mail (
   $item_id = trackers_data_create_item (
     $GLOBALS['sys_group_id'], $vfl, $address
   );
-  # Send an email to notify the admins of the ite update.
-  list ($additional_address, $sendall) =
-    trackers_data_get_item_notification_info ($item_id, ARTIFACT, 1);
-  if ((trim ($address) != "") && $sendall)
-    $address .= ", ";
-  $address .= $additional_address;
-  # Exclude the submitter who has already been sent a notification.
+  $address .= trackers_append_followup_notif_addresses ($address, $item_id);
   trackers_mail_followup ($item_id, $address, false, $user_id);
 }
 
