@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('include/init.php');
 require_once ('include/sendmail.php');
@@ -58,13 +58,11 @@ form_check ('send_mail');
 if (!user_isloggedin ())
   exit_not_logged_in ();
 
-if (!$touser)
-  exit_missing_param (['touser']);
+exit_if_missing ('touser');
 
-$result = db_execute ("
-  SELECT email, user_name FROM user
-  WHERE user_id = ? AND (status = 'A' OR status = 'SQD')",
-  [$touser]
+$result = db_execute (
+  "SELECT email, user_name FROM user WHERE user_id = ? AND status IN (?, ?)",
+  [$touser, USER_STATUS_ACTIVE, USER_STATUS_SQUAD]
 );
 
 if (db_numrows ($result) < 1)
@@ -79,13 +77,7 @@ if (!$send_mail)
     exit;
   }
 
-$missing_params = [];
-foreach (['subject', 'body', 'fromuser'] as $p)
-  if (empty ($$p))
-    $missing_params[] = $p;
-if (!empty ($missing_params))
-  exit_missing_param ($missing_params);
-
+exit_if_missing (['subject', 'body', 'fromuser']);
 if ($cc_me)
   $touser .= ", $fromuser";
 
