@@ -27,7 +27,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 require_once ('../include/vars.php');
@@ -127,7 +127,7 @@ $name_matching = function ($trackers, $vcs_list)
   $names = [
     'specialchars' => [
       'name', 'description',  'base_host', 'homepage_scm',
-      'admin_email_adress', # Sic! adress not address, single 'd'.
+      'admin_email_ad' . 'ress', # Sic! not address, single 'd'.
     ],
     'true' => []
   ];
@@ -222,7 +222,9 @@ if (empty ($type_id))
     ['title' => no_i18n ('Group Type Management'), 'context' => 'admgrptype']
   );
 
-  $result = db_query ("SELECT type_id, name FROM group_type ORDER BY type_id");
+  $result = db_execute (
+    "SELECT type_id, name FROM group_type ORDER BY type_id"
+  );
   print "<br />\n";
   while ($usr = db_fetch_array ($result))
     {
@@ -260,11 +262,10 @@ site_admin_header (
 );
 
 
-print "<h1>{$row_grp['name']} (#{$row_grp['type_id']})</h1>\n";
+print html_h (1, "{$row_grp['name']} (#{$row_grp['type_id']})");
 print form_tag () . form_hidden (['type_id' => $type_id]);
 
-print '<h2>' . no_i18n ("General default settings for groups of this type")
-  . "</h2>\n\n";
+print html_h (2, no_i18n ("General default settings for groups of this type"));
 $textfield_size = '65';
 print '<p>'
   . no_i18n (
@@ -285,7 +286,7 @@ print '<p>'
       . 'from the whole installation settings.')
   . "</p>\n";
 
-print "<h2>" . no_i18n ("General Settings") . "</h2>\n\n";
+print html_h (2, no_i18n ("General Settings"));
 
 $idx = 'name';
 specific_input (
@@ -316,7 +317,7 @@ print "</p>\n";
 # For instance, there will be no viewcvs possibility for Arch managed
 # repository. But this is non-blocker so we let as it is.
 
-print "<h2>" . no_i18n ("Group WWW homepage") . "</h2>\n\n";
+print html_h (2, no_i18n ("Group WWW homepage"));
 print '<p>'
   . no_i18n (
       "This is useful if you provide directly web homepages (created by\n"
@@ -366,7 +367,7 @@ function source_code_manager ($HTML, $row_grp, $textfield_size,
   $vcs_name, $vcs_offix)
 {
   # TRANSLATORS: the argument is VCS name (like Subversion).
-  printf ("<h2>" . no_i18n ("Source Code Manager: %s") . "</h2>", $vcs_name);
+  print html_h (2, sprintf (no_i18n ("Source Code Manager: %s"), $vcs_name));
   print "\n\n<p>";
   printf (
     no_i18n ("This is useful if you provide directly %s repositories\n"
@@ -399,7 +400,7 @@ function source_code_manager ($HTML, $row_grp, $textfield_size,
 foreach ($vcs_list as $title => $name)
   source_code_manager ($HTML, $row_grp, $textfield_size, $title, $name);
 
-print "<h2>" . no_i18n ("Download Area") . "</h2>\n\n";
+print html_h (2, no_i18n ("Download Area"));
 print '<p>'
   . no_i18n (
       "This is useful if you provide directly download areas\n"
@@ -421,7 +422,7 @@ specific_input (
   no_i18n ("Repository URL:"), $row_grp[$idx], $idx, $textfield_size
 );
 
-print "<h2>" . no_i18n ("Licenses") . "</h2>\n\n";
+print html_h (2, no_i18n ("Licenses"));
 print '<p>'
   . no_i18n (
       "This is useful if you want group to select a license on\n"
@@ -430,7 +431,7 @@ print '<p>'
   . "</p>\n";
 show_checkbox (no_i18n ("Can use licenses"), 'can_use_license', $row_grp);
 
-print "<h2>" . no_i18n ("Development Status") . "</h2>\n\n";
+print html_h (2, no_i18n ("Development Status"));
 print '<p>'
   . no_i18n (
       "This is useful if you want group to be able to defines\n"
@@ -445,15 +446,15 @@ show_checkbox (
   no_i18n ("Can use development status"), "can_use_devel_status", $row_grp
 );
 
-print "<h2>" . no_i18n ("Mailing List") . "</h2>\n\n";
+print html_h (2, no_i18n ("Mailing List"));
 print '<p class="warn">'
   . no_i18n (
-      "Important: Everytime a mailing list name should appear,\n"
+      "Important: Every time a mailing list name should appear,\n"
       . "use the special string %LIST."
     )
   . "</p>\n<p>"
   . no_i18n (
-      "Do not configure Mailing list Host, this is a deprecated\n"
+      "Do not configure Mailing list host, this is a deprecated\n"
       . "feature left for backward compatibility."
     )
   . "</p>\n<p>"
@@ -469,7 +470,7 @@ show_checkbox (
 );
 
 $idx = 'mailing_list_host';
-specific_input (no_i18n ("Mailing list Host (DEPRECATED):"),
+specific_input (no_i18n ("Mailing list host (DEPRECATED):"),
   $row_grp[$idx], $idx, $textfield_size
 );
 $idx = 'mailing_list_address';
@@ -564,7 +565,7 @@ function artifact_section ($title, $description, $label, $artifact)
 {
   global $row_grp, $HTML;
 
-  print "<h2>$title</h2>\n\n";
+  print html_h (2, $title);
   if ($description != '')
     print "<p>$description</p>\n";
   if ($artifact != 'cookbook')
@@ -592,7 +593,7 @@ artifact_section (no_i18n ("Bug Tracker"),
   no_i18n (
     "This is one of the main issue tracker of Savane. Unlike the\n"
     . "support tracker, it is supposed to be used mainly to organize\n"
-    . "the workflow amongs group members related to bugs.  Groups\n"
+    . "the workflow among group members related to bugs.  Groups\n"
     . "with large audience should probably not accept item posting\n"
     . "by people that are not member of the group on this tracker,\n"
     . "and instead redirect end user to the support tracker (and only\n"
@@ -606,7 +607,7 @@ artifact_section (no_i18n ("Task Manager"),
   no_i18n (
     "This is one of the main issue tracker of Savane.  Unlike the\n"
     . "support tracker, it is supposed to be used mainly to organize\n"
-    . "the workflow amongs group members related to planned tasks.  It's\n"
+    . "the workflow among group members related to planned tasks.  It's\n"
     . "the counterpart of the bug tracker for regular and planned activities."
   ),
   no_i18n ("Can use task manager"), 'task'
@@ -633,7 +634,7 @@ $update_delete_buttons = "\n<p align='center'>\n"
   . "\n</p>\n";
 print $update_delete_buttons;
 
-print "<h2>" . no_i18n ("Group Menu Settings") . "</h2>\n\n";
+print html_h (2, no_i18n ("Group Menu Settings"));
 
 $i = 1;
 print '<p class="' . utils_altrow ($i) . '">'
