@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 require_once ('../include/init.php');
 
@@ -50,7 +50,7 @@ site_admin_header (
   ['title' => no_i18n ("Check Last Logins"), 'context' => 'admhome']
 );
 
-$res_logins = db_query ("
+$res_logins = db_execute ("
   SELECT s.user_id, s.ip_addr, s.time, u.user_name
   FROM session s, user u
   WHERE s.user_id = u.user_id AND s.user_id > 0 AND s.time > 0
