@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,7 +41,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # We don't internationalize messages in this file because they are
 # for Savannah admins who use English.
@@ -58,16 +58,13 @@ form_check ('update');
 
 $error = '';
 
-# Check for valid register from form post.
-function register_valid ()
+# Return 1 if inputs are valid, else set $error and return 0.
+function validate_inputs ()
 {
-  global $update, $user_id, $error, $form_pw, $form_pw2;
+  global $update, $error, $form_pw, $form_pw2;
 
   if (!$update)
     return 0;
-  # Check against old password.
-  db_execute ("SELECT user_pw FROM user WHERE user_id = ?", [$user_id]);
-
   if (!$form_pw)
     {
       $error = no_i18n ('no password provided');
@@ -83,28 +80,24 @@ function register_valid ()
       $error = no_i18n ('provided password is considered weak');
       return 0;
     }
-
-  # If we got this far, it must be good.
-  db_autoexecute ('user', ['user_pw' => account_encryptpw ($form_pw)],
-    DB_AUTOQUERY_UPDATE, "user_id = ?", [$user_id]
-  );
   return 1;
 }
 
-$title = sprintf (no_i18n ('Change Password for %s'), user_getname ($user_id));
+$title = sprintf (no_i18n ('Change password for %s'), user_getname ($user_id));
 
 $HTML->header (['title' => $title]);
+print html_h (2, no_i18n ('Savannah Password Change'));
 # Check for valid login, if so, congratulate.
-if (register_valid ())
+if (validate_inputs ())
   {
-    print "\n<p><strong>"
-      . no_i18n ('Savannah Change Confirmation') . "</strong></p>\n<p>"
-      . no_i18n ("Congratulations. You have managed to change this user's"
+    # If we got this far, it must be good.
+    account_set_pw ($user_id, $form_pw);
+    print "<p>"
+      . no_i18n ("Congratulations. You have managed to change this user's\n"
           . "password.")
       . "</p>\n<p>";
     printf (
-      no_i18n ('You should now <a href="%s">Return to UserList</a>.'),
-      '/admin/userlist.php'
+      no_i18n ('<a href="%s">Return to user list</a>.'), '/admin/userlist.php'
     );
     print "</p>\n";
   }
@@ -114,8 +107,7 @@ else
       $error = '<p><b>' . no_i18n ('Password change failed')
         . ": $error</b></p>\n";
 
-    print html_h (2, no_i18n ('Savannah Password Change'))
-      . "\n$error" . form_header () . '<p>'
+    print "$error" . form_header () . '<p>'
       . html_label ('form_pw', no_i18n ('New Password:'))
       . "<br />\n" . form_input ('password', 'form_pw') . "\n</p>\n<p>"
       . html_label ('form_pw2', no_i18n ('New Password (repeat):'))
