@@ -28,7 +28,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -41,18 +41,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # We don't internationalize messages in this file because they are
 # for Savannah admins who use English.
 
-require_once ('../include/init.php');
-require_once ('../include/form.php');
-require_once ('../include/account.php');
-require_once ('../include/markup.php');
-require_once ('../include/trackers/data.php');
+$incs =
+  ['init', 'form', 'account', 'trackers/general', 'markup', 'trackers/data'];
+foreach ($incs as $i)
+  require_once ("../include/$i.php");
 
-session_require (['group' => '1','admin_flags' => 'A']);
+session_require (['group' => '1','admin_flags' => MEMBER_FLAGS_ADMIN]);
 
 $actions = ['remove_user_from_group', 'update_user_group',
   'update_user', 'add_user_to_group', 'rename', 'delete', 'activate'
@@ -170,20 +169,20 @@ function entry_text ($entry)
 function output_contributions ($result, $offset, $max_rows)
 {
   $i = 0;
+  $defs = [];
   while ($entry = db_fetch_array ($result))
     {
       if (++$i > $max_rows)
-        return;
+        return $defs;
       $spam = no_i18n ('Spam score') . ' ' . $entry['spamscore'] . '; ';
       $date = utils_format_date ($entry['date'], 'natural');
       $line = "$spam$date {$entry['summary']}";
       if ($entry['spamscore'] > 4)
         $line = "<b>$line</b>";
       $entry_num = $i + $offset;
-      print "  <dt><b>$entry_num</b>: $line</dt>\n";
-      $text = entry_text ($entry);
-      print "<dd>$text</dd>\n";
+      $defs["<b>$entry_num</b>: $line"] = entry_text ($entry);
     }
+  return $defs;
 }
 
 function list_user_contributions ($user_id, $user_name, $offset, $max_rows)
@@ -197,9 +196,9 @@ function list_user_contributions ($user_id, $user_name, $offset, $max_rows)
       return;
     }
   contribution_nextprev ($user_id, $max_rows, $result);
-  print "<dl id=\"comment_results\">\n";
-  output_contributions ($result, $offset, $max_rows);
-  print "</dl>\n";
+  print html_dl (
+    output_contributions ($result, $offset, $max_rows), 'comment_results'
+  );
   contribution_nextprev ($user_id, $max_rows, $result);
 }
 
@@ -214,7 +213,7 @@ function report_db_result ($result, $msg_err, $msg_ok)
 function action_activate ()
 {
   global $user_id;
-  db_execute("UPDATE user SET status='A' WHERE user_id = ?", [$user_id]);
+  db_execute ("UPDATE user SET status='A' WHERE user_id = ?", [$user_id]);
 }
 
 function action_delete ()
@@ -329,13 +328,16 @@ function delete_account_link ($user_id)
 
 function show_status ($user_id, $status)
 {
-  $labels = ['A' => no_i18n ('Active'), 'P' => no_i18n ('Pending')];
+  $labels = [
+    USER_STATUS_ACTIVE => no_i18n ('Active'),
+    USER_STATUS_PENDING => no_i18n ('Pending')
+  ];
   $ret = "<p>Status: ";
   if (array_key_exists ($status, $labels))
     $ret .= $labels[$status];
   else
     $ret .= "[$status]";
-  if ($status == 'P')
+  if ($status == USER_STATUS_PENDING)
     $ret .= ' ' . form_tag ()
       . form_hidden (['action' => 'activate', 'user_id' => $user_id])
       . form_submit (no_i18n ('Activate')) . "\n</form>\n";
@@ -351,9 +353,12 @@ function account_title ($user_id)
 
 function account_form ($user_id, $row_user)
 {
-  $ret = account_title ($user_id);
-   if ($row_user['status'] == 'SQD')
-     return $ret . '<p>' . no_i18n ('This is a squad.') . "</p>\n";
+  $ret = account_title ($user_id)
+    . '<p>'
+    . utils_user_link ($row_user['user_name'], no_i18n ('Public profile of'))
+    . "</p>\n";
+  if ($row_user['status'] == USER_STATUS_SQUAD)
+    return $ret . '<p>' . no_i18n ('This is a squad.') . "</p>\n";
   $ret .= rename_form ($user_id, $row_user['user_name']);
   $ret .= show_status ($user_id, $row_user['status']);
   $ret .= email_form ($user_id, $row_user['email']);
@@ -365,29 +370,29 @@ function account_form ($user_id, $row_user)
 
 function user_group_form ($user_id, $grp_id, $admin_flags)
 {
-  $ret = form_tag ()
+  return form_tag ()
     . form_hidden (
         [ 'action' => 'update_user_group', 'user_id' => $user_id,
           'group_id' => $grp_id
         ]
       )
-    . "<br />\n<label for='admin_flags'>" . no_i18n ('Admin Flags:')
-    . "</label>\n&nbsp;\n"
+    . "<br />\n" . html_label ('admin_flags', no_i18n ('Admin flags:'))
+    . "\n&nbsp;\n"
     . '<input type="text" name="admin_flags" id="admin_flags" '
     . "value=\"$admin_flags\">\n&nbsp;\n"
-    . '<input type="submit" name="update_group" value="'
-    . no_i18n('Update') . "\" />\n";
-  $ret .= "&nbsp;<a href=\"usergroup.php?user_id=$user_id"
-    . "&amp;action=remove_user_from_group&amp;group_id=$grp_id\">"
-    . "\n[" . no_i18n ('Remove user from group') . "]</a>\n";
-  return "$ret</form>\n";
+    . form_submit (no_i18n ('Update')) . "</form>\n"
+    . form_tag ()
+    . form_hidden (['user_id' => $user_id, 'group_id' => $grp_id,
+        'action' => 'remove_user_from_group'])
+    . "<br />\n" . form_submit (no_i18n ('Remove user from group'))
+    . "</form>\n";
 }
 
 function group_entry ($user_id, $status, $row_cat)
 {
   $grp_id = $row_cat['group_id'];
   $grp_name = group_getname ($grp_id);
-  if ($status == 'SQD')
+  if ($status == USER_STATUS_SQUAD)
     return "<p>\n<a href=\"/project/admin/squadadmin.php?"
       . "squad_id=$user_id&amp;group_id=$grp_id\">$grp_name</a></p>\n";
   $ret = html_h (3, $grp_name);
@@ -436,7 +441,7 @@ $row_user = db_fetch_array ($res_user);
 print account_form ($user_id, $row_user);
 list_groups ($user_id, $row_user['status']);
 
-if ($row_user['status'] != 'SQD')
+if ($row_user['status'] != USER_STATUS_SQUAD)
   list_user_contributions (
     $user_id, $row_user['user_name'], $offset, $max_rows
   );
