@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,7 +39,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 # We don't internationalize messages in this file because they are
 # for Savannah admins who use English.
@@ -181,11 +181,11 @@ for ($i = 0; $i < $rows; $i++)
 
     switch ($stat)
       {
-      case 'A': print no_i18n ("Active"); break;
-      case 'D': # Fall through.
-      case 'S': print no_i18n ("Deleted"); break;
-      case 'SQD': print no_i18n ("Active (Squad)"); break;
-      case 'P': print no_i18n ("Pending"); break;
+      case USER_STATUS_ACTIVE: print no_i18n ("Active"); break;
+      case USER_STATUS_DELETED: # Fall through.
+      case USER_STATUS_SUSPENDED: print no_i18n ("Deleted"); break;
+      case USER_STATUS_SQUAD: print no_i18n ("Active (Squad)"); break;
+      case USER_STATUS_PENDING: print no_i18n ("Pending"); break;
       default: print no_i18n ("Unknown status") . ": $stat"; break;
       }
     if ($usr['people_view_skills'] == 1)
