@@ -26,7 +26,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -39,13 +39,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Invocation:
 #
 #   php testing/sane.php
 #
-# In case of fail, diagnositc text is output to stdout.
+# In case of fail, diagnostic text is output to stdout.
 
 require_once('include/sane.php');
 
@@ -75,7 +75,7 @@ function test_sane_import ($in, $names, $out)
   global $sane_test_input;
 
   $sane_test_input = $in;
-  $res = sane_import('test', $names);
+  $res = sane_import ($sane_test_input, $names);
   if ($res == $out)
     return;
   print_reference ();
@@ -133,7 +133,7 @@ $reference = 'account/impersonate.php';
 
   test_sane_import ($in, $names, $out);
 
-  $in['session_hash'] = '9ad59d2d0703e7f015d54e725ce099fc1fd1433c';
+  $in['session_hash'] = '0289;9ad59d2d0703e7f015d54e725ce099fc1fd1433c';
   $out['session_hash'] = $in['session_hash'];
 
   test_sane_import ($in, $names, $out);
@@ -142,7 +142,8 @@ $reference = 'account/impersonate.php';
 $reference = 'account/login.php';
 {
   $names = [
-    'true' => ['brotherhood', 'cookie_for_a_year', 'login', 'cookie_test'],
+    'true' => ['brotherhood', 'login'],
+    'digits' => 'cookie_for_a_year',
     'name' => 'form_loginname',
     'pass' => 'form_pw',
     'internal_uri' => 'uri'
@@ -157,9 +158,20 @@ $reference = 'account/login.php';
   $out = $in;
   $out['login'] = true;
   $out['cookie_for_a_year'] = null;
-  $out['cookie_test'] = null;
 
   test_sane_import ($in, $names, $out);
+  $in['cookie_for_a_year'] = $out['cookie_for_a_year'] = 0;
+  $in['brotherhood'] = false;
+  test_sane_import ($in, $names, $out);
+  $in['cookie_for_a_year'] = $out['cookie_for_a_year'] = 1;
+  test_sane_import ($in, $names, $out);
+  $names = ['digits' => 'session_uid', 'hash' => 'session_hash'];
+  $in = [
+    'session_uid' => 4913,
+    'session_hash' => '0289;9ad59d2d0703e7f015d54e725ce099fc1fd1433c'
+  ];
+  $out = $in;
+  test_sane_import ($in, $names, $out);
 }
 
 $reference = 'account/lostlogin.php';
@@ -212,7 +224,7 @@ $reference = 'account/register.php';
     'form_pw2' => '^',
     'form_year' => '1983',
     'form_realname' => 'A. B. C.',
-    'form_email' => 'agn@test.org'
+    'form_email' => 'agn@example.org'
   ];
   $out = $in;
   $out['update'] = true;
@@ -379,12 +391,15 @@ $reference = 'cvs/admin/index.php';
   test_sane_import ($in, $names, $out);
 }
 
-$reference = 'file';
+$reference = 'file.php';
 {
-  $names = ['preg' => [['file_id', '/^(\d+|test[.]png)$/']]];
+  $names = [
+    'preg' => [['file_id', '/^(\d+|test[.]png)$/']], 'digits' => 'file_uid'
+  ];
   $in = $out = ['file_id' => 'test.png'];
+  $out['file_uid'] = null;
   test_sane_import ($in, $names, $out);
-  $in = $out = ['file_id' => '12345'];
+  $in = $out = ['file_id' => '12345', 'file_uid' => 543];
   test_sane_import ($in, $names, $out);
   $in['file_id'] = '124a';
   $out['file_id'] = null;
@@ -504,7 +519,7 @@ $reference = 'include/context.php';
   test_sane_import ($in, $names, $out);
 }
 
-$reference = 'include/form.php';
+$reference = 'include/form-check.php';
 {
   $names = ['pass' => 'website'];
   $in = ['website' => ''];
@@ -623,7 +638,7 @@ $reference = 'include/session.php';
   ];
 
   $in = [
-    'session_hash' => '0cc175b9c0f1b6a831c399e269772661',
+    'session_hash' => '4913;0cc175b9c0f1b6a831c399e269772661',
     'session_uid' => 83521
   ];
 
@@ -770,18 +785,7 @@ $reference = 'include/trackers/data.php';
 
 $reference = 'include/trackers/format.php';
 {
-  $names = [
-    'strings' => [
-      [
-        'func',
-        [
-         'flagspam', 'unflagspam', 'viewspam', 'delete_file',
-         'delete_cc'
-        ]
-      ]
-    ],
-    'digits' => 'comment_internal_id'
-  ];
+  $names = ['digits' => 'comment_internal_id', 'funcs' => 'func'];
   $in = [
     'func' => 'viewspam',
     'comment_internal_id' => 23
@@ -844,10 +848,9 @@ $reference = 'include/trackers_run/admin/editqueryforms.php';
     'post_changes', 'set_default', 'create_report', 'update_report'
   ];
   $names['specialchars'] = ['rep_name', 'rep_desc'];
-  $names['strings'] = [['rep_scope', 'P']];
 
-  $prefices = ['TFSRCH', 'TFREP', 'TFCW', 'CBSRCH', 'CBREP'];
-  $suffices = [
+  $prefixes = ['TFSRCH', 'TFREP', 'TFCW', 'CBSRCH', 'CBREP'];
+  $suffixes = [
     'bug_id', 'submitted_by', 'date', 'close_date', 'planned_starting_date',
     'planned_close_date', 'category_id', 'priority', 'resolution_id',
     'privacy', 'vote', 'percent_complete', 'assigned_to', 'status_id',
@@ -863,18 +866,17 @@ $reference = 'include/trackers_run/admin/editqueryforms.php';
 
   foreach ($custom_suff as $suf => $num)
     for ($i = 1; $i <= $num; $i++)
-      $suffices[] = 'custom_' . $suf . $i;
+      $suffixes[] = 'custom_' . $suf . $i;
 
   $names['digits'] = [];
-  foreach ($prefices as $pref)
-    foreach ($suffices as $suf)
+  foreach ($prefixes as $pref)
+    foreach ($suffixes as $suf)
       $names['digits'][] = $pref . '_' . $suf;
 
   $in = [
     'post_changes' => 0,
     'rep_name' => '"name<">',
     'rep_desc' => "'description'",
-    'rep_scope' => "W",
     'TFCW_date' => 40,
     'CBSRCH_custom_sb3' => 4,
     'CBREP_custom_tf8' => 3,
@@ -888,7 +890,6 @@ $reference = 'include/trackers_run/admin/editqueryforms.php';
   $out['set_default'] = null;
   $out['create_report'] = null;
   $out['update_report'] = null;
-  $out['rep_scope'] = null;
   $out['rep_name'] = htmlspec ($out['rep_name']);
   $out['rep_desc'] = htmlspec ($out['rep_desc']);
   unset ($out['CBREP_custom_df10']);
@@ -1009,7 +1010,7 @@ $reference = 'include/trackers_run/admin/field_values.php';
     'to' => 2,
     'allowed' => 'F',
     'status' => 'P',
-    'mail_list' => 'agn_ter, dis-cuss0@test.mil, A+B@C@.org,.,'
+    'mail_list' => 'agn_ter, dis-cuss0@example.net, A+B@C@.org,.,'
   ];
   $out = $in;
   $out['post_changes'] = true;
@@ -1287,7 +1288,7 @@ $reference = 'include/trackers_run/index.php';
     'new_vote' => 10,
     'quote_no' => 3,
     'cc_comment' => 'a',
-    'originator_email' => 'agn@test.mil',
+    'originator_email' => 'agn@example.org',
     'add_cc' => 'agn,a@b.ca;d@e.fgh',
     'reassign_change_group' => '0-grep_up',
     'depends_search_only_artifact' => 'bugs',
@@ -1524,7 +1525,7 @@ $reference = 'my/admin/change.php';
    'update' => true,
    'step' => 'discard',
    'test_gpg_key' => true,
-   'session_hash' => md5 (5),
+   'session_hash' => '4913;' . md5 (5),
    'confirm_hash' => md5 (6),
   ];
   test_sane_import ($in, $names, $out);
@@ -1533,13 +1534,13 @@ $reference = 'my/admin/change.php';
 $reference = 'my/admin/editsshkeys.php';
 {
   $names = [
-    'true' => 'update',
+    'true' => ['update', 'form_quiet_ssh'],
     'array' => [['form_keys', ['digits', 'no_quotes']]]
   ];
   $in = $out = [
     'form_keys' => ['a', 'b', 'c', 3 => '"']
   ];
-  $out['update'] = null;
+  $out['update'] = $out['form_quiet_ssh'] = null;
   unset ($out['form_keys'][3]);
   test_sane_import ($in, $names, $out);
 }
@@ -1604,18 +1605,15 @@ $reference = 'my/admin/sessions.php';
     'strings' => [['func', 'del']],
     'true' => 'dkeep_one',
     'digits' => 'dtime',
-    'preg' =>
-      [
-        ['dip_addr', ',^[[:xdigit:]./:]+$,'],
-        ['dsession_hash', '/^[a-f\d]+[.]{3}$/']
-      ],
+    'specialchars' => 'dsession_hash',
+    'preg' => [['dip_addr', ',^[[:xdigit:]./:]+$,']],
   ];
   $in = $out = [
     'dip_addr' => '127.0.0.1/24:5',
     'func' => 'del',
     'dkeep_one' => true,
     'dtime' => 1,
-    'dsession_hash' => substr (md5 (0), 0, 6) . "..."
+    'dsession_hash' => "..." . substr (md5 (0), -8)
   ];
   test_sane_import ($in, $names, $out);
   $in['dip_addr'] = $out['dip_addr'] = '2023:01:da8b:6fa1:b93c:f5b4:3593:26a2';
@@ -1746,7 +1744,7 @@ $reference = 'news/admin/index.php';
   ];
   $in = $out = [
     'update' => true,
-    'form_news_address' => 'news@test.org',
+    'form_news_address' => 'news@example.org',
   ];
   test_sane_import ($in, $names, $out);
 }
@@ -1951,7 +1949,7 @@ $reference = 'project/admin/editgroupnotifications.php';
   ];
   $in = $out = [
     'update' => true,
-    'form_news_address' => 'agn@test.org',
+    'form_news_address' => 'agn@example.org',
     'form_frequency' => 2,
   ];
   test_sane_import ($in, $names, $out);
@@ -2148,7 +2146,7 @@ $reference = 'siteadmin/group_type.php';
   $names = [
     'specialchars' => [
       'name', 'description',  'base_host', 'homepage_scm',
-      'admin_email_adress', # Sic! adress not address
+      'admin_email_ad' . 'ress', # Sic! not address
     ],
     'true' => []
   ];
@@ -2207,11 +2205,9 @@ $reference = 'siteadmin/index.php';
 $reference = 'siteadmin/spamlist.php';
 {
   $names = [
-    'digits' => ['ban_user_id', 'wash_user_id', 'max_rows', 'offset']
-  ];
-  $in = $out = [
-    'ban_user_id' => 1, 'wash_user_id' => 2, 'max_rows' => 3, 'offset' => 4
+    'digits' => ['wash_user_id', 'max_rows', 'offset']
   ];
+  $in = $out = ['wash_user_id' => 2, 'max_rows' => 3, 'offset' => 4];
   test_sane_import ($in, $names, $out);
 }
 
@@ -2250,7 +2246,7 @@ $reference = 'siteadmin/usergroup.php';
     ],
   ];
   $in = $out = [
-    'new_name' => 'new_agn', 'email' => 'test@test.org', 'admin_flags' => 'A'
+    'new_name' => 'new_agn', 'email' => 'tst@example.org', 'admin_flags' => 'A'
   ];
   test_sane_import ($in, $names, $out);
 }
