@@ -30,7 +30,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # Savane is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as
@@ -43,7 +43,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 include dirname (__FILE__) . '/../fingerprints.php';
 
@@ -65,7 +65,7 @@ if ($project->isPublic ())
 
     if ($project->Uses ("cvs"))
       {
-        $cvs_cmd = "$cvs_cmd_base " . $project->getTypeDir ('cvs') . " co ";
+        $cvs_cmd = $cvs_cmd_base . $project->getTypeDir ('cvs') . " co ";
         print "<h4>" . _('Software repository:') . "</h4>\n";
         print "<pre>$cvs_cmd$proj_unix_name</pre>\n";
         print "<p>" . _('With other project modules:') . "</p>\n";
@@ -146,7 +146,7 @@ print "</p>\n<p>";
 printf (
   _("The basic information described further on this page is detailed in\n"
     . "the <a href=\"%s\">Savannah user doc</a>."),
-  "{$sys_home}faq/?group=$sys_unix_group_name"
+  "//savannah.gnu.org/maintenance/back-page/#cvs"
 );
 
 print "</p>\n";
